@@ -11,7 +11,7 @@ doc = ezdxf.new('R2010')
 msp = doc.modelspace()
 
 # Using an UCS simplifies 3D operations, but UCS definition can happen later
-# calculating corner points in local (UCS) coordinates without Vector class
+# calculating corner points in local (UCS) coordinates without Vec3 class
 angle = math.radians(360 / 5)
 corners_ucs = [(math.cos(angle * n), math.sin(angle * n), 0) for n in range(5)]
 
@@ -32,10 +32,8 @@ corners_wcs = list(ucs.points_to_wcs(transformed_corners_ucs))
 
 msp.add_polyline3d(
     points=corners_wcs,
-    dxfattribs={
-        'closed': True,
-        'color': 1,
-    })
+    close=True,
+)
 
 # add lines from center to corners
 center_wcs = ucs.to_wcs((0, .333, .333))
