@@ -1,11 +1,12 @@
 # Copyright (c) 2020 Manfred Moitzi
 # License: MIT License
 from pathlib import Path
+
 OUT_DIR = Path('~/Desktop/Outbox').expanduser()
 
 import math
 import ezdxf
-from ezdxf.math import Vector, UCS
+from ezdxf.math import Vec3, UCS
 
 doc = ezdxf.new('R2010')
 msp = doc.modelspace()
@@ -16,10 +17,10 @@ ucs = UCS(origin=(0, 2, 2)).rotate_local_x(math.radians(-45))
 
 msp.add_lwpolyline(
     # calculating corner points in UCS coordinates
-    points=(Vector.from_deg_angle((360 / 5) * n) for n in range(5)),
+    points=(Vec3.from_deg_angle((360 / 5) * n) for n in range(5)),
     format='xy',  # ignore z-axis
+    close=True,
     dxfattribs={
-        'closed': True,
         'color': 1,
     }
 ).transform(ucs.matrix)
