@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) 2020-2022, Matthew Broadway
+# Copyright (c) 2020-2023, Matthew Broadway
 # License: MIT License
 import argparse
 import sys
@@ -12,8 +12,7 @@ from ezdxf.addons.drawing import RenderContext, Frontend
 from ezdxf.addons.drawing.matplotlib import MatplotlibBackend
 from ezdxf.addons.drawing.config import Configuration
 
-from ezdxf.tools import fonts
-
+# ------------------------------------------------------------------------------
 # The "draw_cad.py" viewer can be executed by the new ezdxf command line
 # launcher:
 #
@@ -21,8 +20,8 @@ from ezdxf.tools import fonts
 #
 # This file remains as an example for the usage of the Matplotlib backend.
 #
-# For the case automatic font loading is disabled:
-fonts.load()
+# docs: https://ezdxf.mozman.at/docs/addons/drawing.html
+# ------------------------------------------------------------------------------
 
 
 def _main():
@@ -79,7 +78,7 @@ def _main():
         sys.exit(4)
 
     # setup drawing add-on configuration
-    config = Configuration.defaults()
+    config = Configuration()
     fig: plt.Figure = plt.figure(dpi=args.dpi)
     ax: plt.Axes = fig.add_axes([0, 0, 1, 1])
     ctx = RenderContext(doc)
