@@ -725,7 +725,7 @@ class PlotStyleFileParser:
                 return get_mapping()
             else:  # it's a simple name=value line
                 value: str = line.split("=", 1)[1]
-                value = value.lstrip('"')  # strings look like this: name="value
+                value = sanitized_value(value)
                 line_index += 1
             return value
 
@@ -745,6 +745,18 @@ class PlotStyleFileParser:
         return self.data.get(name, default)
 
 
+def sanitized_value(value: str) -> str:
+    value = value.strip()
+    if value.startswith('"'):  # strings: <name>="string
+        return value[1:]
+
+    # remove unknown appendix like this: "0.0076200000000 (+7.Z+"8V?S_LC )"
+    # the pattern is "<float|int> (<some data>)", see issue #1069
+    if value.endswith(")"):
+        return value.split(" ")[0]
+    return value
+
+
 def int2color(color: int) -> tuple[int, int, int, int]:
     """Convert color integer value from CTB-file to ``(r, g, b, color_type)
     tuple.
