@@ -1,6 +1,7 @@
-#  Copyright (c) 2021, Manfred Moitzi
+#  Copyright (c) 2021-2022, Manfred Moitzi
 #  License: MIT License
-from typing import Optional, Set, List
+from __future__ import annotations
+from typing import Optional, Set
 from functools import partial
 from pathlib import Path
 import subprocess
@@ -24,7 +25,7 @@ import ezdxf
 from ezdxf.lldxf.const import DXFStructureError, DXFValueError
 from ezdxf.lldxf.types import DXFTag, is_pointer_code
 from ezdxf.lldxf.tags import Tags
-from ezdxf.pp.reflinks import get_reference_link
+from ezdxf.addons.browser.reflinks import get_reference_link
 
 from .model import (
     DXFStructureModel,
@@ -54,8 +55,8 @@ SearchSections = Set[str]
 
 def searchable_entities(
     doc: DXFDocument, search_sections: SearchSections
-) -> List[Tags]:
-    entities: List[Tags] = []
+) -> list[Tags]:
+    entities: list[Tags] = []
     for name, section_entities in doc.sections.items():
         if name in search_sections:
             entities.extend(section_entities)  # type: ignore
@@ -71,8 +72,8 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
     def __init__(
         self,
         filename: str = "",
-        line: int = None,
-        handle: str = None,
+        line: Optional[int] = None,
+        handle: Optional[str] = None,
         resource_path: Path = Path("."),
     ):
         super().__init__()
@@ -82,8 +83,8 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
         self._dxf_tags_table = DXFTagsTable()
         self._current_entity: Optional[Tags] = None
         self._active_search: Optional[SearchIndex] = None
-        self._search_sections: Set[str] = set()
-        self._find_dialog: "FindDialog" = self.create_find_dialog()
+        self._search_sections: set[str] = set()
+        self._find_dialog: FindDialog = self.create_find_dialog()
         self._file_watcher = QFileSystemWatcher()
         self._exclusive_reload_dialog = True  # see ask_for_reloading() method
         self.history = EntityHistory()
@@ -275,7 +276,7 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
             action.setIcon(icon)
         if tip:
             action.setToolTip(tip)
-        action.triggered.connect(slot)  # type: ignore
+        action.triggered.connect(slot)
         return action
 
     def setup_menu(self):
@@ -329,7 +330,7 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
         self.addToolBar(toolbar)
 
     def create_find_dialog(self) -> "FindDialog":
-        dialog = FindDialog(self)
+        dialog = FindDialog()
         dialog.setModal(True)
         dialog.find_forward_button.clicked.connect(self.find_forward)
         dialog.find_backwards_button.clicked.connect(self.find_backwards)
@@ -468,7 +469,9 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
         if entity:
             self.set_current_entity(entity)
 
-    def set_current_entity(self, entity: Tags, select_line_number: int = None):
+    def set_current_entity(
+        self, entity: Tags, select_line_number: Optional[int] = None
+    ):
         if entity:
             self._current_entity = entity
             start_line_number = self.doc.get_line_number(entity)
@@ -607,7 +610,7 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
     def show_entity_found_message(self, entity: Tags, index: int):
         dxftype = entity.dxftype()
         if dxftype == "SECTION":
-            tail = " @ {0} Section".format(entity.get_first_value(2))  # type: ignore
+            tail = " @ {0} Section".format(entity.get_first_value(2))
         else:
             try:
                 handle = entity.get_handle()
@@ -744,12 +747,16 @@ class DXFStructureBrowser(QtWidgets.QMainWindow):
 
 def copy_dxf_to_clipboard(tags: Tags):
     clipboard = QtWidgets.QApplication.clipboard()
-    clipboard.setText(dxfstr(tags), mode=clipboard.Clipboard)
+    try:
+        mode = clipboard.Mode.Clipboard
+    except AttributeError:
+        mode = clipboard.Clipboard  # type: ignore  # legacy location
+    clipboard.setText(dxfstr(tags), mode=mode)
 
 
 class FindDialog(QtWidgets.QDialog, Ui_FindDialog):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self):
+        super().__init__()
         self.setupUi(self)
         self.close_button.clicked.connect(lambda: self.close())
         self.settings = QSettings("ezdxf", "DXFBrowser")
