@@ -1,6 +1,8 @@
-#  Copyright (c) 2021, Manfred Moitzi
-#  License: MIT License
-from typing import Any, List, Dict, Optional
+# Copyright (c) 2021, Manfred Moitzi
+# License: MIT License
+# mypy: ignore_errors=True
+from __future__ import annotations
+from typing import Any, Optional
 import textwrap
 from ezdxf.lldxf.types import (
     render_tag,
@@ -8,7 +10,7 @@ from ezdxf.lldxf.types import (
     GROUP_MARKERS,
     POINTER_CODES,
 )
-from ezdxf.addons.xqt import QModelIndex, QAbstractTableModel, Qt
+from ezdxf.addons.xqt import QModelIndex, QAbstractTableModel, Qt, QtWidgets
 from ezdxf.addons.xqt import QStandardItemModel, QStandardItem, QColor
 from .tags import compile_tags, Tags
 
@@ -33,7 +35,7 @@ def name_fmt(handle, name: str) -> str:
 HEADER_LABELS = ["Group Code", "Data Type", "Content", "4", "5"]
 
 
-def calc_line_numbers(start: int, tags: Tags) -> List[int]:
+def calc_line_numbers(start: int, tags: Tags) -> list[int]:
     numbers = [start]
     index = start
     for tag in tags:
@@ -53,6 +55,8 @@ class DXFTagsModel(QAbstractTableModel):
         self._tags = compile_tags(tags)
         self._line_numbers = calc_line_numbers(start_line_number, self._tags)
         self._valid_handles = valid_handles or set()
+        palette = QtWidgets.QApplication.palette()
+        self._group_marker_color = palette.highlight().color()
 
     def data(self, index: QModelIndex, role: int = ...) -> Any:  # type: ignore
         def is_invalid_handle(tag):
@@ -69,7 +73,7 @@ class DXFTagsModel(QAbstractTableModel):
         elif role == Qt.ForegroundRole:
             tag = self._tags[index.row()]
             if tag.code in GROUP_MARKERS:
-                return QColor("blue")
+                return self._group_marker_color
             elif is_invalid_handle(tag):
                 return QColor("red")
         elif role == DXFTagsRole:
@@ -123,7 +127,7 @@ class DXFTagsModel(QAbstractTableModel):
 
 
 class EntityContainer(QStandardItem):
-    def __init__(self, name: str, entities: List[Tags]):
+    def __init__(self, name: str, entities: list[Tags]):
         super().__init__()
         self.setEditable(False)
         self.setText(name + f" ({len(entities)})")
@@ -189,7 +193,7 @@ class Blocks(EntityContainer):
                 container.clear()
 
 
-def get_section_name(section: List[Tags]) -> str:
+def get_section_name(section: list[Tags]) -> str:
     if len(section) > 0:
         header = section[0]
         if len(header) > 1 and header[0].code == 0 and header[1].code == 2:
@@ -551,7 +555,7 @@ GROUP_CODE_TOOLTIPS = [
 ]
 
 
-def build_group_code_tooltip_dict() -> Dict[int, str]:
+def build_group_code_tooltip_dict() -> dict[int, str]:
     tooltips = dict()
     for code, tooltip in GROUP_CODE_TOOLTIPS:
         tooltip = "\n".join(textwrap.wrap(tooltip, width=80))
