@@ -1,7 +1,6 @@
-# Copyright (c) 2020-2023, Matthew Broadway
+# Copyright (c) 2020-2024, Matthew Broadway
 # License: MIT License
 from __future__ import annotations
-import math
 from typing import (
     Iterable,
     cast,
@@ -12,22 +11,35 @@ from typing import (
     TYPE_CHECKING,
 )
 from typing_extensions import TypeAlias
+import contextlib
+import math
+import os
+import pathlib
 import logging
 import time
 
+import PIL.Image
+import PIL.ImageEnhance
+import PIL.ImageDraw
+import numpy as np
+
+
 import ezdxf.bbox
 from ezdxf.addons.drawing.config import (
     Configuration,
     ProxyGraphicPolicy,
     HatchPolicy,
+    ImagePolicy,
 )
-from ezdxf.addons.drawing.backend import BackendInterface
+from ezdxf.addons.drawing.backend import BackendInterface, ImageData
 from ezdxf.addons.drawing.properties import (
     RenderContext,
     OLE2FRAME_COLOR,
     Properties,
     Filling,
     LayoutProperties,
+    MODEL_SPACE_BG_COLOR,
+    PAPER_SPACE_BG_COLOR,
 )
 from ezdxf.addons.drawing.config import BackgroundPolicy, TextPolicy
 from ezdxf.addons.drawing.text import simplified_text_chunks
@@ -49,12 +61,14 @@ from ezdxf.entities import (
     OLE2Frame,
     Point,
     Viewport,
+    Image,
 )
+from ezdxf.tools import clipping_portal
 from ezdxf.entities.attrib import BaseAttrib
 from ezdxf.entities.polygon import DXFPolygon
 from ezdxf.entities.boundary_paths import AbstractBoundaryPath
 from ezdxf.layouts import Layout
-from ezdxf.math import Vec2, Vec3, OCS, NULLVEC
+from ezdxf.math import Vec2, Vec3, OCS, NULLVEC, Matrix44
 from ezdxf.path import (
     Path,
     make_path,
@@ -66,18 +80,24 @@ from ezdxf import reorder
 from ezdxf.proxygraphic import ProxyGraphic, ProxyGraphicError
 from ezdxf.protocols import SupportsVirtualEntities, virtual_entities
 from ezdxf.tools.text import has_inline_formatting_codes
+from ezdxf.tools import text_layout
 from ezdxf.lldxf import const
 from ezdxf.render import hatching
 from ezdxf.fonts import fonts
+from ezdxf.colors import RGB, RGBA
+from ezdxf.npshapes import NumpyPoints2d
+from ezdxf import xclip
+
 from .type_hints import Color
 
 if TYPE_CHECKING:
-    from .designer import Designer
+    from .pipeline import AbstractPipeline
 
 __all__ = ["Frontend", "UniversalFrontend"]
 
+TEntityFunc: TypeAlias = Callable[[DXFGraphic, Properties], None]
+TDispatchTable: TypeAlias = Dict[str, TEntityFunc]
 
-TDispatchTable: TypeAlias = Dict[str, Callable[[DXFGraphic, Properties], None]]
 POST_ISSUE_MSG = (
     "Please post sample DXF file at https://github.com/mozman/ezdxf/issues."
 )
@@ -106,18 +126,22 @@ class UniversalFrontend:
     def __init__(
         self,
         ctx: RenderContext,
-        designer: Designer,
+        pipeline: AbstractPipeline,
         config: Configuration = Configuration(),
         bbox_cache: Optional[ezdxf.bbox.Cache] = None,
     ):
         # RenderContext contains all information to resolve resources for a
         # specific DXF document.
         self.ctx = ctx
-        # the designer is the connection between frontend and backend
-        self.designer = designer
-        designer.set_draw_entities_callback(self.draw_entities_callback)
+
+        # the render pipeline is the connection between frontend and backend
+        self.pipeline = pipeline
+        pipeline.set_draw_entities_callback(self.draw_entities_callback)
+
+        # update all configs:
         self.config = ctx.update_configuration(config)
-        designer.set_config(self.config)
+        # backend.configure() is called in pipeline.set_config()
+        pipeline.set_config(self.config)
 
         if self.config.pdsize is None or self.config.pdsize <= 0:
             self.log_message("relative point size is not supported")
@@ -145,9 +169,13 @@ class UniversalFrontend:
         # time.
         self._bbox_cache = bbox_cache
 
+        # Entity property override function stack:
+        self._property_override_functions: list[TEntityFunc] = []
+        self.push_property_override_function(self.override_properties)
+
     @property
     def text_engine(self):
-        return self.designer.text_engine
+        return self.pipeline.text_engine
 
     def _build_dispatch_table(self) -> TDispatchTable:
         dispatch_table: TDispatchTable = {
@@ -158,6 +186,7 @@ class UniversalFrontend:
             "WIPEOUT": self.draw_wipeout_entity,
             "MTEXT": self.draw_mtext_entity,
             "OLE2FRAME": self.draw_ole2frame_entity,
+            "IMAGE": self.draw_image_entity,
         }
         for dxftype in ("LINE", "XLINE", "RAY"):
             dispatch_table[dxftype] = self.draw_line_entity
@@ -181,16 +210,56 @@ class UniversalFrontend:
         """Called for skipped entities - override to alter behavior."""
         self.log_message(f'skipped entity {str(entity)}. Reason: "{msg}"')
 
+    def exec_property_override(
+        self, entity: DXFGraphic, properties: Properties
+    ) -> None:
+        """Execute entity property override functions. (internal API)"""
+        for override_fn in self._property_override_functions:
+            override_fn(entity, properties)
+
     def override_properties(self, entity: DXFGraphic, properties: Properties) -> None:
-        """The :meth:`override_properties` filter can change the properties of
-        an entity independent of the DXF attributes.
+        """This method can change the resolved properties of an DXF entity.
+
+        The method has access to the DXF entity attributes, the current render context
+        and the resolved properties.  It is recommended to modify only the resolved
+        `properties` in this method, because the DXF entities are not copies - except
+        for virtual entities.
+
+        .. versionchanged:: 1.3.0
+
+            This method is the first function in the stack of new property override
+            functions.  It is possible to push additional override functions onto this
+            stack, see also :meth:`push_property_override_function`.
 
-        This filter has access to the DXF attributes by the `entity` object,
-        the current render context, and the resolved properties by the
-        `properties` object. It is recommended to modify only the `properties`
-        object in this filter.
         """
 
+    def push_property_override_function(self, override_fn: TEntityFunc) -> None:
+        """The override function can change the resolved properties of an DXF entity.
+
+        The override function has access to the DXF entity attributes and the resolved
+        properties.  It is recommended to modify only the resolved `properties` in this
+        function, because the DXF entities are not copies - except for virtual entities.
+
+        The override functions are called after resolving the DXF attributes of an entity
+        and before the :meth:`Frontend.draw_entity` method in the order from first to
+        last.
+
+        .. versionadded:: 1.3.0
+
+        """
+        self._property_override_functions.append(override_fn)
+
+    def pop_property_override_function(self) -> None:
+        """Remove the last function from the property override stack.
+
+        Does not raise an exception if the override stack is empty.
+
+        .. versionadded:: 1.3.0
+
+        """
+        if self._property_override_functions:
+            self._property_override_functions.pop()
+
     def draw_layout(
         self,
         layout: Layout,
@@ -219,7 +288,6 @@ class UniversalFrontend:
 
         """
         if layout_properties is not None:
-            # TODO: this does not work, layer properties have to be re-evaluated!
             self.ctx.current_layout_properties = layout_properties
         else:
             self.ctx.set_current_layout(layout)
@@ -238,7 +306,7 @@ class UniversalFrontend:
                 filter_func=filter_func,
             )
         if finalize:
-            self.designer.finalize()
+            self.pipeline.finalize()
 
     def set_background(self, color: Color) -> None:
         policy = self.config.background_policy
@@ -247,15 +315,19 @@ class UniversalFrontend:
             override = False
         elif policy == BackgroundPolicy.OFF:
             color = "#ffffff00"  # white, fully transparent
-        elif policy == BackgroundPolicy.BLACK:
-            color = "#000000"
         elif policy == BackgroundPolicy.WHITE:
             color = "#ffffff"
+        elif policy == BackgroundPolicy.BLACK:
+            color = "#000000"
+        elif policy == BackgroundPolicy.PAPERSPACE:
+            color = PAPER_SPACE_BG_COLOR
+        elif policy == BackgroundPolicy.MODELSPACE:
+            color = MODEL_SPACE_BG_COLOR
         elif policy == BackgroundPolicy.CUSTOM:
             color = self.config.custom_bg_color
         if override:
             self.ctx.current_layout_properties.set_colors(color)
-        self.designer.set_background(color)
+        self.pipeline.set_background(color)
 
     def draw_entities(
         self,
@@ -282,10 +354,10 @@ class UniversalFrontend:
             properties: resolved entity properties
 
         """
-        self.designer.enter_entity(entity, properties)
+        self.pipeline.enter_entity(entity, properties)
         if not entity.is_virtual:
             # top level entity
-            self.designer.set_current_entity_handle(entity.dxf.handle)
+            self.pipeline.set_current_entity_handle(entity.dxf.handle)
         if (
             entity.proxy_graphic
             and self.config.proxy_graphic_policy == ProxyGraphicPolicy.PREFER
@@ -316,22 +388,22 @@ class UniversalFrontend:
             else:
                 self.skip_entity(entity, "unsupported")
 
-        self.designer.exit_entity(entity)
+        self.pipeline.exit_entity(entity)
 
     def draw_line_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         d, dxftype = entity.dxf, entity.dxftype()
         if dxftype == "LINE":
-            self.designer.draw_line(d.start, d.end, properties)
+            self.pipeline.draw_line(d.start, d.end, properties)
 
         elif dxftype in ("XLINE", "RAY"):
             start = d.start
             delta = d.unit_vector * self.config.infinite_line_length
             if dxftype == "XLINE":
-                self.designer.draw_line(
+                self.pipeline.draw_line(
                     start - delta / 2, start + delta / 2, properties
                 )
             elif dxftype == "RAY":
-                self.designer.draw_line(start, start + delta, properties)
+                self.pipeline.draw_line(start, start + delta, properties)
         else:
             raise TypeError(dxftype)
 
@@ -349,7 +421,7 @@ class UniversalFrontend:
     def get_font_face(self, properties: Properties) -> fonts.FontFace:
         font_face = properties.font
         if font_face is None:
-            return self.designer.default_font_face
+            return self.pipeline.default_font_face
         return font_face
 
     def draw_text_entity_2d(self, entity: DXFGraphic, properties: Properties) -> None:
@@ -357,7 +429,7 @@ class UniversalFrontend:
             for line, transform, cap_height in simplified_text_chunks(
                 entity, self.text_engine, font_face=self.get_font_face(properties)
             ):
-                self.designer.draw_text(
+                self.pipeline.draw_text(
                     line, transform, properties, cap_height, entity.dxftype()
                 )
         else:
@@ -374,7 +446,10 @@ class UniversalFrontend:
             self.skip_entity(mtext, "3D MTEXT not supported")
             return
         if mtext.has_columns or has_inline_formatting_codes(mtext.text):
-            self.draw_complex_mtext(mtext, properties)
+            try:
+                self.draw_complex_mtext(mtext, properties)
+            except text_layout.LayoutError as e:
+                print(f"skipping {str(mtext)} - {str(e)}")
         else:
             self.draw_simple_mtext(mtext, properties)
 
@@ -383,20 +458,20 @@ class UniversalFrontend:
         for line, transform, cap_height in simplified_text_chunks(
             mtext, self.text_engine, font_face=self.get_font_face(properties)
         ):
-            self.designer.draw_text(
+            self.pipeline.draw_text(
                 line, transform, properties, cap_height, mtext.dxftype()
             )
 
     def draw_complex_mtext(self, mtext: MText, properties: Properties) -> None:
         """Draw the content of a MTEXT entity including inline formatting codes."""
-        complex_mtext_renderer(self.ctx, self.designer, mtext, properties)
+        complex_mtext_renderer(self.ctx, self.pipeline, mtext, properties)
 
     def draw_curve_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         try:
             path = make_path(entity)
         except AttributeError:  # API usage error
             raise TypeError(f"Unsupported DXF type {entity.dxftype()}")
-        self.designer.draw_path(path, properties)
+        self.pipeline.draw_path(path, properties)
 
     def draw_point_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         point = cast(Point, entity)
@@ -413,7 +488,7 @@ class UniversalFrontend:
                 pdmode = 0
 
         if pdmode == 0:
-            self.designer.draw_point(entity.dxf.location, properties)
+            self.pipeline.draw_point(entity.dxf.location, properties)
         else:
             for entity in point.virtual_entities(pdsize, pdmode):
                 dxftype = entity.dxftype()
@@ -421,9 +496,9 @@ class UniversalFrontend:
                     start = entity.dxf.start
                     end = entity.dxf.end
                     if start.isclose(end):
-                        self.designer.draw_point(start, properties)
+                        self.pipeline.draw_point(start, properties)
                     else:  # direct draw by backend is OK!
-                        self.designer.draw_line(start, end, properties)
+                        self.pipeline.draw_line(start, end, properties)
                     pass
                 elif dxftype == "CIRCLE":
                     self.draw_curve_entity(entity, properties)
@@ -446,16 +521,16 @@ class UniversalFrontend:
                 return
             edge_visibility = entity.get_edges_visibility()
             if all(edge_visibility):
-                self.designer.draw_path(from_vertices(points), properties)
+                self.pipeline.draw_path(from_vertices(points), properties)
             else:
                 for a, b, visible in zip(points, points[1:], edge_visibility):
                     if visible:
-                        self.designer.draw_line(a, b, properties)
+                        self.pipeline.draw_line(a, b, properties)
 
         elif isinstance(entity, Solid):
             # set solid fill type for SOLID and TRACE
             properties.filling = Filling()
-            self.designer.draw_filled_polygon(
+            self.pipeline.draw_filled_polygon(
                 entity.wcs_vertices(close=False), properties
             )
         else:
@@ -491,11 +566,12 @@ class UniversalFrontend:
                 line_pattern = baseline.pattern_renderer(line.distance)
                 for s, e in line_pattern.render(line.start, line.end):
                     if ocs.transform:
-                        s, e = ocs.to_wcs((s.x, s.y, elevation)), ocs.to_wcs(
-                            (e.x, e.y, elevation)
+                        s, e = (
+                            ocs.to_wcs((s.x, s.y, elevation)),
+                            ocs.to_wcs((e.x, e.y, elevation)),
                         )
                     lines.append((s, e))
-        self.designer.draw_solid_lines(lines, properties)
+        self.pipeline.draw_solid_lines(lines, properties)
 
     def draw_hatch_entity(
         self,
@@ -545,10 +621,10 @@ class UniversalFrontend:
             paths = closed_loops(boundary_paths, ocs, elevation)
         if show_only_outline:
             for p in ignore_text_boxes(paths):
-                self.designer.draw_path(p, properties)
+                self.pipeline.draw_path(p, properties)
             return
         if paths:
-            self.designer.draw_filled_paths(ignore_text_boxes(paths), properties)
+            self.pipeline.draw_filled_paths(ignore_text_boxes(paths), properties)
 
     def draw_mpolygon_entity(self, entity: DXFGraphic, properties: Properties):
         def resolve_fill_color() -> str:
@@ -583,14 +659,14 @@ class UniversalFrontend:
         properties.color = line_color
         # draw boundary paths as lines
         for loop in loops:
-            self.designer.draw_path(loop, properties)
+            self.pipeline.draw_path(loop, properties)
 
     def draw_wipeout_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         wipeout = cast(Wipeout, entity)
         properties.filling = Filling()
         properties.color = self.ctx.current_layout_properties.background_color
         clipping_polygon = wipeout.boundary_path_wcs()
-        self.designer.draw_filled_polygon(clipping_polygon, properties)
+        self.pipeline.draw_filled_polygon(clipping_polygon, properties)
 
     def draw_viewport(self, vp: Viewport) -> None:
         # the "active" viewport and invisible viewports should be filtered at this
@@ -601,7 +677,7 @@ class UniversalFrontend:
         if not vp.is_top_view:
             self.log_message("Cannot render non top-view viewports")
             return
-        self.designer.draw_viewport(vp, self.ctx, self._bbox_cache)
+        self.pipeline.draw_viewport(vp, self.ctx, self._bbox_cache)
 
     def draw_ole2frame_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         ole2frame = cast(OLE2Frame, entity)
@@ -609,6 +685,121 @@ class UniversalFrontend:
         if not bbox.is_empty:
             self._draw_filled_rect(bbox.rect_vertices(), OLE2FRAME_COLOR)
 
+    def draw_image_entity(self, entity: DXFGraphic, properties: Properties) -> None:
+        image = cast(Image, entity)
+        image_policy = self.config.image_policy
+        image_def = image.image_def
+        assert image_def is not None
+
+        if image_policy in (
+            ImagePolicy.DISPLAY,
+            ImagePolicy.RECT,
+            ImagePolicy.MISSING,
+        ):
+            loaded_image = None
+            show_filename_if_missing = True
+
+            if (
+                image_policy == ImagePolicy.RECT
+                or not image.dxf.flags & Image.SHOW_IMAGE
+            ):
+                loaded_image = None
+                show_filename_if_missing = False
+            elif (
+                image_policy != ImagePolicy.MISSING
+                and self.ctx.document_dir is not None
+            ):
+                image_path = _find_image_path(
+                    self.ctx.document_dir, image_def.dxf.filename
+                )
+                with contextlib.suppress(FileNotFoundError):
+                    loaded_image = PIL.Image.open(image_path)
+
+            if loaded_image is not None:
+                color: RGB | RGBA
+                loaded_image = loaded_image.convert("RGBA")  # type: ignore
+
+                if image.dxf.contrast != 50:
+                    # note: this is only an approximation.
+                    # Unclear what the exact operation AutoCAD uses
+                    amount = image.dxf.contrast / 50
+                    loaded_image = PIL.ImageEnhance.Contrast(loaded_image).enhance(  # type: ignore
+                        amount
+                    )
+
+                if image.dxf.fade != 0:
+                    # note: this is only an approximation.
+                    # Unclear what the exact operation AutoCAD uses
+                    amount = image.dxf.fade / 100
+                    color = RGB.from_hex(
+                        self.ctx.current_layout_properties.background_color
+                    )
+                    loaded_image = _blend_image_towards(loaded_image, amount, color)  # type: ignore
+                if image.dxf.brightness != 50:
+                    # note: this is only an approximation.
+                    # Unclear what the exact operation AutoCAD uses
+                    amount = image.dxf.brightness / 50 - 1
+                    if amount > 0:
+                        color = RGBA(255, 255, 255, 255)
+                    else:
+                        color = RGBA(0, 0, 0, 255)
+                        amount = -amount
+                    loaded_image = _blend_image_towards(loaded_image, amount, color)  # type: ignore
+
+                if not image.dxf.flags & Image.USE_TRANSPARENCY:
+                    loaded_image.putalpha(255)  # type: ignore
+
+                if image.transparency != 0.0:
+                    loaded_image = _multiply_alpha(
+                        loaded_image,  # type: ignore
+                        1.0 - image.transparency,
+                    )
+                image_data = ImageData(
+                    image=np.array(loaded_image),
+                    transform=image.get_wcs_transform(),
+                    pixel_boundary_path=NumpyPoints2d(image.pixel_boundary_path()),
+                    use_clipping_boundary=image.dxf.flags & Image.USE_CLIPPING_BOUNDARY,
+                    remove_outside=image.dxf.clip_mode == 0,
+                )
+                self.pipeline.draw_image(image_data, properties)
+
+            elif show_filename_if_missing:
+                default_cap_height = 20
+                text = image_def.dxf.filename
+                font = self.pipeline.text_engine.get_font(
+                    self.get_font_face(properties)
+                )
+                text_width = font.text_width_ex(text, default_cap_height)
+                image_size = image.dxf.image_size
+                desired_width = image_size.x * 0.75
+                scale = desired_width / text_width
+                translate = Matrix44.translate(
+                    (image_size.x - desired_width) / 2,
+                    (image_size.y - default_cap_height * scale) / 2,
+                    0,
+                )
+                transform = (
+                    Matrix44.scale(scale) @ translate @ image.get_wcs_transform()
+                )
+                self.pipeline.draw_text(
+                    text,
+                    transform,
+                    properties,
+                    default_cap_height,
+                )
+
+            points = [v.vec2 for v in image.boundary_path_wcs()]
+            self.pipeline.draw_solid_lines(list(zip(points, points[1:])), properties)
+
+        elif self.config.image_policy == ImagePolicy.PROXY:
+            self.draw_proxy_graphic(entity.proxy_graphic, entity.doc)
+
+        elif self.config.image_policy == ImagePolicy.IGNORE:
+            pass
+
+        else:
+            raise ValueError(self.config.image_policy)
+
     def _draw_filled_rect(
         self,
         points: Iterable[Vec2],
@@ -618,7 +809,7 @@ class UniversalFrontend:
         props.color = color
         # default SOLID filling
         props.filling = Filling()
-        self.designer.draw_filled_polygon(points, props)
+        self.pipeline.draw_filled_polygon(points, props)
 
     def draw_mesh_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         builder = MeshBuilder.from_mesh(entity)  # type: ignore
@@ -628,7 +819,7 @@ class UniversalFrontend:
         self, builder: MeshBuilder, properties: Properties
     ) -> None:
         for face in builder.faces_as_vertices():
-            self.designer.draw_path(
+            self.pipeline.draw_path(
                 from_vertices(face, close=True), properties=properties
             )
 
@@ -661,22 +852,42 @@ class UniversalFrontend:
                 entity, segments=self.config.circle_approximation_count // 2
             )
             for polygon in trace.polygons():  # polygon is a sequence of Vec2()
+                if len(polygon) < 3:
+                    continue
                 if transform:
                     points = ocs.points_to_wcs(
                         Vec3(v.x, v.y, elevation) for v in polygon
                     )
                 else:
-                    points = polygon
+                    points = polygon  # type: ignore
                 # Set default SOLID filling for LWPOLYLINE
                 properties.filling = Filling()
-                self.designer.draw_filled_polygon(points, properties)
+                self.pipeline.draw_filled_polygon(points, properties)
             return
-
-        self.designer.draw_path(make_path(entity), properties)
+        polyline_path = make_path(entity)
+        if len(polyline_path):
+            self.pipeline.draw_path(polyline_path, properties)
 
     def draw_composite_entity(self, entity: DXFGraphic, properties: Properties) -> None:
         def draw_insert(insert: Insert):
+            # Block reference attributes are located __outside__ the block reference!
             self.draw_entities(insert.attribs)
+            clip = xclip.XClip(insert)
+            is_clipping_active = clip.has_clipping_path and clip.is_clipping_enabled
+
+            if is_clipping_active:
+                boundary_path = clip.get_wcs_clipping_path()
+                if not boundary_path.is_inverted_clip:
+                    clipping_shape = clipping_portal.find_best_clipping_shape(
+                        boundary_path.vertices
+                    )
+                else:  # inverted clipping path
+                    clipping_shape = clipping_portal.make_inverted_clipping_shape(
+                        boundary_path.inner_polygon(),
+                        outer_bounds=boundary_path.outer_bounds(),
+                    )
+                self.pipeline.push_clipping_shape(clipping_shape, None)
+
             # draw_entities() includes the visibility check:
             self.draw_entities(
                 insert.virtual_entities(
@@ -685,6 +896,13 @@ class UniversalFrontend:
                 )
             )
 
+            if is_clipping_active and clip.get_xclip_frame_policy():
+                self.pipeline.draw_path(
+                    path=from_vertices(boundary_path.inner_polygon(), close=True),
+                    properties=properties,
+                )
+                self.pipeline.pop_clipping_shape()
+
         if isinstance(entity, Insert):
             self.ctx.push_state(properties)
             if entity.mcount > 1:
@@ -703,13 +921,14 @@ class UniversalFrontend:
         else:
             raise TypeError(entity.dxftype())
 
-    def draw_proxy_graphic(self, data: bytes, doc) -> None:
-        if data:
-            try:
-                self.draw_entities(virtual_entities(ProxyGraphic(data, doc)))
-            except ProxyGraphicError as e:
-                print(str(e))
-                print(POST_ISSUE_MSG)
+    def draw_proxy_graphic(self, data: bytes | None, doc) -> None:
+        if not data:
+            return
+        try:
+            self.draw_entities(virtual_entities(ProxyGraphic(data, doc)))
+        except ProxyGraphicError as e:
+            print(str(e))
+            print(POST_ISSUE_MSG)
 
 
 class Frontend(UniversalFrontend):
@@ -738,9 +957,9 @@ class Frontend(UniversalFrontend):
         config: Configuration = Configuration(),
         bbox_cache: Optional[ezdxf.bbox.Cache] = None,
     ):
-        from .designer import Designer2d
+        from .pipeline import RenderPipeline2d
 
-        super().__init__(ctx, Designer2d(out), config, bbox_cache)
+        super().__init__(ctx, RenderPipeline2d(out), config, bbox_cache)
 
 
 def is_spatial_text(extrusion: Vec3) -> bool:
@@ -801,7 +1020,7 @@ def _draw_entities(
                 frontend.skip_entity(entity, "Cannot parse DXF entity")
                 continue
         properties = ctx.resolve_all(entity)
-        frontend.override_properties(entity, properties)
+        frontend.exec_property_override(entity, properties)
         if properties.is_visible:
             frontend.draw_entity(entity, properties)
         else:
@@ -835,3 +1054,44 @@ def _draw_viewports(frontend: UniversalFrontend, viewports: list[Viewport]) -> N
     # Draw viewports in order of "status"
     for viewport in viewports:
         frontend.draw_viewport(viewport)
+
+
+def _blend_image_towards(
+    image: PIL.Image.Image, amount: float, color: RGB | RGBA
+) -> PIL.Image.Image:
+    original_alpha = image.split()[-1]
+    destination = PIL.Image.new("RGBA", image.size, color)
+    updated_image = PIL.Image.blend(image, destination, amount)
+    updated_image.putalpha(original_alpha)
+    return updated_image
+
+
+def _multiply_alpha(image: PIL.Image.Image, amount: float) -> PIL.Image.Image:
+    output_image = np.array(image, dtype=np.float64)
+    output_image[:, :, 3] *= amount
+    return PIL.Image.fromarray(output_image.astype(np.uint8), "RGBA")
+
+
+def _find_image_path(document_dir: pathlib.Path, filename: str) -> pathlib.Path:
+    # BricsCAD stores the path to the image as full-path or relative-path if so
+    # set in the "Attach Raster Image" dialog.
+    # See notes in knowledge graph: [[IMAGE File Paths]]
+    # https://ezdxf.mozman.at/notes/#/page/image%20file%20paths
+
+    # BricsCAD/AutoCAD stores filepaths with backslashes.
+    filename = filename.replace("\\", os.path.sep)
+
+    # try absolute path:
+    filepath = pathlib.Path(filename)
+    if filepath.exists():
+        return filepath
+
+    # try relative path to document:
+    filepath = document_dir / filename
+    filepath = filepath.resolve()
+    if filepath.exists():
+        return filepath
+
+    # try document dir:
+    filepath = document_dir / pathlib.Path(filename).name  # stem + suffix
+    return filepath
