@@ -1,4 +1,4 @@
-#  Copyright (c) 2021-2022, Manfred Moitzi
+#  Copyright (c) 2021-2023, Manfred Moitzi
 #  License: MIT License
 from __future__ import annotations
 from typing import Iterable, Iterator
@@ -6,6 +6,7 @@ from ezdxf.entities import DXFGraphic, DXFEntity
 from ezdxf.lldxf import const
 from ezdxf.lldxf.tagwriter import AbstractTagWriter
 from ezdxf.protocols import SupportsVirtualEntities
+from ezdxf.entities.copy import default_copy, CopyNotSupported
 
 
 class DXFGraphicProxy(DXFGraphic):
@@ -31,16 +32,16 @@ class DXFGraphicProxy(DXFGraphic):
     def __virtual_entities__(self) -> Iterator[DXFGraphic]:
         """Implements the SupportsVirtualEntities protocol."""
         if isinstance(self.entity, SupportsVirtualEntities):
-            return self.entity.__virtual_entities__()  # type: ignore
+            return self.entity.__virtual_entities__()
         if hasattr(self.entity, "virtual_entities"):
-            return self.entity.virtual_entities()  # type: ignore
+            return self.entity.virtual_entities()
         return iter([])
 
     def virtual_entities(self) -> Iterable[DXFGraphic]:
         return self.__virtual_entities__()
 
-    def raw_copy(self) -> DXFGraphicProxy:
-        raise const.DXFTypeError(f"Cloning of DXFGraphicProxy() not supported.")
+    def copy(self, copy_strategy=default_copy) -> DXFGraphicProxy:
+        raise CopyNotSupported(f"Copying of DXFGraphicProxy() not supported.")
 
     def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
         # prevent dxf export
