@@ -221,6 +221,13 @@ class Page:
 
     @classmethod
     def from_dxf_layout(cls, layout: DXFLayout) -> Self:
+        """Returns the :class:`Page` based on the DXF attributes stored in the LAYOUT 
+        entity. The modelspace layout often **doesn't** have usable page settings!
+
+        Args:
+            layout: any paperspace layout or the modelspace layout
+
+        """
         # all layout measurements in mm
         width = round(layout.dxf.paper_width, 1)
         height = round(layout.dxf.paper_height, 1)
@@ -291,6 +298,7 @@ class Settings:
             e.g. 0.15 is 15% of :attr:`max_stroke_width`
         output_coordinate_space: expert feature to map the DXF coordinates to the
             output coordinate system [0, output_coordinate_space]
+        output_layers: For supported backends, separate the entities into 'layers' in the output
 
     """
 
@@ -314,6 +322,7 @@ class Settings:
     # dimension - aspect ratio is always preserved - these are CAD drawings!
     # The SVGBackend uses this feature to map all coordinates to integer values:
     output_coordinate_space: float = 1_000_000  # e.g. for SVGBackend
+    output_layers: bool = True
 
     def __post_init__(self) -> None:
         if self.content_rotation not in (0, 90, 180, 270):
@@ -472,7 +481,7 @@ def placement_matrix(
     canvas = BoundingBox2d(corners)
 
     # shift content to first quadrant +x/+y
-    tx, ty = canvas.extmin  # type: ignore
+    tx, ty = canvas.extmin
 
     # align content within margins
     view_box_content_x = (
