@@ -41,6 +41,9 @@ class AuditError(IntEnum):
     REMOVED_STANDALONE_ATTRIB_ENTITY = 13
     MISPLACED_ROOT_DICT = 14
     ROOT_DICT_NOT_FOUND = 15
+    REMOVED_ENTITY_WITH_INVALID_OWNER_HANDLE = 16
+    MODELSPACE_NOT_FOUND = 17
+    ACTIVE_PAPERSPACE_LAYOUT_NOT_FOUND = 18
 
     UNDEFINED_LINETYPE = 100
     UNDEFINED_DIMENSION_STYLE = 101
@@ -89,6 +92,9 @@ class AuditError(IntEnum):
     INVALID_TRANSPARENCY = 223
     INVALID_CREASE_VALUE_COUNT = 224
     INVALID_ELLIPSE_RATIO = 225
+    INVALID_HATCH_BOUNDARY_PATH = 226
+    TAG_ATTRIBUTE_MISSING = 227
+    INVALID_MESH_DATA = 228
 
 
 REQUIRED_ROOT_DICT_ENTRIES = ("ACAD_GROUP", "ACAD_PLOTSTYLENAME")
@@ -235,6 +241,8 @@ class Auditor:
             return self.errors
         self.doc.entitydb.audit(self)
         self.check_root_dict_entries()
+        self.check_modelspace_exist()
+        self.check_active_layout_exist()
         self.check_tables()
         self.doc.objects.audit(self)
         self.doc.blocks.audit(self)
@@ -294,6 +302,26 @@ class Auditor:
                     dxf_entity=rootdict,
                 )
 
+    def check_modelspace_exist(self) -> None:
+        msp = self.doc.modelspace()
+        if not msp.is_alive:
+            self.add_error(
+                code=AuditError.MODELSPACE_NOT_FOUND,
+                message=f"Required modelspace layout not found.",
+            )
+
+    def check_active_layout_exist(self) -> None:
+        try:
+            layout = self.doc.active_layout()
+        except const.DXFStructureError:
+            layout = None  # type: ignore
+
+        if layout is None or layout.is_alive is False:
+            self.add_error(
+                code=AuditError.ACTIVE_PAPERSPACE_LAYOUT_NOT_FOUND,
+                message=f"Required active paperspace layout not found.",
+            )
+
     def check_tables(self) -> None:
         table_section = self.doc.tables
         table_section.viewports.audit(self)
@@ -491,7 +519,7 @@ class BlockCycleDetector:
         self.blocks = self._build_block_ledger(doc.blocks)
 
     def _build_block_ledger(self, blocks: BlocksSection) -> dict[str, set[str]]:
-        ledger = dict()
+        ledger = {}
         for block in blocks:
             inserts = {
                 self.key(insert.dxf.get("name", "")) for insert in block.query("INSERT")
