@@ -1,7 +1,10 @@
-# Copyright (c) 2019-2023 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import TYPE_CHECKING, Iterable, Union, Optional, Sequence
+from typing import TYPE_CHECKING, Iterable, Union, Optional, Sequence, Any
+from typing_extensions import Self, override
+import logging
+
 from ezdxf.lldxf.attributes import (
     DXFAttr,
     DXFAttributes,
@@ -9,24 +12,23 @@ from ezdxf.lldxf.attributes import (
     XType,
     group_code_mapping,
 )
-from ezdxf.lldxf.const import (
-    SUBCLASS_MARKER,
-    DXF2000,
-    DXF2004,
-    DXFTypeError,
-    DXF2013,
-    DXFStructureError,
-)
+from ezdxf.lldxf import const
 from ezdxf.lldxf.tags import Tags, DXFTag
 from ezdxf.math import Matrix44
-from ezdxf.tools import crypt
+from ezdxf.tools import crypt, guid
+from ezdxf import msgtypes
+
 from .dxfentity import base_class, SubclassProcessor
 from .dxfgfx import DXFGraphic, acdb_entity
 from .factory import register_entity
+from .copy import default_copy
+from .temporary_transform import TransformByBlockReference
 
 if TYPE_CHECKING:
     from ezdxf.entities import DXFNamespace
     from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf import xref
+
 
 __all__ = [
     "Body",
@@ -39,12 +41,13 @@ __all__ = [
     "SweptSurface",
 ]
 
+logger = logging.getLogger("ezdxf")
 acdb_modeler_geometry = DefSubclass(
     "AcDbModelerGeometry",
     {
         "version": DXFAttr(70, default=1),
-        "flags": DXFAttr(290, dxfversion=DXF2013),
-        "uid": DXFAttr(2, dxfversion=DXF2013),
+        "flags": DXFAttr(290, dxfversion=const.DXF2013),
+        "uid": DXFAttr(2, dxfversion=const.DXF2013),
     },
 )
 acdb_modeler_geometry_group_codes = group_code_mapping(acdb_modeler_geometry)
@@ -80,7 +83,7 @@ class Body(DXFGraphic):
 
     DXFTYPE = "BODY"
     DXFATTRIBS = DXFAttributes(base_class, acdb_entity, acdb_modeler_geometry)
-    MIN_DXF_VERSION_FOR_EXPORT = DXF2000
+    MIN_DXF_VERSION_FOR_EXPORT = const.DXF2000
 
     def __init__(self) -> None:
         super().__init__()
@@ -89,6 +92,7 @@ class Body(DXFGraphic):
         self._sat: Sequence[str] = tuple()
         self._sab: bytes = b""
         self._update = False
+        self._temporary_transformation = TransformByBlockReference()
 
     @property
     def acis_data(self) -> Union[bytes, Sequence[str]]:
@@ -113,9 +117,7 @@ class Body(DXFGraphic):
     def sab(self) -> bytes:
         """Get/Set :term:`SAB` data as bytes."""
         if (  # load SAB data on demand
-            self.doc is not None
-            and self.has_binary_data
-            and len(self._sab) == 0
+            self.doc is not None and self.has_binary_data and len(self._sab) == 0
         ):
             self._sab = self.doc.acdsdata.get_acis_data(self.dxf.handle)
         return self._sab
@@ -132,14 +134,63 @@ class Body(DXFGraphic):
         ``False`` if the entity contains :term:`SAT` data.
         """
         if self.doc:
-            return self.doc.dxfversion >= DXF2013
+            return self.doc.dxfversion >= const.DXF2013
         else:
             return False
 
-    def raw_copy(self):
-        """Prevent copying. (internal interface)"""
-        raise DXFTypeError("Copying of ACIS data not supported.")
+    @override
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        assert isinstance(entity, Body)
+        entity.sat = self.sat
+        entity.sab = self.sab  # load SAB on demand
+        entity.dxf.uid = guid()
+        entity._temporary_transformation = self._temporary_transformation
 
+    @override
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        super().map_resources(clone, mapping)
+        clone.convert_acis_data()
+
+    def convert_acis_data(self) -> None:
+        if self.doc is None:
+            return
+        msg = ""
+        dxfversion = self.doc.dxfversion
+        if dxfversion < const.DXF2013:
+            if self._sab:
+                self._sab = b""
+                msg = "DXF version mismatch, can't convert ACIS data from SAB to SAT, SAB data removed."
+        else:
+            if self._sat:
+                self._sat = tuple()
+                msg = "DXF version mismatch, can't convert ACIS data from SAT to SAB, SAT data removed."
+        if msg:
+            logger.info(msg)
+
+    @override
+    def notify(self, message_type: int, data: Any = None) -> None:
+        if message_type == msgtypes.COMMIT_PENDING_CHANGES:
+            self._temporary_transformation.apply_transformation(self)
+
+    @override
+    def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
+        msg = ""
+        if tagwriter.dxfversion < const.DXF2013:
+            valid = len(self.sat) > 0
+            if not valid:
+                msg = f"{str(self)} doesn't have SAT data, skipping DXF export"
+        else:
+            valid = len(self.sab) > 0
+            if not valid:
+                msg = f"{str(self)} doesn't have SAB data, skipping DXF export"
+        if not valid:
+            logger.info(msg)
+        if valid and self._temporary_transformation.get_matrix() is not None:
+            logger.warning(f"{str(self)} has unapplied temporary transformations.")
+        return valid
+
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -158,11 +209,12 @@ class Body(DXFGraphic):
         text_lines = tags2textlines(tag for tag in tags if tag.code in (1, 3))
         self._sat = tuple(crypt.decode(text_lines))
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags. (internal API)"""
         super().export_entity(tagwriter)
-        tagwriter.write_tag2(SUBCLASS_MARKER, acdb_modeler_geometry.name)
-        if tagwriter.dxfversion >= DXF2013:
+        tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_modeler_geometry.name)
+        if tagwriter.dxfversion >= const.DXF2013:
             # ACIS data is stored in the ACDSDATA section as SAB
             if self.doc and self._update:
                 # write back changed SAB data into AcDsDataSection or create
@@ -195,13 +247,19 @@ class Body(DXFGraphic):
         else:
             return "\n".join(self.sat)
 
+    @override
     def destroy(self) -> None:
         if self.has_binary_data:
             self.doc.acdsdata.del_acis_data(self.dxf.handle)  # type: ignore
         super().destroy()
 
-    def transform(self, m: Matrix44) -> DXFGraphic:
-        raise NotImplementedError("cannot transform ACIS entities")
+    @override
+    def transform(self, m: Matrix44) -> Self:
+        self._temporary_transformation.add_matrix(m)
+        return self
+
+    def temporary_transformation(self) -> TransformByBlockReference:
+        return self._temporary_transformation
 
 
 def tags2textlines(tags: Iterable) -> Iterable[str]:
@@ -259,6 +317,7 @@ class Solid3d(Body):
         base_class, acdb_entity, acdb_modeler_geometry, acdb_3dsolid
     )
 
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -267,27 +326,26 @@ class Solid3d(Body):
             processor.fast_load_dxfattribs(dxf, acdb_3dsolid_group_codes, 3)
         return dxf
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         # AcDbModelerGeometry export is done by parent class
-        if tagwriter.dxfversion > DXF2004:
-            tagwriter.write_tag2(SUBCLASS_MARKER, acdb_3dsolid.name)
+        if tagwriter.dxfversion > const.DXF2004:
+            tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_3dsolid.name)
             self.dxf.export_dxf_attribs(tagwriter, "history_handle")
 
 
 def load_matrix(subclass: Tags, code: int) -> Matrix44:
     values = [tag.value for tag in subclass.find_all(code)]
     if len(values) != 16:
-        raise DXFStructureError("Invalid transformation matrix.")
+        raise const.DXFStructureError("Invalid transformation matrix.")
     return Matrix44(values)
 
 
-def export_matrix(
-    tagwriter: AbstractTagWriter, code: int, matrix: Matrix44
-) -> None:
+def export_matrix(tagwriter: AbstractTagWriter, code: int, matrix: Matrix44) -> None:
     for value in list(matrix):
         tagwriter.write_tag2(code, value)
 
@@ -311,6 +369,7 @@ class Surface(Body):
         base_class, acdb_entity, acdb_modeler_geometry, acdb_surface
     )
 
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -319,13 +378,14 @@ class Surface(Body):
             processor.fast_load_dxfattribs(dxf, acdb_surface_group_codes, 3)
         return dxf
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         # AcDbModelerGeometry export is done by parent class
-        tagwriter.write_tag2(SUBCLASS_MARKER, acdb_surface.name)
+        tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_surface.name)
         self.dxf.export_dxf_attribs(tagwriter, ["u_count", "v_count"])
 
 
@@ -356,9 +416,7 @@ acdb_extruded_surface = DefSubclass(
         "base_point_set": DXFAttr(294, default=0),  # bool
         "sweep_entity_transform_computed": DXFAttr(295, default=0),  # bool
         "path_entity_transform_computed": DXFAttr(296, default=0),  # bool
-        "reference_vector_for_controlling_twist": DXFAttr(
-            11, xtype=XType.point3d
-        ),
+        "reference_vector_for_controlling_twist": DXFAttr(11, xtype=XType.point3d),
     },
 )
 acdb_extruded_surface_group_codes = group_code_mapping(acdb_extruded_surface)
@@ -383,6 +441,21 @@ class ExtrudedSurface(Surface):
         self.sweep_entity_transformation_matrix = Matrix44()
         self.path_entity_transformation_matrix = Matrix44()
 
+    @override
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        assert isinstance(entity, ExtrudedSurface)
+        super().copy_data(entity, copy_strategy)
+        entity.transformation_matrix_extruded_entity = (
+            self.transformation_matrix_extruded_entity.copy()
+        )
+        entity.sweep_entity_transformation_matrix = (
+            self.sweep_entity_transformation_matrix.copy()
+        )
+        entity.path_entity_transformation_matrix = (
+            self.path_entity_transformation_matrix.copy()
+        )
+
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -399,13 +472,14 @@ class ExtrudedSurface(Surface):
         self.sweep_entity_transformation_matrix = load_matrix(tags, code=46)
         self.path_entity_transformation_matrix = load_matrix(tags, code=47)
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         # AcDbModelerGeometry export is done by parent class
-        tagwriter.write_tag2(SUBCLASS_MARKER, acdb_extruded_surface.name)
+        tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_extruded_surface.name)
         self.dxf.export_dxf_attribs(tagwriter, ["class_id", "sweep_vector"])
         export_matrix(
             tagwriter,
@@ -426,9 +500,7 @@ class ExtrudedSurface(Surface):
         export_matrix(
             tagwriter, code=46, matrix=self.sweep_entity_transformation_matrix
         )
-        export_matrix(
-            tagwriter, code=47, matrix=self.path_entity_transformation_matrix
-        )
+        export_matrix(tagwriter, code=47, matrix=self.path_entity_transformation_matrix)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
@@ -485,6 +557,15 @@ class LoftedSurface(Surface):
         super().__init__()
         self.transformation_matrix_lofted_entity = Matrix44()
 
+    @override
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        assert isinstance(entity, LoftedSurface)
+        super().copy_data(entity, copy_strategy)
+        entity.transformation_matrix_lofted_entity = (
+            self.transformation_matrix_lofted_entity.copy()
+        )
+
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -499,19 +580,18 @@ class LoftedSurface(Surface):
     def load_matrices(self, tags: Tags):
         self.transformation_matrix_lofted_entity = load_matrix(tags, code=40)
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         # AcDbModelerGeometry export is done by parent class
-        tagwriter.write_tag2(SUBCLASS_MARKER, acdb_lofted_surface.name)
+        tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_lofted_surface.name)
         export_matrix(
             tagwriter, code=40, matrix=self.transformation_matrix_lofted_entity
         )
-        self.dxf.export_dxf_attribs(
-            tagwriter, acdb_lofted_surface.attribs.keys()
-        )
+        self.dxf.export_dxf_attribs(tagwriter, acdb_lofted_surface.attribs.keys())
 
 
 acdb_revolved_surface = DefSubclass(
@@ -552,6 +632,15 @@ class RevolvedSurface(Surface):
         super().__init__()
         self.transformation_matrix_revolved_entity = Matrix44()
 
+    @override
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        assert isinstance(entity, RevolvedSurface)
+        super().copy_data(entity, copy_strategy)
+        entity.transformation_matrix_revolved_entity = (
+            self.transformation_matrix_revolved_entity.copy()
+        )
+
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -566,13 +655,14 @@ class RevolvedSurface(Surface):
     def load_matrices(self, tags: Tags):
         self.transformation_matrix_revolved_entity = load_matrix(tags, code=42)
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         # AcDbModelerGeometry export is done by parent class
-        tagwriter.write_tag2(SUBCLASS_MARKER, acdb_revolved_surface.name)
+        tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_revolved_surface.name)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
@@ -635,9 +725,7 @@ acdb_swept_surface = DefSubclass(
         "base_point_set": DXFAttr(294, default=0),  # bool
         "sweep_entity_transform_computed": DXFAttr(295, default=0),  # bool
         "path_entity_transform_computed": DXFAttr(296, default=0),  # bool
-        "reference_vector_for_controlling_twist": DXFAttr(
-            11, xtype=XType.point3d
-        ),
+        "reference_vector_for_controlling_twist": DXFAttr(11, xtype=XType.point3d),
     },
 )
 acdb_swept_surface_group_codes = group_code_mapping(acdb_swept_surface)
@@ -663,6 +751,24 @@ class SweptSurface(Surface):
         self.sweep_entity_transformation_matrix = Matrix44()
         self.path_entity_transformation_matrix = Matrix44()
 
+    @override
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        assert isinstance(entity, SweptSurface)
+        super().copy_data(entity, copy_strategy)
+        entity.transformation_matrix_sweep_entity = (
+            self.transformation_matrix_sweep_entity.copy()
+        )
+        entity.transformation_matrix_path_entity = (
+            self.transformation_matrix_path_entity.copy()
+        )
+        entity.sweep_entity_transformation_matrix = (
+            self.sweep_entity_transformation_matrix.copy()
+        )
+        entity.path_entity_transformation_matrix = (
+            self.path_entity_transformation_matrix.copy()
+        )
+
+    @override
     def load_dxf_attribs(
         self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
@@ -680,13 +786,14 @@ class SweptSurface(Surface):
         self.sweep_entity_transformation_matrix = load_matrix(tags, code=46)
         self.path_entity_transformation_matrix = load_matrix(tags, code=47)
 
+    @override
     def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         # base class export is done by parent class
         super().export_entity(tagwriter)
         # AcDbEntity export is done by parent class
         # AcDbModelerGeometry export is done by parent class
-        tagwriter.write_tag2(SUBCLASS_MARKER, acdb_swept_surface.name)
+        tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_swept_surface.name)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
@@ -697,9 +804,7 @@ class SweptSurface(Surface):
         export_matrix(
             tagwriter, code=40, matrix=self.transformation_matrix_sweep_entity
         )
-        export_matrix(
-            tagwriter, code=41, matrix=self.transformation_matrix_path_entity
-        )
+        export_matrix(tagwriter, code=41, matrix=self.transformation_matrix_path_entity)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
@@ -715,9 +820,7 @@ class SweptSurface(Surface):
         export_matrix(
             tagwriter, code=46, matrix=self.sweep_entity_transformation_matrix
         )
-        export_matrix(
-            tagwriter, code=47, matrix=self.path_entity_transformation_matrix
-        )
+        export_matrix(tagwriter, code=47, matrix=self.path_entity_transformation_matrix)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
