@@ -1,12 +1,8 @@
-# Copyright (c) 2019-2021, Manfred Moitzi
+# Copyright (c) 2019-2024, Manfred Moitzi
 # License: MIT-License
-from typing import (
-    TYPE_CHECKING,
-    Union,
-    Dict,
-    Optional,
-    List,
-)
+from __future__ import annotations
+from typing import TYPE_CHECKING, Union, Optional
+from typing_extensions import Self
 import logging
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.const import (
@@ -14,6 +10,7 @@ from ezdxf.lldxf.const import (
     DXFKeyError,
     DXFValueError,
     DXFTypeError,
+    DXFStructureError,
 )
 from ezdxf.lldxf.attributes import (
     DXFAttr,
@@ -27,22 +24,33 @@ from ezdxf.audit import AuditError
 from ezdxf.entities import factory, DXFGraphic
 from .dxfentity import base_class, SubclassProcessor, DXFEntity
 from .dxfobj import DXFObject
-
-logger = logging.getLogger("ezdxf")
+from .copy import default_copy, CopyNotSupported
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, Drawing, DXFNamespace, Auditor, XRecord
+    from ezdxf.entities import DXFNamespace, XRecord
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.document import Drawing
+    from ezdxf.audit import Auditor
+    from ezdxf import xref
 
 __all__ = ["Dictionary", "DictionaryWithDefault", "DictionaryVar"]
+logger = logging.getLogger("ezdxf")
 
 acdb_dictionary = DefSubclass(
     "AcDbDictionary",
     {
-        # If set to 1, indicates that elements of the dictionary are to be treated
-        # as hard-owned:
+        # If hard_owned is set to 1 the entries are owned by the DICTIONARY.
+        # The 1 state seems to be the default value, but is not documented by
+        # the DXF reference.
+        # BricsCAD creates the root DICTIONARY and the top level DICTIONARY entries
+        # without group code 280 tags, and they are all definitely hard owner of their
+        # entries, because their entries have the DICTIONARY handle as owner handle.
         "hard_owned": DXFAttr(
             280,
-            default=0,
+            default=1,
+
+            # 2024-11-18: changed to False because of issue #1203
+            # 2025-04-09: changed back to True because of issue #1279
             optional=True,
             validator=validator.is_integer_bool,
             fixer=RETURN_DEFAULT,
@@ -64,6 +72,8 @@ acdb_dictionary = DefSubclass(
         # 350: entry handle, some DICTIONARY objects have 360 as handle group code,
         # this is accepted by AutoCAD but not documented by the DXF reference!
         # ezdxf replaces group code 360 by 350.
+        # - group code 350 is a soft-owner handle
+        # - group code 360 is a hard-owner handle
     },
 )
 acdb_dictionary_group_codes = group_code_mapping(acdb_dictionary)
@@ -89,12 +99,12 @@ class Dictionary(DXFObject):
     DXFTYPE = "DICTIONARY"
     DXFATTRIBS = DXFAttributes(base_class, acdb_dictionary)
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
-        self._data: Dict[str, Union[str, DXFObject]] = dict()
+        self._data: dict[str, Union[str, DXFObject]] = dict()
         self._value_code = VALUE_CODE
 
-    def _copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
         """Copy hard owned entities but do not store the copies in the entity
         database, this is a second step (factory.bind), this is just real copying.
         """
@@ -102,9 +112,58 @@ class Dictionary(DXFObject):
         entity._value_code = self._value_code
         if self.dxf.hard_owned:
             # Reactors are removed from the cloned DXF objects.
-            entity._data = {key: entity.copy() for key, entity in self.items()}
+            data: dict[str, DXFEntity] = dict()
+            for key, ent in self.items():
+                # ignore strings and None - these entities do not exist
+                # in the entity database
+                if isinstance(ent, DXFEntity):
+                    try:  # todo: follow CopyStrategy.ignore_copy_errors_in_linked entities
+                        data[key] = ent.copy(copy_strategy=copy_strategy)
+                    except CopyNotSupported:
+                        if copy_strategy.settings.ignore_copy_errors_in_linked_entities:
+                            logger.warning(
+                                f"copy process ignored {str(ent)} - this may cause problems in AutoCAD"
+                            )
+                        else:
+                            raise
+            entity._data = data  # type: ignore
         else:
-            entity._data = {key: entity for key, entity in self.items()}
+            entity._data = dict(self._data)
+
+    def get_handle_mapping(self, clone: Dictionary) -> dict[str, str]:
+        """Returns handle mapping for in-object copies."""
+        handle_mapping: dict[str, str] = dict()
+        if not self.is_hard_owner:
+            return handle_mapping
+
+        for key, entity in self.items():
+            if not isinstance(entity, DXFEntity):
+                continue
+            copied_entry = clone.get(key)
+            if copied_entry:
+                handle_mapping[entity.dxf.handle] = copied_entry.dxf.handle
+        return handle_mapping
+
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        assert isinstance(clone, Dictionary)
+        super().map_resources(clone, mapping)
+        if self.is_hard_owner:
+            return
+        data = dict()
+        for key, entity in self.items():
+            if not isinstance(entity, DXFEntity):
+                continue
+            entity_copy = mapping.get_reference_of_copy(entity.dxf.handle)
+            if entity_copy:
+                data[key] = entity
+        clone._data = data  # type: ignore
+
+    def del_source_of_copy(self) -> None:
+        super().del_source_of_copy()
+        for _, entity in self.items():
+            if isinstance(entity, DXFEntity) and entity.is_alive:
+                entity.del_source_of_copy()
 
     def post_bind_hook(self) -> None:
         """Called by binding a new or copied dictionary to the document,
@@ -126,8 +185,8 @@ class Dictionary(DXFObject):
             object_section.add_object(entity)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             tags = processor.fast_load_dxfattribs(
@@ -157,7 +216,7 @@ class Dictionary(DXFObject):
         # Use same value code as loaded:
         self._value_code = value_code
 
-    def post_load_hook(self, doc: "Drawing") -> None:
+    def post_load_hook(self, doc: Drawing) -> None:
         super().post_load_hook(doc)
         db = doc.entitydb
 
@@ -171,14 +230,14 @@ class Dictionary(DXFObject):
             for k, v in list(items()):
                 self.__setitem__(k, v)
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_dictionary.name)
         self.dxf.export_dxf_attribs(tagwriter, ["hard_owned", "cloning"])
         self.export_dict(tagwriter)
 
-    def export_dict(self, tagwriter: "TagWriter"):
+    def export_dict(self, tagwriter: AbstractTagWriter):
         # key: dict key string
         # value: DXFEntity or handle as string
         # Ignore invalid handles at export, because removing can create an empty
@@ -264,9 +323,7 @@ class Dictionary(DXFObject):
 
     count = __len__
 
-    def get(
-        self, key: str, default: Optional[DXFObject] = None
-    ) -> Optional[DXFObject]:
+    def get(self, key: str, default: Optional[DXFObject] = None) -> Optional[DXFObject]:
         """Returns the :class:`DXFEntity` for `key`, if `key` exist else
         `default`. An entity can be a handle string if the entity
         does not exist.
@@ -274,8 +331,20 @@ class Dictionary(DXFObject):
         """
         return self._data.get(key, default)  # type: ignore
 
+    def find_key(self, entity: DXFEntity) -> str:
+        """Returns the DICTIONARY key string for `entity` or an empty string if not
+        found.
+        """
+        for key, entry in self._data.items():
+            if entry is entity:
+                return key
+        return ""
+
     def add(self, key: str, entity: DXFObject) -> None:
-        """Add entry ``(key, value)``.
+        """Add entry (key, value).
+
+        If the DICTIONARY is hard owner of its entries, the :meth:`add` does NOT take
+        ownership of the entity automatically.
 
         Raises:
             DXFValueError: invalid entity handle
@@ -284,11 +353,9 @@ class Dictionary(DXFObject):
         """
         if isinstance(entity, str):
             if not is_valid_handle(entity):
-                raise DXFValueError(
-                    f"Invalid entity handle #{entity} for key {key}"
-                )
+                raise DXFValueError(f"Invalid entity handle #{entity} for key {key}")
         elif isinstance(entity, DXFGraphic):
-            if self.doc is not None and self.doc.is_loading:
+            if self.doc is not None and self.doc.is_loading:  # type: ignore
                 # AutoCAD add-ons can store graphical entities in DICTIONARIES
                 # in the OBJECTS section and AutoCAD does not complain - so just
                 # preserve them!
@@ -297,11 +364,14 @@ class Dictionary(DXFObject):
             else:
                 # Do not allow ezdxf users to add graphical entities to a
                 # DICTIONARY object!
-                raise DXFTypeError(
-                    f"Graphic entities not allowed: {entity.dxftype()}"
-                )
+                raise DXFTypeError(f"Graphic entities not allowed: {entity.dxftype()}")
         self._data[key] = entity
 
+    def take_ownership(self, key: str, entity: DXFObject):
+        """Add entry (key, value) and take ownership."""
+        self.add(key, entity)
+        entity.dxf.owner = self.dxf.handle
+
     def remove(self, key: str) -> None:
         """Delete entry `key`. Raises :class:`DXFKeyError`, if `key` does not
         exist. Destroys hard owned DXF entities.
@@ -316,7 +386,6 @@ class Dictionary(DXFObject):
             entity = self.__getitem__(key)
             # Presumption: hard owned DXF objects always reside in the OBJECTS
             # section.
-            # TODO: use entity.destroy()?
             self.doc.objects.delete_entity(entity)  # type: ignore
         del data[key]
 
@@ -342,11 +411,10 @@ class Dictionary(DXFObject):
         # Presumption: hard owned DXF objects always reside in the OBJECTS section
         objects = self.doc.objects  # type: ignore
         for key, entity in self.items():
-            # TODO: is entity a string?
-            # TODO: use entity.destroy()?
-            objects.delete_entity(entity)  # type: ignore
+            if isinstance(entity, DXFEntity):
+                objects.delete_entity(entity)  # type: ignore
 
-    def add_new_dict(self, key: str, hard_owned: bool = False) -> "Dictionary":
+    def add_new_dict(self, key: str, hard_owned: bool = False) -> Dictionary:
         """Create a new sub-dictionary of type :class:`Dictionary`.
 
         Args:
@@ -360,7 +428,7 @@ class Dictionary(DXFObject):
         self.add(key, dxf_dict)
         return dxf_dict
 
-    def add_dict_var(self, key: str, value: str) -> "DictionaryVar":
+    def add_dict_var(self, key: str, value: str) -> DictionaryVar:
         """Add a new :class:`DictionaryVar`.
 
         Args:
@@ -374,7 +442,7 @@ class Dictionary(DXFObject):
         self.add(key, new_var)
         return new_var
 
-    def add_xrecord(self, key: str) -> "XRecord":
+    def add_xrecord(self, key: str) -> XRecord:
         """Add a new :class:`XRecord`.
 
         Args:
@@ -387,7 +455,7 @@ class Dictionary(DXFObject):
         self.add(key, new_xrecord)
         return new_xrecord
 
-    def set_or_add_dict_var(self, key: str, value: str) -> "DictionaryVar":
+    def set_or_add_dict_var(self, key: str, value: str) -> DictionaryVar:
         """Set or add new :class:`DictionaryVar`.
 
         Args:
@@ -420,31 +488,27 @@ class Dictionary(DXFObject):
         self.add(name, obj)
         obj.dxf.owner = self.dxf.handle
 
-    def get_required_dict(self, key: str, hard_owned=False) -> "Dictionary":
+    def get_required_dict(self, key: str, hard_owned=False) -> Dictionary:
         """Get entry `key` or create a new :class:`Dictionary`,
         if `Key` not exist.
         """
         dxf_dict = self.get(key)
         if dxf_dict is None:
             dxf_dict = self.add_new_dict(key, hard_owned=hard_owned)
-        return dxf_dict  # type: ignore
+        elif not isinstance(dxf_dict, Dictionary):
+            raise DXFStructureError(
+                f"expected a DICTIONARY entity, got {str(dxf_dict)} for key: {key}"
+            )
+        return dxf_dict
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         if not self.is_alive:
             return
-        if not self.dxf.hasattr("owner"):
-            rootdict = auditor.doc.rootdict
-            if self is rootdict:
-                self.dxf.owner = "0"
-            else:  # most likely scenario, avoids deleting required tables
-                self.dxf.owner = rootdict.dxf.handle
-
         super().audit(auditor)
-        self._check_invalid_entries(auditor)
+        self._remove_keys_to_missing_entities(auditor)
 
-    def _check_invalid_entries(self, auditor: "Auditor"):
-        trash: List[str] = []  # do not delete content while iterating
-        owner_handle = self.dxf.handle
+    def _remove_keys_to_missing_entities(self, auditor: Auditor):
+        trash: list[str] = []
         append = trash.append
         db = auditor.entitydb
         for key, entry in self._data.items():
@@ -455,14 +519,7 @@ class Dictionary(DXFObject):
                 if entry.dxf.handle not in db:
                     append(key)
                     continue
-                # valid entry object
-                if entry.dxf.owner != owner_handle:
-                    entry.dxf.owner = owner_handle
-                    auditor.fixed_error(
-                        code=AuditError.INVALID_OWNER_HANDLE,
-                        message=f'Fixed invalid owner handle in {str(entry)}',
-                    )
-            else:  # entry is destroyed
+            else:  # entity is destroyed, remove key
                 append(key)
         for key in trash:
             del self._data[key]
@@ -494,36 +551,33 @@ acdb_dict_with_default_group_codes = group_code_mapping(acdb_dict_with_default)
 @factory.register_entity
 class DictionaryWithDefault(Dictionary):
     DXFTYPE = "ACDBDICTIONARYWDFLT"
-    DXFATTRIBS = DXFAttributes(
-        base_class, acdb_dictionary, acdb_dict_with_default
-    )
+    DXFATTRIBS = DXFAttributes(base_class, acdb_dictionary, acdb_dict_with_default)
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self._default: Optional[DXFObject] = None
 
-    def _copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        super().copy_data(entity, copy_strategy=copy_strategy)
         assert isinstance(entity, DictionaryWithDefault)
         entity._default = self._default
 
-    def post_load_hook(self, doc: "Drawing") -> None:
+    def post_load_hook(self, doc: Drawing) -> None:
         # Set _default to None if default object not exist - audit() replaces
-        # a not existing default object by a place holder object.
+        # a not existing default object by a placeholder object.
         # AutoCAD ignores not existing default objects!
         self._default = doc.entitydb.get(self.dxf.default)  # type: ignore
         super().post_load_hook(doc)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
-            processor.fast_load_dxfattribs(
-                dxf, acdb_dict_with_default_group_codes, 2
-            )
+            processor.fast_load_dxfattribs(dxf, acdb_dict_with_default_group_codes, 2)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_dict_with_default.name)
         self.dxf.export_dxf_attribs(tagwriter, "default")
@@ -531,9 +585,7 @@ class DictionaryWithDefault(Dictionary):
     def __getitem__(self, key: str):
         return self.get(key)
 
-    def get(
-        self, key: str, default: Optional[DXFObject] = None
-    ) -> Optional[DXFObject]:
+    def get(self, key: str, default: Optional[DXFObject] = None) -> Optional[DXFObject]:
         # `default` argument is ignored, exist only for API compatibility,
         """Returns :class:`DXFEntity` for `key` or the predefined dictionary
         wide :attr:`dxf.default` entity if `key` does not exist or ``None``
@@ -552,11 +604,9 @@ class DictionaryWithDefault(Dictionary):
         self._default = default
         self.dxf.default = self._default.dxf.handle
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         def create_missing_default_object():
-            placeholder = self.doc.objects.add_placeholder(
-                owner=self.dxf.handle
-            )
+            placeholder = self.doc.objects.add_placeholder(owner=self.dxf.handle)
             self.set_default(placeholder)
             auditor.fixed_error(
                 code=AuditError.CREATED_MISSING_OBJECT,
@@ -627,14 +677,14 @@ class DictionaryVar(DXFObject):
         self.dxf.set("value", str(data))
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(dxf, acdb_dict_var_group_codes, 1)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_dict_var.name)
