@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2022 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 """ :class:`DXFEntity` is the super class of all DXF entities.
 
@@ -25,7 +25,8 @@ from typing import (
     TypeVar,
     Callable,
 )
-import copy
+from typing_extensions import Self
+
 import logging
 import uuid
 from ezdxf import options
@@ -40,6 +41,7 @@ from .appdata import AppData, Reactors
 from .dxfns import DXFNamespace, SubclassProcessor
 from .xdata import XData
 from .xdict import ExtensionDict
+from .copy import default_copy, CopyNotSupported
 
 if TYPE_CHECKING:
     from ezdxf.audit import Auditor
@@ -50,6 +52,7 @@ if TYPE_CHECKING:
     from ezdxf.math import Matrix44
     from ezdxf import xref
 
+
 __all__ = ["DXFEntity", "DXFTagStorage", "base_class", "SubclassProcessor"]
 logger = logging.getLogger("ezdxf")
 
@@ -300,48 +303,7 @@ class DXFEntity:
         # Do not set copy state, this is not a real copy!
         return entity
 
-    def raw_copy(self: T) -> T:
-        """Returns a raw copy of `self` but without handle, owner and reactors.
-
-        This is the first stage of the copy process, see copy() method.
-
-        (internal API)
-        """
-        entity = self.__class__()
-        doc = self.doc
-        entity.doc = doc
-        # copy and bind dxf namespace to new entity
-        entity.dxf = self.dxf.copy(entity)
-        entity.dxf.reset_handles()
-
-        xdict = self.extension_dict
-        if xdict is not None and doc is not None and xdict.is_alive:
-            # All linked DXF objects are copied and added to the OBJECTS section:
-            entity.extension_dict = xdict.copy()
-        else:
-            entity.extension_dict = None
-
-        # Do not copy reactors:
-        entity.reactors = None
-        entity.proxy_graphic = self.proxy_graphic  # immutable bytes
-
-        # if appdata contains handles, they are treated as shared resources
-        entity.appdata = copy.deepcopy(self.appdata)
-
-        # if xdata contains handles, they are treated as shared resources
-        entity.xdata = copy.deepcopy(self.xdata)
-        return entity
-
-    def copy_data(self, entity: DXFEntity) -> None:
-        """Copy entity data like vertices or attribs to the copy of the entity.
-
-        This is the second stage of the copy process, see copy() method.
-
-        (internal API)
-        """
-        pass
-
-    def copy(self: T) -> T:
+    def copy(self, copy_strategy=default_copy) -> Self:
         """Internal entity copy for usage in the same document or as virtual entity.
 
         Returns a copy of `self` but without handle, owner and reactors.
@@ -352,43 +314,16 @@ class DXFEntity:
 
         (internal API)
         """
-        clone = self.raw_copy()
-        clone.set_source_of_copy(self)
-        # DO NOT COPY DYN_SOURCE_BLOCK_REFERENCE_ATTRIBUTE.
-        # Copying an entity from a block reference, takes it out of context of
-        # this block reference!
-        self.copy_data(clone)
-        return clone
+        return copy_strategy.copy(self)
 
-    def copy_external(self: T) -> T:
-        """External entity copy for usage in another document, this copy mode requires
-        registering and mapping of resources, see also:
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
+        """Copy entity data like vertices or attribs to the copy of the entity.
 
-            - DXFEntity.register_resources()
-            - DXFEntity.map_resources()
-
-        Introduced for usage by the xref module.
+        This is the second stage of the copy process, see copy() method.
 
         (internal API)
         """
-        clone = self.raw_copy()
-        # source of copy is not required
-        self.copy_data(clone)
-        return clone
-
-    def __deepcopy__(self, memodict: Optional[dict] = None):
-        """Some entities maybe linked by more than one entity, to be safe use
-        `memodict` for bookkeeping.
-
-        (internal API)
-        """
-        memodict = memodict or {}
-        try:
-            return memodict[id(self)]
-        except KeyError:
-            copy = self.copy()
-            memodict[id(self)] = copy
-            return copy
+        pass
 
     def set_source_of_copy(self, source: Optional[DXFEntity]):
         """Set immediate source entity of a copy.
@@ -494,7 +429,7 @@ class DXFEntity:
         """
         blockref = getattr(self, DYN_SOURCE_BLOCK_REFERENCE_ATTRIBUTE, None)
         if blockref is not None and blockref.is_alive:
-            return blockref  # type: ignore
+            return blockref
         return None
 
     def set_source_block_reference(self, blockref: Insert) -> None:
@@ -685,6 +620,10 @@ class DXFEntity:
         del self.doc
         del self.dxf  # check mark for is_alive
 
+    def notify(self, message_type: int, data: Any = None) -> None:
+        """Internal messaging system.  (internal API)"""
+        pass
+
     def preprocess_export(self, tagwriter: AbstractTagWriter) -> bool:
         """Pre requirement check and pre-processing for export.
 
@@ -730,12 +669,12 @@ class DXFEntity:
         if tagwriter.dxfversion >= const.DXF2000:
             tagwriter.write_tag2(_handle_code, self.dxf.handle)
             if self.appdata:
-                self.appdata.export_dxf(tagwriter)  # type: ignore
+                self.appdata.export_dxf(tagwriter)
             if self.has_extension_dict:
                 self.extension_dict.export_dxf(tagwriter)  # type: ignore
             if self.reactors:
-                self.reactors.export_dxf(tagwriter)  # type: ignore
-            tagwriter.write_tag2(const.OWNER_CODE, self.dxf.owner)
+                self.reactors.export_dxf(tagwriter)
+            tagwriter.write_tag2(const.OWNER_CODE, self.dxf.get("owner", "0"))
         else:  # DXF R12
             if tagwriter.write_handles:
                 tagwriter.write_tag2(_handle_code, self.dxf.handle)
@@ -794,18 +733,26 @@ class DXFEntity:
             raise AttributeError("Entity has no extension dictionary.")
 
     def new_extension_dict(self) -> ExtensionDict:
-        """Create a new :class:`~ezdxf.entities.xdict.ExtensionDict` instance ."""
+        """Create a new :class:`~ezdxf.entities.xdict.ExtensionDict` instance."""
         assert self.doc is not None
         xdict = ExtensionDict.new(self.dxf.handle, self.doc)
         self.extension_dict = xdict
         return xdict
 
     def discard_extension_dict(self) -> None:
-        """Delete :class:`~ezdxf.entities.xdict.ExtensionDict` instance ."""
+        """Delete :class:`~ezdxf.entities.xdict.ExtensionDict` instance."""
         if isinstance(self.extension_dict, ExtensionDict):
             self.extension_dict.destroy()
         self.extension_dict = None
 
+    def discard_empty_extension_dict(self) -> None:
+        """Delete :class:`~ezdxf.entities.xdict.ExtensionDict` instance when empty."""
+        if (
+            isinstance(self.extension_dict, ExtensionDict)
+            and len(self.extension_dict) == 0
+        ):
+            self.discard_extension_dict()
+
     def has_app_data(self, appid: str) -> bool:
         """Returns ``True`` if application defined data for `appid` exist."""
         if self.appdata:
@@ -989,7 +936,7 @@ class DXFEntity:
                 for tag in tags.get_hard_owner_handles():
                     registry.add_handle(tag.value)
 
-    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
         """Translate resources from self to the copied entity."""
 
         def map_xdata_resources():
@@ -1031,12 +978,12 @@ class DXFTagStorage(DXFEntity):
         self.xtags = ExtendedTags()
         self.embedded_objects: Optional[list[Tags]] = None
 
-    def raw_copy(self: T) -> T:
-        raise const.DXFTypeError(
-            f"Cloning of tag storage {self.dxftype()} not supported."
+    def copy(self, copy_strategy=default_copy) -> Self:
+        raise CopyNotSupported(
+            f"Copying of tag storage {self.dxftype()} not supported."
         )
 
-    def transform(self, m: Matrix44) -> DXFGraphic:
+    def transform(self, m: Matrix44) -> Self:
         raise NotImplementedError("cannot transform DXF tag storage")
 
     @property
@@ -1135,7 +1082,6 @@ class DXFTagStorage(DXFEntity):
             for e in ProxyGraphic(self.proxy_graphic, self.doc).virtual_entities():
                 e.set_source_of_copy(self)
                 yield e
-        return []
 
     def virtual_entities(self) -> Iterator[DXFGraphic]:
         """Yields proxy graphic as "virtual" entities."""
