@@ -1,7 +1,9 @@
-# Copyright (c) 2019-2022 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import TYPE_CHECKING, Optional, Iterable, Any
+from typing_extensions import Self, TypeGuard
+
 from ezdxf.entities import factory
 from ezdxf import options
 from ezdxf.lldxf import validator
@@ -177,7 +179,6 @@ class DXFGraphic(DXFEntity):
     """Common base class for all graphic entities, a subclass of
     :class:`~ezdxf.entities.dxfentity.DXFEntity`. These entities resides in
     entity spaces like modelspace, paperspace or block.
-
     """
 
     DXFTYPE = "DXFGFX"
@@ -214,8 +215,9 @@ class DXFGraphic(DXFEntity):
         processor.fast_load_dxfattribs(dxf, acdb_entity_group_codes, 1)
         return dxf
 
-    def post_new_hook(self):
-        """Post-processing and integrity validation after entity creation
+    def post_new_hook(self) -> None:
+        """Post-processing and integrity validation after entity creation.
+
         (internal API)
         """
         if self.doc:
@@ -225,20 +227,26 @@ class DXFGraphic(DXFEntity):
                 )
 
     @property
-    def rgb(self) -> Optional[clr.RGB]:
-        """Returns RGB true color as (r, g, b) tuple or None if true_color is
-        not set.
-        """
+    def rgb(self) -> tuple[int, int, int] | None:
+        """Returns RGB true color as (r, g, b) tuple or None if true_color is not set."""
         if self.dxf.hasattr("true_color"):
             return clr.int2rgb(self.dxf.get("true_color"))
-        else:
-            return None
+        return None
 
     @rgb.setter
-    def rgb(self, rgb: clr.RGB) -> None:
-        """Set RGB true color as (r, g , b) tuple e.g. (12, 34, 56)."""
+    def rgb(self, rgb: clr.RGB | tuple[int, int, int]) -> None:
+        """Set RGB true color as (r, g , b) tuple e.g. (12, 34, 56).
+
+        Raises:
+            TypeError: input value `rgb` has invalid type
+        """
         self.dxf.set("true_color", clr.rgb2int(rgb))
 
+    @rgb.deleter
+    def rgb(self) -> None:
+        """Delete RGB true color value."""
+        self.dxf.discard("true_color")
+
     @property
     def transparency(self) -> float:
         """Get transparency as float value between 0 and 1, 0 is opaque and 1
@@ -272,7 +280,6 @@ class DXFGraphic(DXFEntity):
     def graphic_properties(self) -> dict:
         """Returns the important common properties layer, color, linetype,
         lineweight, ltscale, true_color and color_name as `dxfattribs` dict.
-
         """
         attribs = dict()
         for key in GRAPHIC_PROPERTIES:
@@ -284,7 +291,6 @@ class DXFGraphic(DXFEntity):
         """Returns object coordinate system (:ref:`ocs`) for 2D entities like
         :class:`Text` or :class:`Circle`, returns a pass-through OCS for
         entities without OCS support.
-
         """
         # extrusion is only defined for 2D entities like Text, Circle, ...
         if self.dxf.is_supported("extrusion"):
@@ -316,7 +322,7 @@ class DXFGraphic(DXFEntity):
         self.export_acdb_entity(tagwriter)
         # XDATA and embedded objects export is also done by the parent class.
 
-    def export_acdb_entity(self, tagwriter: AbstractTagWriter):
+    def export_acdb_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export subclass 'AcDbEntity' as DXF tags. (internal API)"""
         # Full control over tag order and YES, sometimes order matters
         not_r12 = tagwriter.dxfversion > DXF12
@@ -333,6 +339,7 @@ class DXFGraphic(DXFEntity):
                 "color",
                 "lineweight",
                 "ltscale",
+                "invisible",
                 "true_color",
                 "color_name",
                 "transparency",
@@ -374,7 +381,6 @@ class DXFGraphic(DXFEntity):
         It is more efficient to call the
         :meth:`~ezdxf.layouts.BaseLayout.unlink_entity` method of the associated
         layout, especially if you have to unlink more than one entity.
-
         """
         if not self.is_alive:
             raise TypeError("Can not unlink destroyed entity.")
@@ -403,7 +409,6 @@ class DXFGraphic(DXFEntity):
 
         Raises:
             DXFStructureError: for moving between different DXF drawings
-
         """
         if source is None:
             source = self.get_layout()
@@ -411,7 +416,7 @@ class DXFGraphic(DXFEntity):
                 raise const.DXFValueError("Source layout for entity not found.")
         source.move_to_layout(self, layout)
 
-    def copy_to_layout(self, layout: BaseLayout) -> DXFEntity:
+    def copy_to_layout(self, layout: BaseLayout) -> Self:
         """
         Copy entity to another `layout`, returns new created entity as
         :class:`DXFEntity` object. Copying between different DXF drawings is
@@ -422,7 +427,6 @@ class DXFGraphic(DXFEntity):
 
         Raises:
             DXFStructureError: for copying between different DXF drawings
-
         """
         if self.doc != layout.doc:
             raise const.DXFStructureError(
@@ -444,7 +448,6 @@ class DXFGraphic(DXFEntity):
                 auditor.trash(entity)
 
             to delete invalid entities after auditing automatically.
-
         """
         assert self.doc is auditor.doc, "Auditor for different DXF document."
         if not self.is_alive:
@@ -466,13 +469,11 @@ class DXFGraphic(DXFEntity):
         if dxf.hasattr("transparency"):
             auditor.check_transparency(self)
 
-    def transform(self, m: Matrix44) -> DXFGraphic:
-        """Inplace transformation interface, returns `self`
-        (floating interface).
+    def transform(self, m: Matrix44) -> Self:
+        """Inplace transformation interface, returns `self` (floating interface).
 
         Args:
              m: 4x4 transformation matrix (:class:`ezdxf.math.Matrix44`)
-
         """
         raise NotImplementedError()
 
@@ -486,68 +487,61 @@ class DXFGraphic(DXFEntity):
         """Check if post transform call is required."""
         return self.xdata is not None
 
-    def translate(self, dx: float, dy: float, dz: float) -> DXFGraphic:
+    def translate(self, dx: float, dy: float, dz: float) -> Self:
         """Translate entity inplace about `dx` in x-axis, `dy` in y-axis and
         `dz` in z-axis, returns `self` (floating interface).
 
         Basic implementation uses the :meth:`transform` interface, subclasses
         may have faster implementations.
-
         """
         return self.transform(Matrix44.translate(dx, dy, dz))
 
-    def scale(self, sx: float, sy: float, sz: float) -> DXFGraphic:
+    def scale(self, sx: float, sy: float, sz: float) -> Self:
         """Scale entity inplace about `dx` in x-axis, `dy` in y-axis and `dz`
         in z-axis, returns `self` (floating interface).
-
         """
         return self.transform(Matrix44.scale(sx, sy, sz))
 
-    def scale_uniform(self, s: float) -> DXFGraphic:
+    def scale_uniform(self, s: float) -> Self:
         """Scale entity inplace uniform about `s` in x-axis, y-axis and z-axis,
         returns `self` (floating interface).
-
         """
         return self.transform(Matrix44.scale(s))
 
-    def rotate_axis(self, axis: UVec, angle: float) -> DXFGraphic:
+    def rotate_axis(self, axis: UVec, angle: float) -> Self:
         """Rotate entity inplace about vector `axis`, returns `self`
         (floating interface).
 
         Args:
             axis: rotation axis as tuple or :class:`Vec3`
             angle: rotation angle in radians
-
         """
         return self.transform(Matrix44.axis_rotate(axis, angle))
 
-    def rotate_x(self, angle: float) -> DXFGraphic:
+    def rotate_x(self, angle: float) -> Self:
         """Rotate entity inplace about x-axis, returns `self`
         (floating interface).
 
         Args:
             angle: rotation angle in radians
-
         """
         return self.transform(Matrix44.x_rotate(angle))
 
-    def rotate_y(self, angle: float) -> DXFGraphic:
+    def rotate_y(self, angle: float) -> Self:
         """Rotate entity inplace about y-axis, returns `self`
         (floating interface).
 
         Args:
             angle: rotation angle in radians
-
         """
         return self.transform(Matrix44.y_rotate(angle))
 
-    def rotate_z(self, angle: float) -> DXFGraphic:
+    def rotate_z(self, angle: float) -> Self:
         """Rotate entity inplace about z-axis, returns `self`
         (floating interface).
 
         Args:
             angle: rotation angle in radians
-
         """
         return self.transform(Matrix44.z_rotate(angle))
 
@@ -611,7 +605,7 @@ class DXFGraphic(DXFEntity):
         self.dxf.discard("plotstyle_enum")
         self.dxf.discard("plotstyle_handle")
 
-    def _new_compound_entity(self, type_: str, dxfattribs) -> DXFGraphic:
+    def _new_compound_entity(self, type_: str, dxfattribs) -> Self:
         """Create and bind  new entity with same layout settings as `self`.
 
         Used by INSERT & POLYLINE to create appended DXF entities, don't use it
@@ -645,7 +639,7 @@ class DXFGraphic(DXFEntity):
         # - visualstyle_handle
         # - plotstyle_handle
 
-    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
         """Translate resources from self to the copied entity."""
         super().map_resources(clone, mapping)
         clone.dxf.layer = mapping.get_layer(self.dxf.layer)
@@ -687,7 +681,6 @@ def add_entity(entity: DXFGraphic, layout: BaseLayout) -> None:
 def replace_entity(source: DXFGraphic, target: DXFGraphic, layout: BaseLayout) -> None:
     """Add `target` entity to the entity database and to the given `layout`
     and replace the `source` entity by the `target` entity.
-
     """
     assert target.dxf.handle is None
     assert layout is not None
@@ -701,7 +694,7 @@ def replace_entity(source: DXFGraphic, target: DXFGraphic, layout: BaseLayout) -
         source.destroy()
 
 
-def is_graphic_entity(entity: DXFEntity) -> bool:
+def is_graphic_entity(entity: DXFEntity) -> TypeGuard[DXFGraphic]:
     """Returns ``True`` if the `entity` has a graphical representations and
     can reside in the model space, a paper space or a block layout,
     otherwise the entity is a table or class entry or a DXF object from the
