@@ -86,6 +86,15 @@ class DXFNamespace:
     def __deepcopy__(self, memodict: Optional[dict] = None):
         return self.copy(self._entity)
 
+    def __getstate__(self) -> object:
+        return self.__dict__
+
+    def __setstate__(self, state: object) -> None:
+        if not isinstance(state, dict):
+            raise TypeError(f"invalid state: {type(state).__name__}")
+        # bypass __setattr__
+        object.__setattr__(self, "__dict__", state)
+
     def reset_handles(self):
         """Reset handle and owner to None."""
         self.__dict__["handle"] = None
@@ -326,57 +335,95 @@ class DXFNamespace:
     def _export_dxf_attribute_optional(
         self, tagwriter: AbstractTagWriter, name: str
     ) -> None:
-        """Exports DXF attribute `name` by `tagwriter`. Optional tags are only
-        written if different to default value.
+        """Exports DXF attribute `name` by `tagwriter`.
 
-        Args:
-            tagwriter: tag writer object
-            name: DXF attribute name
+        Optional tags are only written if they differ from the default value.
 
         """
-        export_dxf_version = tagwriter.dxfversion
-        not_force_optional = not tagwriter.force_optional
         attrib: Optional[DXFAttr] = self.dxfattribs.get(name)
-
-        if attrib:
-            optional = attrib.optional
-            default = attrib.default
-            value = self.get(name, None)
-            # Force default value e.g. layer
-            if value is None and not optional:
-                # Default value could be None
-                value = default
-
-                # Do not export None values
-            if (value is not None) and (
-                export_dxf_version >= attrib.dxfversion
-            ):
-                # Do not write explicit optional attribs if equal to default
-                # value
-                if (
-                    optional
-                    and not_force_optional
-                    and default is not None
-                    and default == value
-                ):
-                    return
-                    # Just export x, y for 2D points, if value is a 3D point
-                if attrib.xtype == XType.point2d and len(value) > 2:
-                    try:  # Vec3
-                        value = (value.x, value.y)
-                    except AttributeError:
-                        value = value[:2]
-
-                if isinstance(value, str):
-                    assert "\n" not in value, "line break '\\n' not allowed"
-                    assert "\r" not in value, "line break '\\r' not allowed"
-                tag = dxftag(attrib.code, value)
-                tagwriter.write_tag(tag)
-        else:
+        if attrib is None:
             raise const.DXFAttributeError(
                 ERR_INVALID_DXF_ATTRIB.format(name, self.dxftype)
             )
 
+        optional = attrib.optional
+        default = attrib.default
+        value = self.get(name, None)
+
+        # Force default value e.g. layer
+        if value is None and not optional:
+            # default value can be None!
+            value = default
+
+        if value is None:
+            logger.debug(
+                f"DXF attribute '{name}' not written because its a None value."
+            )
+            return
+
+        # Do not write explicit optional attribs if equal to the default value
+        if (
+            optional
+            and (not tagwriter.force_optional)
+            and default is not None
+            and default == value
+        ):
+            return
+        _export_group_codes(tagwriter, attrib, value)
+
+    def export_dxf_attribute_if_exists(
+        self, tagwriter: AbstractTagWriter, name: str
+    ) -> None:
+        """Exports DXF attribute `name` by `tagwriter` if exists.
+
+        If the attribute exists, and it's not None it will be written, the optional-flag
+        is ignored.
+
+        No default value will be written if the attribute doesn't exist!
+        This method can not be used for attributes that are required (e.g. layer)!
+
+        """
+        if not self.hasattr(name):
+            return
+
+        attrib: Optional[DXFAttr] = self.dxfattribs.get(name)
+        assert (
+            attrib is not None
+        ), f"existing DXF attribute '{name}' has no definition class - internal error"
+
+        value = self.get(name)
+        if value is None:
+            logger.debug(
+                f"DXF attribute '{name}' not written because its a None value."
+            )
+            return
+        _export_group_codes(tagwriter, attrib, value)
+
+
+def _export_group_codes(
+    tagwriter: AbstractTagWriter, attrib: DXFAttr, value: Any
+) -> None:
+    assert attrib is not None
+    assert value is not None
+
+    # Do write the attribute if the export DXF version is lower than the minimal
+    # required DXF version for the attribute.
+    if tagwriter.dxfversion < attrib.dxfversion:
+        return
+
+    # For explicit 2D points export only x- and y-coordinates.
+    if attrib.xtype == XType.point2d and len(value) > 2:
+        try:  # Vec3, Vec2
+            value = (value.x, value.y)
+        except AttributeError:
+            value = value[:2]
+
+    if isinstance(value, str):
+        assert "\n" not in value, "line break '\\n' not allowed"
+        assert "\r" not in value, "line break '\\r' not allowed"
+    tag = dxftag(attrib.code, value)
+    tagwriter.write_tag(tag)
+
 
 BASE_CLASS_CODES = {0, 5, 102, 330}
 
@@ -396,9 +443,7 @@ class SubclassProcessor:
         # AcDbEntity.
         # Exception: CLASS has also only one subclass and no subclass marker,
         # handled as DXF R12 entity
-        self.r12: bool = (dxfversion == const.DXF12) or (
-            len(self.subclasses) == 1
-        )
+        self.r12: bool = (dxfversion == const.DXF12) or (len(self.subclasses) == 1)
         self.name: str = tags.dxftype()
         self.handle: str
         try:
@@ -421,9 +466,7 @@ class SubclassProcessor:
                 entity = ""
                 if handle:
                     entity = f" in entity #{handle}"
-                logger.info(
-                    f"ignored {repr(tag)} in subclass {subclass}" + entity
-                )
+                logger.info(f"ignored {repr(tag)} in subclass {subclass}" + entity)
 
     def find_subclass(self, name: str) -> Optional[Tags]:
         for subclass in self.subclasses:
@@ -533,9 +576,7 @@ class SubclassProcessor:
         if len(unprocessed_tags) and log:
             # First tag is the subclass specifier (100, "AcDb...")
             name = tags[0].value
-            self.log_unprocessed_tags(
-                tags, subclass=name, handle=dxf.get("handle")
-            )
+            self.log_unprocessed_tags(tags, subclass=name, handle=dxf.get("handle"))
         return unprocessed_tags
 
     def append_base_class_to_acdb_entity(self) -> None:
@@ -554,15 +595,13 @@ class SubclassProcessor:
         acdb_entity_tags = self.subclasses[1]
         if acdb_entity_tags[0] == (100, "AcDbEntity"):
             acdb_entity_tags.extend(
-                tag
-                for tag in self.subclasses[0]
-                if tag.code not in BASE_CLASS_CODES
+                tag for tag in self.subclasses[0] if tag.code not in BASE_CLASS_CODES
             )
 
     def simple_dxfattribs_loader(
         self, dxf: DXFNamespace, group_code_mapping: dict[int, str]
     ) -> None:
-        # Most tests in text suite 201 for the POINT entity
+        # tested in test suite 201 for the POINT entity
         """Load DXF attributes from all subclasses into the DXF namespace.
 
         Can not handle same group codes in different subclasses, does not remove
@@ -609,9 +648,7 @@ class SubclassProcessor:
         for tag in tags:
             name = get_attrib_name(tag.code)
             if isinstance(name, str) and not name.startswith("*"):
-                unprotected_set_attrib(
-                    name, cast_value(tag.code, tag.value)  # type: ignore
-                )
+                unprotected_set_attrib(name, cast_value(tag.code, tag.value))
 
 
 GRAPHIC_ATTRIBUTES_TO_RECOVER = {
