@@ -1,6 +1,7 @@
 # Copyright (c) 2019-2022 Manfred Moitzi
 # License: MIT License
-from typing import TYPE_CHECKING, Iterable
+from __future__ import annotations
+from typing import TYPE_CHECKING, Iterable, Optional
 import math
 from ezdxf.audit import AuditError
 from ezdxf.lldxf import validator
@@ -35,24 +36,33 @@ from .dxfgfx import (
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, DXFNamespace, Spline, Auditor
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.entities import DXFNamespace, Spline
+    from ezdxf.audit import Auditor
+
 
 __all__ = ["Ellipse"]
 
-MIN_RATIO = 1e-6
-MAX_RATIO = 1.0
+MIN_RATIO = 1e-10  # tested with DWG TrueView 2022
+MAX_RATIO = 1.0  # tested with DWG TrueView 2022
+TOL = 1e-9  # arbitrary choice
 
 
 def is_valid_ratio(ratio: float) -> bool:
-    return MIN_RATIO <= abs(ratio) <= MAX_RATIO
+    """Check if axis-ratio is in valid range, takes an upper bound tolerance into
+    account for floating point imprecision.
+    """
+    return MIN_RATIO <= abs(ratio) < (MAX_RATIO + TOL)
 
 
-def fix_ratio(ratio: float) -> float:
+def clamp_axis_ratio(ratio: float) -> float:
+    """Clamp axis-ratio into valid range and remove possible floating point imprecision.
+    """
     sign = -1 if ratio < 0 else +1
     ratio = abs(ratio)
     if ratio < MIN_RATIO:
         return MIN_RATIO * sign
-    elif ratio > MAX_RATIO:
+    if ratio > MAX_RATIO:
         return MAX_RATIO * sign
     return ratio * sign
 
@@ -78,13 +88,13 @@ acdb_ellipse = DefSubclass(
             validator=validator.is_not_null_vector,
             fixer=RETURN_DEFAULT,
         ),
-        # Ratio has to be in the range from 1e-6 to 1, but could be negative:
+        # Ratio has to be in the range: -1.0 ... -1e-10 and +1e-10 ... +1.0:
         "ratio": DXFAttr(
-            40, default=MAX_RATIO, validator=is_valid_ratio, fixer=fix_ratio
+            40, default=MAX_RATIO, validator=is_valid_ratio, fixer=clamp_axis_ratio
         ),
         # Start of ellipse, this value is 0.0 for a full ellipse:
         "start_param": DXFAttr(41, default=0),
-        # End of ellipse, this value is 2*pi for a full ellipse:
+        # End of ellipse, this value is 2π for a full ellipse:
         "end_param": DXFAttr(42, default=math.tau),
     },
 )
@@ -105,8 +115,8 @@ class Ellipse(DXFGraphic):
     MIN_DXF_VERSION_FOR_EXPORT = DXF2000
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         """Loading interface. (internal API)"""
         # bypass DXFGraphic, loading proxy graphic is skipped!
         dxf = super(DXFGraphic, self).load_dxf_attribs(processor)
@@ -114,12 +124,16 @@ class Ellipse(DXFGraphic):
             processor.simple_dxfattribs_loader(dxf, merged_ellipse_group_codes)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_ellipse.name)
-
-        assert is_valid_ratio(self.dxf.ratio)
+        # is_valid_ratio() takes floating point imprecision on the upper bound of
+        # +/- 1.0 into account:
+        assert is_valid_ratio(
+            self.dxf.ratio
+        ), f"axis-ratio out of range [{MIN_RATIO}, {MAX_RATIO}]"
+        self.dxf.ratio = clamp_axis_ratio(self.dxf.ratio)
         self.dxf.export_dxf_attribs(
             tagwriter,
             [
@@ -135,16 +149,14 @@ class Ellipse(DXFGraphic):
     @property
     def minor_axis(self) -> Vec3:
         dxf = self.dxf
-        return ellipse.minor_axis(
-            Vec3(dxf.major_axis), Vec3(dxf.extrusion), dxf.ratio
-        )
+        return ellipse.minor_axis(Vec3(dxf.major_axis), Vec3(dxf.extrusion), dxf.ratio)
 
     @property
-    def start_point(self) -> "Vec3":
+    def start_point(self) -> Vec3:
         return list(self.vertices([self.dxf.start_param]))[0]
 
     @property
-    def end_point(self) -> "Vec3":
+    def end_point(self) -> Vec3:
         return list(self.vertices([self.dxf.end_param]))[0]
 
     def construction_tool(self) -> ConstructionEllipse:
@@ -159,7 +171,7 @@ class Ellipse(DXFGraphic):
             dxf.end_param,
         )
 
-    def apply_construction_tool(self, e: ConstructionEllipse) -> "Ellipse":
+    def apply_construction_tool(self, e: ConstructionEllipse) -> Ellipse:
         """Set ELLIPSE data from construction tool
         :class:`ezdxf.math.ConstructionEllipse`.
 
@@ -168,10 +180,10 @@ class Ellipse(DXFGraphic):
         return self  # floating interface
 
     def params(self, num: int) -> Iterable[float]:
-        """Returns `num` params from start- to end param in counter
-        clockwise order.
+        """Returns `num` params from start- to end param in counter-clockwise
+        order.
 
-        All params are normalized in the range from [0, 2pi).
+        All params are normalized in the range [0, 2π).
 
         """
         start = self.dxf.start_param % math.tau
@@ -182,8 +194,8 @@ class Ellipse(DXFGraphic):
         """Yields vertices on ellipse for iterable `params` in WCS.
 
         Args:
-            params: param values in the range from ``0`` to ``2*pi`` in radians,
-                param goes counter clockwise around the extrusion vector,
+            params: param values in the range from 0 to 2π in radians,
+                param goes counter-clockwise around the extrusion vector,
                 major_axis = local x-axis = 0 rad.
 
         """
@@ -193,16 +205,14 @@ class Ellipse(DXFGraphic):
         """Adaptive recursive flattening. The argument `segments` is the
         minimum count of approximation segments, if the distance from the center
         of the approximation segment to the curve is bigger than `distance` the
-        segment will be subdivided. Returns a closed polygon for a full ellipse:
-        start vertex == end vertex.
+        segment will be subdivided. Returns a closed polygon for a full ellipse
+        where the start vertex has the same value as the end vertex.
 
         Args:
             distance: maximum distance from the projected curve point onto the
                 segment chord.
             segments: minimum segment count
 
-        .. versionadded:: 0.15
-
         """
         return self.construction_tool().flattening(distance, segments)
 
@@ -213,14 +223,14 @@ class Ellipse(DXFGraphic):
         self.update_dxf_attribs(e.dxfattribs())
 
     @classmethod
-    def from_arc(cls, entity: "DXFGraphic") -> "Ellipse":
-        """Create a new ELLIPSE entity from ARC or CIRCLE entity.
+    def from_arc(cls, entity: DXFGraphic) -> Ellipse:
+        """Create a new virtual ELLIPSE entity from an ARC or a CIRCLE entity.
 
-        The new SPLINE entity has no owner, no handle, is not stored in
-        the entity database nor assigned to any layout!
+        The new entity has no owner, no handle, is not stored in the entity database nor
+        assigned to any layout!
 
         """
-        assert entity.dxftype() in {"ARC", "CIRCLE"}
+        assert entity.dxftype() in {"ARC", "CIRCLE"}, "ARC or CIRCLE entity required"
         attribs = entity.dxfattribs(drop={"owner", "handle", "thickness"})
         e = ellipse.ConstructionEllipse.from_arc(
             center=attribs.get("center", NULLVEC),
@@ -231,9 +241,9 @@ class Ellipse(DXFGraphic):
             end_angle=attribs.pop("end_angle", 360),
         )
         attribs.update(e.dxfattribs())
-        return Ellipse.new(dxfattribs=attribs, doc=entity.doc)
+        return cls.new(dxfattribs=attribs, doc=entity.doc)
 
-    def transform(self, m: Matrix44) -> "Ellipse":
+    def transform(self, m: Matrix44) -> Ellipse:
         """Transform the ELLIPSE entity by transformation matrix `m` inplace."""
         e = self.construction_tool()
         e.transform(m)
@@ -241,7 +251,7 @@ class Ellipse(DXFGraphic):
         self.post_transform(m)
         return self
 
-    def translate(self, dx: float, dy: float, dz: float) -> "Ellipse":
+    def translate(self, dx: float, dy: float, dz: float) -> Ellipse:
         """Optimized ELLIPSE translation about `dx` in x-axis, `dy` in y-axis
         and `dz` in z-axis, returns `self` (floating interface).
 
@@ -252,7 +262,7 @@ class Ellipse(DXFGraphic):
             self.post_transform(Matrix44.translate(dx, dy, dz))
         return self
 
-    def to_spline(self, replace=True) -> "Spline":
+    def to_spline(self, replace=True) -> Spline:
         """Convert ELLIPSE to a :class:`~ezdxf.entities.Spline` entity.
 
         Adds the new SPLINE entity to the entity database and to the
@@ -278,7 +288,7 @@ class Ellipse(DXFGraphic):
         # different way!
         return OCS()
 
-    def audit(self, auditor: "Auditor") -> None:
+    def audit(self, auditor: Auditor) -> None:
         if not self.is_alive:
             return
         super().audit(auditor)
@@ -291,15 +301,20 @@ class Ellipse(DXFGraphic):
                 message=f"Removed {entity} with invalid major axis: (0, 0, 0).",
             )
             return
-        if abs(self.dxf.ratio) > MAX_RATIO:
+        axis_ratio = self.dxf.ratio
+        if is_valid_ratio(axis_ratio):
+            # remove possible floating point imprecision:
+            self.dxf.ratio = clamp_axis_ratio(axis_ratio)
+            return
+        if abs(axis_ratio) > MAX_RATIO:
             self.swap_axis()
             auditor.fixed_error(
                 code=AuditError.INVALID_ELLIPSE_RATIO,
-                message=f"Fixed invalid ratio in {entity} by swapping axis.",
+                message=f"Fixed invalid axis-ratio in {entity} by swapping axis.",
             )
-        elif abs(self.dxf.ratio) < MIN_RATIO:
-            self.dxf.ratio = fix_ratio(self.dxf.ratio)
+        elif abs(axis_ratio) < MIN_RATIO:
+            self.dxf.ratio = clamp_axis_ratio(axis_ratio)
             auditor.fixed_error(
                 code=AuditError.INVALID_ELLIPSE_RATIO,
-                message=f"Fixed invalid ratio in {entity}, set to {MIN_RATIO}.",
+                message=f"Fixed invalid axis-ratio in {entity}, set to {MIN_RATIO}.",
             )
