@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2023 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
@@ -10,7 +10,10 @@ from typing import (
     Optional,
     Callable,
 )
+from typing_extensions import Self
 import math
+import logging
+
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.attributes import (
     DXFAttr,
@@ -68,6 +71,7 @@ if TYPE_CHECKING:
 
 __all__ = ["Insert"]
 
+logger = logging.getLogger("ezdxf")
 ABS_TOL = 1e-9
 
 # DXF files as XREF:
@@ -234,7 +238,7 @@ class Insert(LinkedEntities):
         super().register_resources(registry)
         registry.add_block_name(self.dxf.name)
 
-    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
         # The attached ATTRIB entities are mapped by the parent class LinkedEntities
         super().map_resources(clone, mapping)
         clone.dxf.name = mapping.get_block_name(self.dxf.name)
@@ -528,7 +532,7 @@ class Insert(LinkedEntities):
             insert -= m.transform_direction(block_layout.block.dxf.base_point)  # type: ignore
 
         # set translation
-        m.set_row(3, insert.xyz)  # type: ignore
+        m.set_row(3, insert.xyz)
         return m
 
     def ucs(self):
@@ -627,6 +631,10 @@ class Insert(LinkedEntities):
         if the property :attr:`Insert.mcount` > 1, use the :meth:`Insert.multi_insert`
         method to resolve the MINSERT entity into multiple INSERT entities.
 
+        This method does not apply the clipping path created by the XCLIP command.
+        The method returns all entities and ignores the clipping path polygon and no
+        entity is clipped.
+
         The `skipped_entity_callback()` will be called for all entities which are not
         processed, signature:
         :code:`skipped_entity_callback(entity: DXFEntity, reason: str)`,
@@ -713,18 +721,39 @@ class Insert(LinkedEntities):
                 pairs
 
         """
-
         def unpack(dxfattribs) -> tuple[str, str, UVec]:
             tag = dxfattribs.pop("tag")
-            text = values.get(tag, "")
+            text = values.get(tag, None)
+            if text is None:  # get default value from ATTDEF
+                text = dxfattribs.get("text", "")
             location = dxfattribs.pop("insert")
             return tag, text, location
 
         def autofill() -> None:
             for attdef in block_layout.attdefs():  # type: ignore
                 dxfattribs = attdef.dxfattribs(drop={"prompt", "handle"})
+
+                # Caution! Some mandatory values may not exist!
+                # These are DXF structure errors, but loaded DXF files may have errors!
+                if "tag" not in dxfattribs:
+                    # ATTRIB without "tag" makes no sense!
+                    logger.warning(
+                        f"Skipping {str(attdef)}: missing mandatory 'tag' attribute"
+                    )
+                    continue
+                if "insert" not in dxfattribs:
+                    # Don't know where to place the ATTRIB entity.
+                    logger.warning(
+                        f"Skipping {str(attdef)}: missing mandatory 'insert' attribute"
+                    )
+                    continue
+
                 tag, text, location = unpack(dxfattribs)
                 attrib = self.add_attrib(tag, text, location, dxfattribs)
+                if attdef.has_embedded_mtext_entity:
+                    mtext = attdef.virtual_mtext_entity()
+                    mtext.text = text
+                    attrib.embed_mtext(mtext)
                 attrib.transform(m)
 
         block_layout = self.block()
