@@ -1,6 +1,8 @@
-#  Copyright (c) 2020-2021, Manfred Moitzi
+#  Copyright (c) 2020-2024, Manfred Moitzi
 #  License: MIT License
-from typing import TYPE_CHECKING
+from __future__ import annotations
+from typing import TYPE_CHECKING, Optional
+from typing_extensions import Self
 from ezdxf.lldxf import validator
 from ezdxf.lldxf.const import SUBCLASS_MARKER
 from ezdxf.lldxf.attributes import (
@@ -12,12 +14,14 @@ from ezdxf.lldxf.attributes import (
     group_code_mapping,
 )
 from ezdxf.math import Vec3, Vec2, NULLVEC, X_AXIS, Y_AXIS
-from .dxfentity import base_class, SubclassProcessor
+from .dxfentity import base_class, SubclassProcessor, DXFEntity
 from .dxfobj import DXFObject
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, DXFNamespace
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf.entities.dxfns import DXFNamespace
+    from ezdxf import xref
 
 __all__ = ["PlotSettings", "DXFLayout"]
 
@@ -26,7 +30,9 @@ acdb_plot_settings = DefSubclass(
     {
         # acdb_plot_settings is also part of LAYOUT and LAYOUT has a 'name' attribute
         "page_setup_name": DXFAttr(1, default=""),
-        "plot_configuration_file": DXFAttr(2, default="Adobe PDF"),
+
+        # An optional empty string selects the default printer/plotter
+        "plot_configuration_file": DXFAttr(2, default="", optional=True),
         "paper_size": DXFAttr(4, default="A3"),
         "plot_view_name": DXFAttr(6, default=""),
         "left_margin": DXFAttr(40, default=7.5),  # in mm
@@ -46,20 +52,21 @@ acdb_plot_settings = DefSubclass(
         # Denominator of custom print scale: drawing units:
         "scale_denominator": DXFAttr(143, default=1.0),
         # Plot layout flags:
-        # 1 = Plot Viewport Borders
-        # 2 = Show Plot Styles
-        # 4 = Plot Centered
-        # 8 = Plot Hidden
-        # 16 = Use Standard Scale
-        # 32 = Plot Plot Styles
-        # 64 = Scale Lineweights
-        # 128 = Print Lineweights
-        # 512 = Draw Viewports First
-        # 1024 = Model Type
-        # 2048 = Update Paper
-        # 4096 = Zoom To Paper On Update
-        # 8192 = Initializing
-        # 16384 = Prev PlotInit
+        # 1 = plot viewport borders
+        # 2 = show plot-styles
+        # 4 = plot centered
+        # 8 = plot hidden == hide paperspace entities?
+        # 16 = use standard scale
+        # 32 = plot with plot-styles
+        # 64 = scale lineweights
+        # 128 = plot entity lineweights
+        # 512 = draw viewports first
+        # 1024 = model type
+        # 2048 = update paper
+        # 4096 = zoom to paper on update
+        # 8192 = initializing
+        # 16384 = prev plot-init
+        # the "Plot transparencies" option is stored in the XDATA section
         "plot_layout_flags": DXFAttr(70, default=688),
         # Plot paper units:
         # 0 = Plot in inches
@@ -78,7 +85,7 @@ acdb_plot_settings = DefSubclass(
         # 3 = 90 degrees clockwise
         "plot_rotation": DXFAttr(
             73,
-            default=1,
+            default=0,
             validator=validator.is_in_integer_range(0, 4),
             fixer=RETURN_DEFAULT,
         ),
@@ -185,6 +192,13 @@ acdb_plot_settings = DefSubclass(
 )
 acdb_plot_settings_group_codes = group_code_mapping(acdb_plot_settings)
 
+# The "Plot transparencies" option is stored in the XDATA section of the
+# LAYOUT entity:
+# 1001
+# PLOTTRANSPARENCY
+# 1071
+# 1
+
 
 @register_entity
 class PlotSettings(DXFObject):
@@ -192,16 +206,14 @@ class PlotSettings(DXFObject):
     DXFATTRIBS = DXFAttributes(base_class, acdb_plot_settings)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
-            processor.fast_load_dxfattribs(
-                dxf, acdb_plot_settings_group_codes, 1
-            )
+            processor.fast_load_dxfattribs(dxf, acdb_plot_settings_group_codes, 1)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags."""
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_plot_settings.name)
@@ -242,6 +254,18 @@ class PlotSettings(DXFObject):
             ],
         )
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        super().register_resources(registry)
+        registry.add_handle(self.dxf.get("shade_plot_handle"))
+
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
+        super().map_resources(clone, mapping)
+        shade_plot_handle = self.dxf.get("shade_plot_handle")
+        if shade_plot_handle and shade_plot_handle != "0":
+            clone.dxf.shade_plot_handle = mapping.get_handle(shade_plot_handle)
+        else:
+            clone.dxf.discard("shade_plot_handle")
+
 
 acdb_layout = DefSubclass(
     "AcDbLayout",
@@ -264,13 +288,9 @@ acdb_layout = DefSubclass(
         # Insertion base point for this layout:
         "insert_base": DXFAttr(12, xtype=XType.point3d, default=NULLVEC),
         # Minimum extents for this layout:
-        "extmin": DXFAttr(
-            14, xtype=XType.point3d, default=Vec3(1e20, 1e20, 1e20)
-        ),
+        "extmin": DXFAttr(14, xtype=XType.point3d, default=Vec3(1e20, 1e20, 1e20)),
         # Maximum extents for this layout:
-        "extmax": DXFAttr(
-            15, xtype=XType.point3d, default=Vec3(-1e20, -1e20, -1e20)
-        ),
+        "extmax": DXFAttr(15, xtype=XType.point3d, default=Vec3(-1e20, -1e20, -1e20)),
         "elevation": DXFAttr(146, default=0.0),
         "ucs_origin": DXFAttr(13, xtype=XType.point3d, default=NULLVEC),
         "ucs_xaxis": DXFAttr(
@@ -324,18 +344,16 @@ class DXFLayout(PlotSettings):
     DXFATTRIBS = DXFAttributes(base_class, acdb_plot_settings, acdb_layout)
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
-    ) -> "DXFNamespace":
+        self, processor: Optional[SubclassProcessor] = None
+    ) -> DXFNamespace:
         dxf = super().load_dxf_attribs(processor)
         if processor:
             processor.fast_load_dxfattribs(dxf, acdb_layout_group_codes, 2)
         return dxf
 
-    def export_entity(self, tagwriter: "TagWriter") -> None:
-        # Set correct Model Type flag
-        self.set_flag_state(
-            1024, self.dxf.name.upper() == "MODEL", "plot_layout_flags"
-        )
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
+        # Set correct model type flag
+        self.set_flag_state(1024, self.dxf.name.upper() == "MODEL", "plot_layout_flags")
         super().export_entity(tagwriter)
         tagwriter.write_tag2(SUBCLASS_MARKER, acdb_layout.name)
         self.dxf.export_dxf_attribs(
@@ -360,3 +378,17 @@ class DXFLayout(PlotSettings):
                 "base_ucs_handle",
             ],
         )
+
+    def register_resources(self, registry: xref.Registry) -> None:
+        super().register_resources(registry)
+        registry.add_handle(self.dxf.get("ucs_handle"))
+        registry.add_handle(self.dxf.get("base_ucs_handle"))
+
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
+        super().map_resources(clone, mapping)
+
+        # The content of paperspace layouts is not copied automatically and the
+        # associated BLOCK_RECORD is created and assigned in a special method.
+        mapping.map_existing_handle(self, clone, "ucs_handle", optional=True)
+        mapping.map_existing_handle(self, clone, "base_ucs_handle", optional=True)
+        mapping.map_existing_handle(self, clone, "viewport_handle", optional=True)
