@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2022 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
@@ -10,7 +10,7 @@ from typing import (
     Iterator,
     Optional,
 )
-from typing_extensions import TypeAlias
+from typing_extensions import TypeAlias, Self
 import array
 import copy
 from contextlib import contextmanager
@@ -40,6 +40,7 @@ from ezdxf.query import EntityQuery
 from .dxfentity import base_class, SubclassProcessor
 from .dxfgfx import DXFGraphic, acdb_entity
 from .factory import register_entity
+from .copy import default_copy
 
 if TYPE_CHECKING:
     from ezdxf.entities import DXFNamespace, Line, Arc, DXFEntity
@@ -102,7 +103,7 @@ class LWPolyline(DXFGraphic):
         super().__init__()
         self.lwpoints = LWPolylinePoints()
 
-    def copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
         """Copy lwpoints."""
         assert isinstance(entity, LWPolyline)
         entity.lwpoints = copy.deepcopy(self.lwpoints)
@@ -253,9 +254,7 @@ class LWPolyline(DXFGraphic):
         for x, y in self.vertices():
             yield Vec3(x, y, elevation)
 
-    def append(
-        self, point: Sequence[float], format: str = DEFAULT_FORMAT
-    ) -> None:
+    def append(self, point: Sequence[float], format: str = DEFAULT_FORMAT) -> None:
         """Append `point` to polyline, `format` specifies a user defined
         point format.
 
@@ -304,9 +303,7 @@ class LWPolyline(DXFGraphic):
             self.lwpoints.append(point, format=format)
 
     @contextmanager
-    def points(
-        self, format: str = DEFAULT_FORMAT
-    ) -> Iterator[list[Sequence[float]]]:
+    def points(self, format: str = DEFAULT_FORMAT) -> Iterator[list[Sequence[float]]]:
         """Context manager for polyline points. Returns a standard Python list
         of points, according to the format string.
 
@@ -437,24 +434,24 @@ class LWPolylinePoints(VertexArray):
     VERTEX_SIZE = 5
 
     @classmethod
-    def from_tags(cls, tags):
+    def from_tags(cls, tags: Iterable[DXFTag]) -> tuple[Self, Tags]:  # type: ignore
         """Setup point array from tags."""
 
-        def get_vertex():
+        def build_vertex(point: list[float]) -> list[float]:
             point.append(attribs.get(cls.START_WIDTH_CODE, 0))
             point.append(attribs.get(cls.END_WIDTH_CODE, 0))
             point.append(attribs.get(cls.BULGE_CODE, 0))
-            return tuple(point)
+            return point
 
         unprocessed_tags = Tags()
-        data = []
-        point = None
-        attribs = {}
+        vertices: list[Sequence[float]]= []
+        point: list[float] | None = None
+        attribs: dict[int, float]= {}
         for tag in tags:
             if tag.code in LWPOINTCODES:
                 if tag.code == 10:
                     if point is not None:
-                        data.extend(get_vertex())
+                        vertices.append(build_vertex(point))
                     # just use x- and  y-axis
                     point = list(tag.value[0:2])
                     attribs = {}
@@ -463,12 +460,10 @@ class LWPolylinePoints(VertexArray):
             else:
                 unprocessed_tags.append(tag)
         if point is not None:
-            data.extend(get_vertex())
-        return cls(data=data), unprocessed_tags
+            vertices.append(build_vertex(point))
+        return cls(data=vertices), unprocessed_tags
 
-    def append(
-        self, point: Sequence[float], format: str = DEFAULT_FORMAT
-    ) -> None:
+    def append(self, point: Sequence[float], format: str = DEFAULT_FORMAT) -> None:
         super().append(compile_array(point, format=format))
 
     def dxftags(self) -> Iterator[DXFTag]:
@@ -484,9 +479,7 @@ class LWPolylinePoints(VertexArray):
                 yield DXFTag(self.BULGE_CODE, bulge)
 
 
-def format_point(
-    point: Sequence[float], format: str = "xyseb"
-) -> Sequence[float]:
+def format_point(point: Sequence[float], format: str = "xyseb") -> Sequence[float]:
     """Reformat point components.
 
     Format codes:
