@@ -1,4 +1,4 @@
-# Copyright (c) 2018-2022, Manfred Moitzi
+# Copyright (c) 2018-2024, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import (
@@ -8,6 +8,8 @@ from typing import (
     Sequence,
     Iterator,
 )
+from typing_extensions  import Self
+
 from collections import OrderedDict, namedtuple
 import math
 
@@ -29,7 +31,7 @@ from .dxfentity import base_class, SubclassProcessor
 from .dxfobj import DXFObject
 from .dxfgfx import DXFGraphic, acdb_entity
 from .objectcollection import ObjectCollection
-
+from .copy import default_copy
 import logging
 
 if TYPE_CHECKING:
@@ -330,7 +332,7 @@ class MLine(DXFGraphic):
         """Count of MLINE vertices."""
         return len(self.vertices)
 
-    def copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
         assert isinstance(entity, MLine)
         entity.vertices = [v.copy() for v in self.vertices]
 
@@ -374,7 +376,7 @@ class MLine(DXFGraphic):
         super().register_resources(registry)
         registry.add_handle(self.dxf.style_handle)
 
-    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
         """Translate resources from self to the copied entity."""
         super().map_resources(clone, mapping)
         style = mapping.get_reference_of_copy(self.dxf.style_handle)
@@ -541,7 +543,7 @@ class MLine(DXFGraphic):
         )
         # Transform given vertices into UCS and project them into the
         # UCS-xy-plane by setting the z-axis to 0:
-        vertices = [v.replace(z=0.) for v in ucs.points_from_wcs(vertices)]
+        vertices = [v.replace(z=0.0) for v in ucs.points_from_wcs(vertices)]
         start_angle = style.dxf.start_angle
         end_angle = style.dxf.end_angle
 
@@ -623,7 +625,7 @@ class MLine(DXFGraphic):
                 return
         self.dxf.style_name = "Standard"
 
-    def transform(self, m: Matrix44) -> DXFGraphic:
+    def transform(self, m: Matrix44) -> Self:
         """Transform MLINE entity by transformation matrix `m` inplace."""
         for vertex in self.vertices:
             vertex.transform(m)
@@ -873,7 +875,7 @@ class MLineStyle(DXFObject):
         super().__init__()
         self.elements = MLineStyleElements()
 
-    def copy_data(self, entity: DXFEntity) -> None:
+    def copy_data(self, entity: Self, copy_strategy=default_copy) -> None:
         assert isinstance(entity, MLineStyle)
         entity.elements = self.elements.copy()
 
@@ -939,7 +941,7 @@ class MLineStyle(DXFObject):
         for element in self.elements:
             registry.add_linetype(element.linetype)
 
-    def map_resources(self, clone: DXFEntity, mapping: xref.ResourceMapper) -> None:
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
         """Translate resources from self to the copied entity."""
         assert isinstance(clone, MLineStyle)
         super().map_resources(clone, mapping)
@@ -960,6 +962,6 @@ class MLineStyleCollection(ObjectCollection[MLineStyle]):
 
     def create_required_entries(self) -> None:
         if "Standard" not in self:
-            entity: MLineStyle = self.new("Standard")  # type: ignore
+            entity: MLineStyle = self.new("Standard")
             entity.elements.append(0.5, 256)
             entity.elements.append(-0.5, 256)
