@@ -1,9 +1,9 @@
-# Copyright (c) 2019-2022 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import TYPE_CHECKING, Tuple, Optional
+from typing import TYPE_CHECKING, Optional
+from typing_extensions import Self
 import math
-import warnings
 
 from ezdxf.lldxf import validator
 from ezdxf.lldxf import const
@@ -18,10 +18,8 @@ from ezdxf.lldxf.attributes import (
 )
 from ezdxf.enums import (
     TextEntityAlignment,
-    MAP_STRING_ALIGN_TO_ENUM_ALIGN,
     MAP_TEXT_ENUM_TO_ALIGN_FLAGS,
     MAP_TEXT_ALIGN_FLAGS_TO_ENUM,
-    MAP_FLAGS_TO_STRING_ALIGN,
 )
 from ezdxf.math import Vec3, UVec, Matrix44, NULLVEC, Z_AXIS
 from ezdxf.math.transformtools import OCSTransform
@@ -38,7 +36,10 @@ from .dxfgfx import (
 from .factory import register_entity
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import TagWriter, DXFNamespace, Drawing
+    from ezdxf.document import Drawing
+    from ezdxf.entities import DXFNamespace, DXFEntity
+    from ezdxf.lldxf.tagwriter import AbstractTagWriter
+    from ezdxf import xref
 
 __all__ = ["Text", "acdb_text", "acdb_text_group_codes"]
 
@@ -176,7 +177,7 @@ class Text(DXFGraphic):
     UPSIDE_DOWN = MIRROR_Y
 
     def load_dxf_attribs(
-        self, processor: SubclassProcessor = None
+        self, processor: Optional[SubclassProcessor] = None
     ) -> DXFNamespace:
         """Loading interface. (internal API)"""
         dxf = super(DXFGraphic, self).load_dxf_attribs(processor)
@@ -187,13 +188,13 @@ class Text(DXFGraphic):
                 elevation_to_z_axis(dxf, ("insert", "align_point"))
         return dxf
 
-    def export_entity(self, tagwriter: TagWriter) -> None:
+    def export_entity(self, tagwriter: AbstractTagWriter) -> None:
         """Export entity specific data as DXF tags. (internal API)"""
         super().export_entity(tagwriter)
         self.export_acdb_text(tagwriter)
         self.export_acdb_text2(tagwriter)
 
-    def export_acdb_text(self, tagwriter: TagWriter) -> None:
+    def export_acdb_text(self, tagwriter: AbstractTagWriter) -> None:
         """Export TEXT data as DXF tags. (internal API)"""
         if tagwriter.dxfversion > const.DXF12:
             tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_text.name)
@@ -215,63 +216,17 @@ class Text(DXFGraphic):
             ],
         )
 
-    def export_acdb_text2(self, tagwriter: TagWriter) -> None:
+    def export_acdb_text2(self, tagwriter: AbstractTagWriter) -> None:
         """Export TEXT data as DXF tags. (internal API)"""
         if tagwriter.dxfversion > const.DXF12:
             tagwriter.write_tag2(const.SUBCLASS_MARKER, acdb_text2.name)
         self.dxf.export_dxf_attribs(tagwriter, "valign")
 
-    def set_pos(self, p1: UVec, p2: UVec = None, align: str = None) -> Text:
-        """Set text alignment and location. (deprecated)
-
-        The alignments "ALIGNED" and "FIT"
-        are special, they require a second alignment point, the text is aligned
-        on the virtual line between these two points and sits vertically at the
-        base line.
-
-        - "ALIGNED": Text is stretched or compressed
-          to fit exactly between `p1` and `p2` and the text height is also
-          adjusted to preserve height/width ratio.
-        - "FIT": Text is stretched or compressed to fit
-          exactly between `p1` and `p2` but only the text width is adjusted,
-          the text height is fixed by the :attr:`dxf.height` attribute.
-        - "MIDDLE": also a special adjustment, centered
-          text like "MIDDLE_CENTER", but vertically
-          centred at the total height of the text.
-
-        .. warning::
-
-            Will be removed in v1.0.0, use :meth:`set_placement`
-
-        Args:
-            p1: first alignment point as (x, y[, z])
-            p2: second alignment point as (x, y[, z]), required for "ALIGNED"
-                and "FIT" else ignored
-            align: new alignment as string or ``None`` to preserve the existing
-                alignment.
-
-        """
-        warnings.warn(
-            "set_pos() is deprecated, replaced by set_placement()",
-            DeprecationWarning,
-        )
-
-        if align is None:
-            align_enum = self.get_align_enum()
-        else:
-            assert isinstance(align, str)
-            try:
-                align_enum = MAP_STRING_ALIGN_TO_ENUM_ALIGN[align.upper()]
-            except KeyError:
-                raise ValueError(align)
-            self.set_align_enum(align_enum)
-        return self.set_placement(p1, p2, align_enum)
-
     def set_placement(
         self,
         p1: UVec,
-        p2: UVec = None,
-        align: TextEntityAlignment = None,
+        p2: Optional[UVec] = None,
+        align: Optional[TextEntityAlignment] = None,
     ) -> Text:
         """Set text alignment and location.
 
@@ -314,33 +269,7 @@ class Text(DXFGraphic):
         self.dxf.align_point = p2
         return self
 
-    def get_pos(self) -> Tuple[str, Vec3, Optional[Vec3]]:
-        """Returns a tuple (`align`, `p1`, `p2`), `align` is the alignment
-        method, `p1` is the alignment point, `p2` is only relevant if `align`
-        is "ALIGNED" or "FIT", otherwise it is ``None`` (deprecated).
-
-        .. warning::
-
-            Will be removed in v1.0.0, use :meth:`get_placement()`
-
-        """
-        warnings.warn(
-            "get_pos() is deprecated, replaced by get_placement()",
-            DeprecationWarning,
-        )
-
-        p1 = Vec3(self.dxf.insert)
-        # Except for "LEFT" is the "align point" the real insert point:
-        # If the required "align point" is not present use "insert"!
-        p2 = Vec3(self.dxf.get("align_point", p1))
-        align = self.get_align()
-        if align == "LEFT":
-            return align, p1, None
-        if align in ("FIT", "ALIGNED"):
-            return align, p1, p2
-        return align, p2, None
-
-    def get_placement(self) -> Tuple[TextEntityAlignment, Vec3, Optional[Vec3]]:
+    def get_placement(self) -> tuple[TextEntityAlignment, Vec3, Optional[Vec3]]:
         """Returns a tuple (`align`, `p1`, `p2`), `align` is the alignment
         enum :class:`~ezdxf.enum.TextEntityAlignment`, `p1` is the
         alignment point, `p2` is only relevant if `align` is :attr:`ALIGNED` or
@@ -358,29 +287,6 @@ class Text(DXFGraphic):
             return align, p1, p2
         return align, p2, None
 
-    def set_align(self, align: str = "LEFT") -> Text:
-        """Set the text alignment as string (deprecated)
-
-        .. warning::
-
-            Will be removed in v1.0.0, use :meth:`set_align_enum`
-
-        """
-        warnings.warn(
-            "set_align() is deprecated, replaced by set_align_enum()",
-            DeprecationWarning,
-        )
-        assert isinstance(align, str)
-        align = align.upper()
-        try:
-            align_enum = MAP_STRING_ALIGN_TO_ENUM_ALIGN[align]
-        except KeyError:
-            raise ValueError(f"invalid argument align: {align}")
-        halign, valign = MAP_TEXT_ENUM_TO_ALIGN_FLAGS[align_enum]
-        self.dxf.halign = halign
-        self.dxf.valign = valign
-        return self
-
     def set_align_enum(self, align=TextEntityAlignment.LEFT) -> Text:
         """Just for experts: Sets the text alignment without setting the
         alignment points, set adjustment points attr:`dxf.insert` and
@@ -395,25 +301,6 @@ class Text(DXFGraphic):
         self.dxf.valign = valign
         return self
 
-    def get_align(self) -> str:
-        """Returns the current text alignment as string (deprecated).
-
-        .. warning::
-
-            Will be removed in v1.0.0, use :meth:`get_align_enum`
-
-        """
-        warnings.warn(
-            "get_align() is deprecated, replaced by get_align_enum()",
-            DeprecationWarning,
-        )
-
-        halign = self.dxf.get("halign", 0)
-        valign = self.dxf.get("valign", 0)
-        if halign > 2:
-            valign = 0
-        return MAP_FLAGS_TO_STRING_ALIGN.get((halign, valign), "LEFT")
-
     def get_align_enum(self) -> TextEntityAlignment:
         """Returns the current text alignment as :class:`~ezdxf.enums.TextEntityAlignment`,
         see also :meth:`set_placement`.
@@ -476,7 +363,7 @@ class Text(DXFGraphic):
             self.post_transform(Matrix44.translate(dx, dy, dz))
         return self
 
-    def remove_dependencies(self, other: Drawing = None) -> None:
+    def remove_dependencies(self, other: Optional[Drawing] = None) -> None:
         """Remove all dependencies from actual document.
 
         (internal API)
@@ -489,6 +376,18 @@ class Text(DXFGraphic):
         if not has_style:
             self.dxf.style = "Standard"
 
+    def register_resources(self, registry: xref.Registry) -> None:
+        """Register required resources to the resource registry."""
+        super().register_resources(registry)
+        if self.dxf.hasattr("style"):
+            registry.add_text_style(self.dxf.style)
+
+    def map_resources(self, clone: Self, mapping: xref.ResourceMapper) -> None:
+        """Translate resources from self to the copied entity."""
+        super().map_resources(clone, mapping)
+        if clone.dxf.hasattr("style"):
+            clone.dxf.style = mapping.get_text_style(clone.dxf.style)
+
     def plain_text(self) -> str:
         """Returns text content without formatting codes."""
         return plain_text(self.dxf.text)
