@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2022, Manfred Moitzi
+# Copyright (c) 2019-2023, Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
 from typing import TYPE_CHECKING, Iterator, Union, Iterable, Optional
@@ -51,9 +51,6 @@ class _AbstractLayout(CreatorInterface):
         """Returns drawing entity database. (internal API)"""
         return self.doc.entitydb
 
-    def rename(self, name) -> None:
-        pass
-
     def __len__(self) -> int:
         """Returns count of entities owned by the layout."""
         return len(self.entity_space)
@@ -134,7 +131,9 @@ class BaseLayout(_AbstractLayout):
     @property
     def is_active_paperspace(self) -> bool:
         """``True`` if is active layout."""
-        return self.block_record.is_active_paperspace
+        if self.block_record.is_alive:
+            return self.block_record.is_active_paperspace
+        return False
 
     @property
     def is_any_paperspace(self) -> bool:
@@ -185,9 +184,15 @@ class BaseLayout(_AbstractLayout):
 
     def add_entity(self, entity: DXFGraphic) -> None:
         """Add an existing :class:`DXFGraphic` entity to a layout, but be sure
-        to unlink (:meth:`~BaseLayout.unlink_entity`) entity from the previous
-        owner layout. Adding entities from a different DXF drawing is not
+        to unlink (:meth:`~BaseLayout.unlink_entity`) `entity` from the previous
+        owner layout.  Adding entities from a different DXF drawing is not
         supported.
+
+        .. warning:: 
+        
+            This is a low-level tool - use it with caution and make sure you understand 
+            what you are doing! If used improperly, the DXF document may be damaged.
+
         """
         # bind virtual entities to the DXF document:
         doc = self.doc
@@ -249,6 +254,8 @@ class BaseLayout(_AbstractLayout):
             copy: if ``True`` copy entity from other document else unlink from
                 other document
 
+        Raises:
+            CopyNotSupported: copying of `entity` i not supported
         """
         foreign_doc = entity.doc
         dxftype = entity.dxftype()
@@ -289,9 +296,7 @@ class BaseLayout(_AbstractLayout):
         for entity in list(self):
             self.delete_entity(entity)
 
-    def move_to_layout(
-        self, entity: DXFGraphic, layout: BaseLayout
-    ) -> None:
+    def move_to_layout(self, entity: DXFGraphic, layout: BaseLayout) -> None:
         """Move entity to another layout.
 
         Args:
@@ -347,9 +352,7 @@ class BaseLayout(_AbstractLayout):
                 )
         return sortents_table
 
-    def set_redraw_order(
-        self, handles: Union[dict, Iterable[tuple[str, str]]]
-    ) -> None:
+    def set_redraw_order(self, handles: Union[dict, Iterable[tuple[str, str]]]) -> None:
         """If the header variable $SORTENTS `Regen` flag (bit-code value 16)
         is set, AutoCAD regenerates entities in ascending handles order.
 
