@@ -1,7 +1,7 @@
 # Copyright (c) 2019-2021, Manfred Moitzi
 # License: MIT License
 from typing import Iterable, Optional
-from ezdxf.math import Vec3
+from ezdxf.math import Vec3, UVec
 from ezdxf.lldxf import const
 from .base import BaseLayout
 from ezdxf.entities import DXFGraphic, AttDef, Block, EndBlk
@@ -74,9 +74,13 @@ class BlockLayout(BaseLayout):
 
     @property
     def base_point(self) -> Vec3:
-        """Returns the base point of the block."""
+        """Get/Set the base point of the block."""
         return Vec3(self.block.dxf.base_point)  # type: ignore
 
+    @base_point.setter
+    def base_point(self, value: UVec) -> None:
+        self.block.dxf.base_point = Vec3(value)  # type: ignore
+
     def attdefs(self) -> Iterable[AttDef]:
         """Returns iterable of all :class:`~ezdxf.entities.attrib.Attdef`
         entities.
@@ -89,7 +93,7 @@ class BlockLayout(BaseLayout):
         """
         return self.get_attdef(tag) is not None
 
-    def get_attdef(self, tag: str) -> Optional[DXFGraphic]:
+    def get_attdef(self, tag: str) -> Optional[AttDef]:
         """Returns attached :class:`~ezdxf.entities.attrib.Attdef` entity by
         `tag` name.
         """
