@@ -1,6 +1,6 @@
-# Copyright (c) 2011-2021, Manfred Moitzi
+# Copyright (c) 2011-2022, Manfred Moitzi
 # License: MIT License
-from typing import List
+from __future__ import annotations
 from dataclasses import dataclass
 
 DXF9 = "AC1004"
@@ -185,6 +185,8 @@ TABLE_NAMES_ACAD_ORDER = [
     "BLOCK_RECORD",
 ]
 
+DEFAULT_TEXT_STYLE = "Standard"
+DEFAULT_TEXT_FONT = "txt"
 APP_DATA_MARKER = 102
 SUBCLASS_MARKER = 100
 XDATA_MARKER = 1001
@@ -440,10 +442,10 @@ BOUNDARY_PATH_TEXTBOX = 8
 BOUNDARY_PATH_OUTERMOST = 16
 
 
-def boundary_path_flag_names(flags: int) -> List[str]:
+def boundary_path_flag_names(flags: int) -> list[str]:
     if flags == 0:
         return ["default"]
-    types: List[str] = []
+    types: list[str] = []
     if flags & BOUNDARY_PATH_EXTERNAL:
         types.append("external")
     if flags & BOUNDARY_PATH_POLYLINE:
@@ -568,6 +570,10 @@ ATTRIB_CONST = 2  # This is a constant attribute
 ATTRIB_VERIFY = 4  # Verification is required on input of this attribute
 ATTRIB_IS_PRESET = 8  # no prompt during insertion
 
+ATTRIB_TYPE_SINGLE_LINE = 1
+ATTRIB_TYPE_MULTI_LINE = 2
+ATTDEF_TYPE_MULTI_LINE = 4
+
 # '|' is allowed in layer name, as ltype name ...
 INVALID_NAME_CHARACTERS = '<>/\\":;?*=`'
 INVALID_LAYER_NAME_CHARACTERS = set(INVALID_NAME_CHARACTERS)
@@ -608,6 +614,7 @@ STD_SCALES = {
 }
 
 RASTER_UNITS = {
+    "none": 0,
     "mm": 1,
     "cm": 2,
     "m": 3,
@@ -617,6 +624,7 @@ RASTER_UNITS = {
     "yd": 7,
     "mi": 8,
 }
+REVERSE_RASTER_UNITS = {value: name for name, value in RASTER_UNITS.items()}
 
 MODEL_SPACE_R2000 = "*Model_Space"
 MODEL_SPACE_R12 = "$Model_Space"
