@@ -2,19 +2,22 @@
 # Home-page: http://code.google.com/p/gameobjects/
 # Author: Will McGugan
 # Download-URL: http://code.google.com/p/gameobjects/downloads/list
-# Copyright (c) 2010-2022 Manfred Moitzi
+# Copyright (c) 2011-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import Sequence, Iterable, List, Tuple, Iterator, TYPE_CHECKING
+from typing import Sequence, Iterable, Iterator, TYPE_CHECKING, Optional
 import math
+import numpy as np
+import numpy.typing as npt
+
 from math import sin, cos, tan
 from itertools import chain
 
 # The pure Python implementation can't import from ._ctypes or ezdxf.math!
-from ._vector import Vec3, X_AXIS, Y_AXIS, Z_AXIS, NULLVEC
+from ._vector import Vec3, X_AXIS, Y_AXIS, Z_AXIS, NULLVEC, Vec2
 
 if TYPE_CHECKING:
-    from ._vector import UVec
+    from ezdxf.math import UVec
 
 __all__ = ["Matrix44"]
 
@@ -23,12 +26,12 @@ __all__ = ["Matrix44"]
 # optimization is not needed
 
 
-def floats(items: Iterable) -> List[float]:
+def floats(items: Iterable) -> list[float]:
     return [float(v) for v in items]
 
 
 class Matrix44:
-    """This is a 4x4 `transformation matrix`_.
+    """An optimized 4x4 `transformation matrix`_.
 
     The utility functions for constructing transformations and transforming
     vectors and points assumes that vectors are stored as row vectors, meaning
@@ -47,15 +50,18 @@ class Matrix44:
 
     """
 
+    __slots__ = ("_matrix",)
+    _matrix: npt.NDArray[np.float64]
+
     # fmt: off
-    _identity = (
+    _identity = np.array([
         1.0, 0.0, 0.0, 0.0,
         0.0, 1.0, 0.0, 0.0,
         0.0, 0.0, 1.0, 0.0,
         0.0, 0.0, 0.0, 1.0
+        ], dtype=np.float64
     )
     # fmt: on
-    __slots__ = ("_matrix",)
 
     def __init__(self, *args):
         """
@@ -68,32 +74,30 @@ class Matrix44:
         """
         nargs = len(args)
         if nargs == 0:
-            self._matrix = floats(Matrix44._identity)
+            self._matrix = Matrix44._identity.copy()
         elif nargs == 1:
-            self._matrix = floats(args[0])
+            self._matrix = np.array(args[0], dtype=np.float64)
         elif nargs == 4:
-            self._matrix = floats(chain(*args))
+            self._matrix = np.array(list(chain(*args)), dtype=np.float64)
         else:
             raise ValueError(
                 "Invalid count of arguments (4 row vectors or one "
                 "list with 16 values)."
             )
-        if len(self._matrix) != 16:
+        if self._matrix.shape != (16,):
             raise ValueError("Invalid matrix count")
 
     def __repr__(self) -> str:
-        """Returns the representation string of the matrix:
+        """Returns the representation string of the matrix in row-major order:
         ``Matrix44((col0, col1, col2, col3), (...), (...), (...))``
         """
 
         def format_row(row):
             return "(%s)" % ", ".join(str(value) for value in row)
 
-        return "Matrix44(%s)" % ", ".join(
-            format_row(row) for row in self.rows()
-        )
+        return "Matrix44(%s)" % ", ".join(format_row(row) for row in self.rows())
 
-    def get_2d_transformation(self) -> Tuple[float, ...]:
+    def get_2d_transformation(self) -> tuple[float, ...]:
         """Returns a 2D transformation as a row-major matrix in a linear
         array (tuple).
 
@@ -103,7 +107,27 @@ class Matrix44:
         m = self._matrix
         return m[0], m[1], 0.0, m[4], m[5], 0.0, m[12], m[13], 1.0
 
-    def get_row(self, row: int) -> Tuple[float, ...]:
+    @staticmethod
+    def from_2d_transformation(components: Sequence[float]) -> Matrix44:
+        """Returns the :class:`Matrix44` class for an affine 2D (3x3) transformation
+        matrix defined by 6 float values: m11, m12, m21, m22, m31, m32.
+        """
+        if len(components) != 6:
+            raise ValueError(
+                "First 2 columns of a 3x3 matrix required: m11, m12, m21, m22, m31, m32"
+            )
+
+        m44 = Matrix44()
+        m = m44._matrix
+        m[0] = components[0]
+        m[1] = components[1]
+        m[4] = components[2]
+        m[5] = components[3]
+        m[12] = components[4]
+        m[13] = components[5]
+        return m44
+
+    def get_row(self, row: int) -> tuple[float, ...]:
         """Get row as list of four float values.
 
         Args:
@@ -112,7 +136,7 @@ class Matrix44:
         """
         if 0 <= row < 4:
             index = row * 4
-            return tuple(self._matrix[index: index + 4])
+            return tuple(self._matrix[index : index + 4])
         else:
             raise IndexError(f"invalid row index: {row}")
 
@@ -126,11 +150,11 @@ class Matrix44:
         """
         if 0 <= row < 4:
             index = row * 4
-            self._matrix[index: index + len(values)] = floats(values)
+            self._matrix[index : index + len(values)] = floats(values)
         else:
             raise IndexError(f"invalid row index: {row}")
 
-    def get_col(self, col: int) -> Tuple[float, ...]:
+    def get_col(self, col: int) -> tuple[float, ...]:
         """Returns a column as a tuple of four floats.
 
         Args:
@@ -213,7 +237,9 @@ class Matrix44:
         )
 
     @classmethod
-    def scale(cls, sx: float, sy: float = None, sz: float = None) -> Matrix44:
+    def scale(
+        cls, sx: float, sy: Optional[float] = None, sz: Optional[float] = None
+    ) -> Matrix44:
         """Returns a scaling transformation matrix. If `sy` is ``None``,
         `sy` = `sx`, and if `sz` is ``None`` `sz` = `sx`.
 
@@ -324,9 +350,7 @@ class Matrix44:
         # fmt: on
 
     @classmethod
-    def xyz_rotate(
-        cls, angle_x: float, angle_y: float, angle_z: float
-    ) -> Matrix44:
+    def xyz_rotate(cls, angle_x: float, angle_y: float, angle_z: float) -> Matrix44:
         """Returns a rotation matrix for rotation about each axis.
 
         Args:
@@ -463,7 +487,7 @@ class Matrix44:
         ))
         # fmt: on
 
-    def __setitem__(self, index: Tuple[int, int], value: float):
+    def __setitem__(self, index: tuple[int, int], value: float):
         """Set (row, column) element."""
         row, col = index
         if 0 <= row < 4 and 0 <= col < 4:
@@ -471,7 +495,7 @@ class Matrix44:
         else:
             raise IndexError(f"index out of range: {index}")
 
-    def __getitem__(self, index: Tuple[int, int]):
+    def __getitem__(self, index: tuple[int, int]):
         """Get (row, column) element."""
         row, col = index
         if 0 <= row < 4 and 0 <= col < 4:
@@ -487,9 +511,10 @@ class Matrix44:
         """Returns a new matrix as result of the matrix multiplication with
         another matrix.
         """
-        res_matrix = self.copy()
-        res_matrix.__imul__(other)
-        return res_matrix
+        m1 = self._matrix.reshape(4, 4)
+        m2 = other._matrix.reshape(4, 4)
+        result = np.matmul(m1, m2)
+        return self.__class__(np.ravel(result))
 
     # __matmul__ = __mul__ does not work!
 
@@ -497,44 +522,24 @@ class Matrix44:
         """Returns a new matrix as result of the matrix multiplication with
         another matrix.
         """
-        res_matrix = self.copy()
-        res_matrix.__imul__(other)
-        return res_matrix
+        m1 = self._matrix.reshape(4, 4)
+        m2 = other._matrix.reshape(4, 4)
+        result = np.matmul(m1, m2)
+        return self.__class__(np.ravel(result))
 
     def __imul__(self, other: Matrix44) -> Matrix44:
         """Inplace multiplication with another matrix."""
-        m1 = self._matrix
-        m2 = other._matrix
-        # fmt: off
-        self._matrix = [
-            m1[0] * m2[0] + m1[1] * m2[4] + m1[2] * m2[8] + m1[3] * m2[12],
-            m1[0] * m2[1] + m1[1] * m2[5] + m1[2] * m2[9] + m1[3] * m2[13],
-            m1[0] * m2[2] + m1[1] * m2[6] + m1[2] * m2[10] + m1[3] * m2[14],
-            m1[0] * m2[3] + m1[1] * m2[7] + m1[2] * m2[11] + m1[3] * m2[15],
-
-            m1[4] * m2[0] + m1[5] * m2[4] + m1[6] * m2[8] + m1[7] * m2[12],
-            m1[4] * m2[1] + m1[5] * m2[5] + m1[6] * m2[9] + m1[7] * m2[13],
-            m1[4] * m2[2] + m1[5] * m2[6] + m1[6] * m2[10] + m1[7] * m2[14],
-            m1[4] * m2[3] + m1[5] * m2[7] + m1[6] * m2[11] + m1[7] * m2[15],
-
-            m1[8] * m2[0] + m1[9] * m2[4] + m1[10] * m2[8] + m1[11] * m2[12],
-            m1[8] * m2[1] + m1[9] * m2[5] + m1[10] * m2[9] + m1[11] * m2[13],
-            m1[8] * m2[2] + m1[9] * m2[6] + m1[10] * m2[10] + m1[11] * m2[14],
-            m1[8] * m2[3] + m1[9] * m2[7] + m1[10] * m2[11] + m1[11] * m2[15],
-
-            m1[12] * m2[0] + m1[13] * m2[4] + m1[14] * m2[8] + m1[15] * m2[12],
-            m1[12] * m2[1] + m1[13] * m2[5] + m1[14] * m2[9] + m1[15] * m2[13],
-            m1[12] * m2[2] + m1[13] * m2[6] + m1[14] * m2[10] + m1[15] * m2[14],
-            m1[12] * m2[3] + m1[13] * m2[7] + m1[14] * m2[11] + m1[15] * m2[15]
-        ]
-        # fmt: on
+        m1 = self._matrix.reshape(4, 4)
+        m2 = other._matrix.reshape(4, 4)
+        result = np.matmul(m1, m2)
+        self._matrix = np.ravel(result)
         return self
 
-    def rows(self) -> Iterable[Tuple[float, ...]]:
+    def rows(self) -> Iterator[tuple[float, ...]]:
         """Iterate over rows as 4-tuples."""
         return (self.get_row(index) for index in (0, 1, 2, 3))
 
-    def columns(self) -> Iterable[Tuple[float, ...]]:
+    def columns(self) -> Iterator[tuple[float, ...]]:
         """Iterate over columns as 4-tuples."""
         return (self.get_col(index) for index in (0, 1, 2, 3))
 
@@ -565,7 +570,7 @@ class Matrix44:
 
     ocs_to_wcs = transform_direction
 
-    def transform_vertices(self, vectors: Iterable[UVec]) -> Iterable[Vec3]:
+    def transform_vertices(self, vectors: Iterable[UVec]) -> Iterator[Vec3]:
         """Returns an iterable of transformed vertices."""
         # fmt: off
         (
@@ -585,9 +590,65 @@ class Matrix44:
             )
             # fmt: on
 
+    def fast_2d_transform(self, points: Iterable[UVec]) -> Iterator[Vec2]:
+        """Fast transformation of 2D points. For 3D input points the z-axis will be
+        ignored.  This only works reliable if only 2D transformations have been applied
+        to the 4x4 matrix!
+
+        Profiling results - speed gains over :meth:`transform_vertices`:
+
+            - pure Python code: ~1.6x
+            - Python with C-extensions: less than 1.1x
+            - PyPy 3.8: ~4.3x
+
+        But speed isn't everything, returning the processed input points as :class:`Vec2`
+        instances is another advantage.
+
+        .. versionadded:: 1.1
+
+        """
+        m = self._matrix
+        m0 = m[0]
+        m1 = m[1]
+        m4 = m[4]
+        m5 = m[5]
+        m12 = m[12]
+        m13 = m[13]
+        for pnt in points:
+            v = Vec2(pnt)
+            x = v.x
+            y = v.y
+            yield Vec2(x * m0 + y * m4 + m12, x * m1 + y * m5 + m13)
+
+    def transform_array_inplace(self, array: np.ndarray, ndim: int) -> None:
+        """Transforms a numpy array inplace, the argument `ndim` defines the dimensions
+        to transform, this allows 2D/3D transformation on arrays with more columns
+        e.g. a polyline array which stores points as (x, y, start_width, end_width,
+        bulge) values.
+
+        .. versionadded:: 1.1
+
+        """
+        # This implementation exist only for compatibility to the Cython implementation!
+        # This version is 3.4x faster than the Cython version of Matrix44.fast_2d_transform()
+        # for larger point arrays but 10.5x slower than the Cython version of this method.
+        if ndim == 2:
+            m = np.array(self.get_2d_transformation(), dtype=np.float64)
+            m.shape = (3, 3)
+        elif ndim == 3:
+            m = np.array(self._matrix, dtype=np.float64)
+            m.shape = (4, 4)
+        else:
+            raise ValueError("ndim has to be 2 or 3")
+
+        v = np.matmul(
+            np.concatenate((array[:, :ndim], np.ones((array.shape[0], 1))), axis=1), m
+        )
+        array[:, :ndim] = v[:, :ndim].copy()
+
     def transform_directions(
         self, vectors: Iterable[UVec], normalize=False
-    ) -> Iterable[Vec3]:
+    ) -> Iterator[Vec3]:
         """Returns an iterable of transformed direction vectors without
         translation.
 
@@ -636,46 +697,12 @@ class Matrix44:
 
     def transpose(self) -> None:
         """Swaps the rows for columns inplace."""
-        # fmt: off
-        (
-            m00, m01, m02, m03,
-            m10, m11, m12, m13,
-            m20, m21, m22, m23,
-            m30, m31, m32, m33,
-        ) = self._matrix
-        self._matrix = [
-            m00, m10, m20, m30,
-            m01, m11, m21, m31,
-            m02, m12, m22, m32,
-            m03, m13, m23, m33
-        ]
-        # fmt: on
+        m = self._matrix.reshape(4, 4)
+        self._matrix = np.ravel(m.T)
 
     def determinant(self) -> float:
         """Returns determinant."""
-        # fmt: off
-        (
-            m00, m01, m02, m03,
-            m10, m11, m12, m13,
-            m20, m21, m22, m23,
-            m30, m31, m32, m33,
-        ) = self._matrix
-
-        return (
-            m00 * m11 * m22 * m33 - m00 * m11 * m23 * m32 +
-            m00 * m12 * m23 * m31 - m00 * m12 * m21 * m33 +
-            m00 * m13 * m21 * m32 - m00 * m13 * m22 * m31 -
-            m01 * m12 * m23 * m30 + m01 * m12 * m20 * m33 -
-            m01 * m13 * m20 * m32 + m01 * m13 * m22 * m30 -
-            m01 * m10 * m22 * m33 + m01 * m10 * m23 * m32 +
-            m02 * m13 * m20 * m31 - m02 * m13 * m21 * m30 +
-            m02 * m10 * m21 * m33 - m02 * m10 * m23 * m31 +
-            m02 * m11 * m23 * m30 - m02 * m11 * m20 * m33 -
-            m03 * m10 * m21 * m32 + m03 * m10 * m22 * m31 -
-            m03 * m11 * m22 * m30 + m03 * m11 * m20 * m32 -
-            m03 * m12 * m20 * m31 + m03 * m12 * m21 * m30
-        )
-        # fmt: on
+        return np.linalg.det(self._matrix.reshape(4, 4))
 
     def inverse(self) -> None:
         """Calculates the inverse of the matrix.
@@ -684,63 +711,8 @@ class Matrix44:
              ZeroDivisionError: if matrix has no inverse.
 
         """
-        det = self.determinant()
-        f = 1.0 / det  # catch ZeroDivisionError by caller
-        # fmt: off
-        (
-            m00, m01, m02, m03,
-            m10, m11, m12, m13,
-            m20, m21, m22, m23,
-            m30, m31, m32, m33,
-        ) = self._matrix
-        self._matrix = [
-            (
-                m12 * m23 * m31 - m13 * m22 * m31 + m13 * m21 * m32 -
-                m11 * m23 * m32 - m12 * m21 * m33 + m11 * m22 * m33) * f,
-            (
-                m03 * m22 * m31 - m02 * m23 * m31 - m03 * m21 * m32 +
-                m01 * m23 * m32 + m02 * m21 * m33 - m01 * m22 * m33) * f,
-            (
-                m02 * m13 * m31 - m03 * m12 * m31 + m03 * m11 * m32 -
-                m01 * m13 * m32 - m02 * m11 * m33 + m01 * m12 * m33) * f,
-            (
-                m03 * m12 * m21 - m02 * m13 * m21 - m03 * m11 * m22 +
-                m01 * m13 * m22 + m02 * m11 * m23 - m01 * m12 * m23) * f,
-            (
-                m13 * m22 * m30 - m12 * m23 * m30 - m13 * m20 * m32 +
-                m10 * m23 * m32 + m12 * m20 * m33 - m10 * m22 * m33) * f,
-            (
-                m02 * m23 * m30 - m03 * m22 * m30 + m03 * m20 * m32 -
-                m00 * m23 * m32 - m02 * m20 * m33 + m00 * m22 * m33) * f,
-            (
-                m03 * m12 * m30 - m02 * m13 * m30 - m03 * m10 * m32 +
-                m00 * m13 * m32 + m02 * m10 * m33 - m00 * m12 * m33) * f,
-            (
-                m02 * m13 * m20 - m03 * m12 * m20 + m03 * m10 * m22 -
-                m00 * m13 * m22 - m02 * m10 * m23 + m00 * m12 * m23) * f,
-            (
-                m11 * m23 * m30 - m13 * m21 * m30 + m13 * m20 * m31 -
-                m10 * m23 * m31 - m11 * m20 * m33 + m10 * m21 * m33) * f,
-            (
-                m03 * m21 * m30 - m01 * m23 * m30 - m03 * m20 * m31 +
-                m00 * m23 * m31 + m01 * m20 * m33 - m00 * m21 * m33) * f,
-            (
-                m01 * m13 * m30 - m03 * m11 * m30 + m03 * m10 * m31 -
-                m00 * m13 * m31 - m01 * m10 * m33 + m00 * m11 * m33) * f,
-            (
-                m03 * m11 * m20 - m01 * m13 * m20 - m03 * m10 * m21 +
-                m00 * m13 * m21 + m01 * m10 * m23 - m00 * m11 * m23) * f,
-            (
-                m12 * m21 * m30 - m11 * m22 * m30 - m12 * m20 * m31 +
-                m10 * m22 * m31 + m11 * m20 * m32 - m10 * m21 * m32) * f,
-            (
-                m01 * m22 * m30 - m02 * m21 * m30 + m02 * m20 * m31 -
-                m00 * m22 * m31 - m01 * m20 * m32 + m00 * m21 * m32) * f,
-            (
-                m02 * m11 * m30 - m01 * m12 * m30 - m02 * m10 * m31 +
-                m00 * m12 * m31 + m01 * m10 * m32 - m00 * m11 * m32) * f,
-            (
-                m01 * m12 * m20 - m02 * m11 * m20 + m02 * m10 * m21 -
-                m00 * m12 * m21 - m01 * m10 * m22 + m00 * m11 * m22) * f,
-        ]
-        # fmt: on
+        try:
+            inverse = np.linalg.inv(self._matrix.reshape(4, 4))
+        except np.linalg.LinAlgError:
+            raise ZeroDivisionError
+        self._matrix = np.ravel(inverse)
