@@ -1,18 +1,20 @@
-# Copyright (c) 2018-2022 Manfred Moitzi
+# Copyright (c) 2018-2024 Manfred Moitzi
 # License: MIT License
 from __future__ import annotations
-from typing import TYPE_CHECKING, Tuple, Iterable, Sequence
+from typing import TYPE_CHECKING, Iterable, Sequence, Iterator, Optional
 import math
+import numpy as np
 
 from ezdxf.math import Vec2, UVec
 from .bbox import BoundingBox2d
-from .construct2d import enclosing_angles, linspace
+from .construct2d import enclosing_angles
 from .circle import ConstructionCircle
 from .line import ConstructionRay, ConstructionLine
 from .ucs import UCS
 
 if TYPE_CHECKING:
-    from ezdxf.eztypes import BaseLayout, Arc
+    from ezdxf.entities import Arc
+    from ezdxf.layouts import BaseLayout
 
 __all__ = ["ConstructionArc", "arc_chord_length", "arc_segment_count"]
 
@@ -20,11 +22,11 @@ QUARTER_ANGLES = [0, math.pi * 0.5, math.pi, math.pi * 1.5]
 
 
 class ConstructionArc:
-    """This is a helper class to create parameters for the DXF
-    :class:`~ezdxf.entities.Arc` class.
+    """Construction tool for 2D arcs.
 
     :class:`ConstructionArc` represents a 2D arc in the xy-plane, use an
-    :class:`UCS` to place arc in 3D space, see method :meth:`add_to_layout`.
+    :class:`UCS` to place a DXF :class:`~ezdxf.entities.Arc` entity in 3D space,
+    see method :meth:`add_to_layout`.
 
     Implements the 2D transformation tools: :meth:`translate`,
     :meth:`scale_uniform` and :meth:`rotate_z`
@@ -44,7 +46,7 @@ class ConstructionArc:
         radius: float = 1.0,
         start_angle: float = 0.0,
         end_angle: float = 360.0,
-        is_counter_clockwise: bool = True,
+        is_counter_clockwise: Optional[bool] = True,
     ):
 
         self.center = Vec2(center)
@@ -95,7 +97,7 @@ class ConstructionArc:
         stop: float = self.end_angle % 360
         if stop <= start:
             stop += 360
-        for angle in linspace(start, stop, num=num, endpoint=True):
+        for angle in np.linspace(start, stop, num=num, endpoint=True):
             yield angle % 360
 
     @property
@@ -126,8 +128,6 @@ class ConstructionArc:
         max. distance from the center of an arc segment to the center of its
         chord.
 
-        .. versionadded:: 0.15
-
         """
         radius: float = abs(self.radius)
         if radius > 0:
@@ -141,7 +141,7 @@ class ConstructionArc:
                 stop += 360
             angle_span: float = math.radians(stop - start)
             count = arc_segment_count(radius, angle_span, sagitta)
-            yield from self.vertices(linspace(start, stop, count + 1))
+            yield from self.vertices(np.linspace(start, stop, count + 1))
 
     def tangents(self, a: Iterable[float]) -> Iterable[Vec2]:
         """Yields tangents on arc for angles in iterable `a` in WCS as
@@ -156,7 +156,7 @@ class ConstructionArc:
             r: float = math.radians(angle)
             yield Vec2((-math.sin(r), math.cos(r)))
 
-    def main_axis_points(self):
+    def main_axis_points(self) -> Iterator[Vec2]:
         center: Vec2 = self.center
         radius: float = self.radius
         start: float = math.radians(self.start_angle)
@@ -209,13 +209,11 @@ class ConstructionArc:
     @staticmethod
     def validate_start_and_end_point(
         start_point: UVec, end_point: UVec
-    ) -> Tuple[Vec2, Vec2]:
+    ) -> tuple[Vec2, Vec2]:
         start_point = Vec2(start_point)
         end_point = Vec2(end_point)
         if start_point == end_point:
-            raise ValueError(
-                "Start- and end point have to be different points."
-            )
+            raise ValueError("Start- and end point have to be different points.")
         return start_point, end_point
 
     @classmethod
@@ -304,7 +302,7 @@ class ConstructionArc:
         mid_point: Vec2 = _end_point.lerp(_start_point, factor=0.5)
         distance: float = _end_point.distance(_start_point)
         distance2: float = distance / 2.0
-        height: float = math.sqrt(radius ** 2 - distance2 ** 2)
+        height: float = math.sqrt(radius**2 - distance2**2)
         center: Vec2 = mid_point + (_end_point - _start_point).orthogonal(
             ccw=center_is_left
         ).normalize(height)
@@ -342,9 +340,7 @@ class ConstructionArc:
         )
         def_point = Vec2(def_point)
         if def_point == start_point or def_point == end_point:
-            raise ValueError(
-                "def_point has to be different to start- and end point"
-            )
+            raise ValueError("def_point has to be different to start- and end point")
 
         circle = ConstructionCircle.from_3p(start_point, end_point, def_point)
         center = Vec2(circle.center)
@@ -357,8 +353,8 @@ class ConstructionArc:
         )
 
     def add_to_layout(
-        self, layout: "BaseLayout", ucs: UCS = None, dxfattribs=None
-    ) -> "Arc":
+        self, layout: BaseLayout, ucs: Optional[UCS] = None, dxfattribs=None
+    ) -> Arc:
         """Add arc as DXF :class:`~ezdxf.entities.Arc` entity to a layout.
 
         Supports 3D arcs by using an :ref:`UCS`. An :class:`ConstructionArc` is
@@ -402,8 +398,6 @@ class ConstructionArc:
             2           line intersects the arc at two points
             =========== ==================================
 
-        .. versionadded:: 0.17.1
-
         """
         assert isinstance(ray, ConstructionRay)
         return [
@@ -433,8 +427,6 @@ class ConstructionArc:
             2           line intersects the arc at two points
             =========== ==================================
 
-        .. versionadded:: 0.17.1
-
         """
         assert isinstance(line, ConstructionLine)
         return [
@@ -464,8 +456,6 @@ class ConstructionArc:
             2           circle intersects the arc at two points
             =========== ==================================
 
-        .. versionadded:: 0.17.1
-
         """
         assert isinstance(circle, ConstructionCircle)
         return [
@@ -495,8 +485,6 @@ class ConstructionArc:
             2           other arc intersects the arc at two points
             =========== ==================================
 
-        .. versionadded:: 0.17.1
-
         """
         assert isinstance(other, ConstructionArc)
         return [
@@ -526,7 +514,10 @@ def arc_chord_length(radius: float, sagitta: float) -> float:
         sagitta: distance from the center of the arc to the center of its base
 
     """
-    return 2.0 * math.sqrt(2.0 * radius * sagitta - sagitta * sagitta)
+    try:
+        return 2.0 * math.sqrt(2.0 * radius * sagitta - sagitta * sagitta)
+    except ValueError:
+        return 0.0
 
 
 def arc_segment_count(radius: float, angle: float, sagitta: float) -> int:
@@ -540,6 +531,9 @@ def arc_segment_count(radius: float, angle: float, sagitta: float) -> int:
             center of its chord
 
     """
-    chord_length: float = arc_chord_length(radius, sagitta)
-    alpha: float = math.asin(chord_length / 2.0 / radius) * 2.0
-    return math.ceil(angle / alpha)
+    try:
+        chord_length: float = arc_chord_length(radius, sagitta)
+        alpha: float = math.asin(chord_length / 2.0 / radius) * 2.0
+        return math.ceil(angle / alpha)
+    except (ValueError, ZeroDivisionError):
+        return 1
