@@ -1,4 +1,4 @@
-# Copyright (c) 2012-2022, Manfred Moitzi
+# Copyright (c) 2012-2024, Manfred Moitzi
 # License: MIT License
 """
 B-Splines
@@ -22,10 +22,13 @@ from typing import (
     Iterator,
     Sequence,
     TYPE_CHECKING,
-    Union,
     Optional,
+    Any,
+    no_type_check,
 )
 import math
+import numpy as np
+
 from ezdxf.math import (
     Vec3,
     UVec,
@@ -34,13 +37,12 @@ from ezdxf.math import (
     Evaluator,
     create_t_vector,
     estimate_end_tangent_magnitude,
-    linspace,
     distance_point_line_3d,
     arc_angle_span_deg,
 )
+from .bbox import BoundingBox
 from ezdxf.math import linalg
 from ezdxf.lldxf.const import DXFValueError
-from ezdxf import PYPY
 
 if TYPE_CHECKING:
     from ezdxf.math import (
@@ -50,11 +52,6 @@ if TYPE_CHECKING:
         Bezier4P,
     )
 
-# Acceleration of banded diagonal matrix solver kicks in at:
-# N=15 for CPython on Windows and Linux
-# N=60 for pypy3 on Windows and Linux
-USE_BANDED_MATRIX_SOLVER_CPYTHON_LIMIT = 15
-USE_BANDED_MATRIX_SOLVER_PYPY_LIMIT = 60
 
 __all__ = [
     # High level functions:
@@ -550,24 +547,26 @@ def double_knots(n: int, p: int, t: Sequence[float]) -> list[float]:
     return u
 
 
-def _get_best_solver(matrix: Union[list, linalg.Matrix], degree: int):
-    """Returns best suited linear equation solver depending on matrix
-    configuration and python interpreter.
+def _get_best_solver(matrix: list | linalg.Matrix, degree: int) -> linalg.Solver:
+    """Returns best suited linear equation solver depending on matrix configuration and
+    python interpreter.
     """
-    A = matrix if isinstance(matrix, linalg.Matrix) else linalg.Matrix(matrix=matrix)
-    if PYPY:
-        limit = USE_BANDED_MATRIX_SOLVER_PYPY_LIMIT
-    else:
-        limit = USE_BANDED_MATRIX_SOLVER_CPYTHON_LIMIT
-    if A.nrows < limit:  # use default equation solver
-        return linalg.LUDecomposition(A.matrix)
+    # v1.2: added NumpySolver
+    #   Acceleration of banded diagonal matrix solver is still a thing but only for
+    #   really big matrices N > 30 in pure Python and N > 20 for C-extension np_support
+    # PyPy has no advantages when using the NumpySolver
+    if not isinstance(matrix, linalg.Matrix):
+        matrix = linalg.Matrix(matrix)
+
+    if matrix.nrows < 20:  # use default equation solver
+        return linalg.NumpySolver(matrix.matrix)
     else:
         # Theory: band parameters m1, m2 are at maximum degree-1, for
         # B-spline interpolation and approximation:
         # m1 = m2 = degree-1
         # But the speed gain is not that big and just to be sure:
-        m1, m2 = linalg.detect_banded_matrix(A, check_all=False)
-        A = linalg.compact_banded_matrix(A, m1, m2)
+        m1, m2 = linalg.detect_banded_matrix(matrix, check_all=False)
+        A = linalg.compact_banded_matrix(matrix, m1, m2)
         return linalg.BandedMatrixLU(A, m1, m2)
 
 
@@ -605,7 +604,8 @@ def unconstrained_global_bspline_interpolation(
     )
     N = Basis(knots=knots, order=degree + 1, count=len(fit_points))
     solver = _get_best_solver([N.basis_vector(t) for t in t_vector], degree)
-    control_points = solver.solve_matrix(fit_points)
+    mat_B = np.array(fit_points, dtype=np.float64)
+    control_points = solver.solve_matrix(mat_B)
     return Vec3.list(control_points.rows()), knots
 
 
@@ -769,11 +769,13 @@ def global_bspline_interpolation_first_derivatives(
         [1.0] + [0.0] * (count - 1),  # Q0
         [-1.0, +1.0] + [0.0] * (count - 2),  # D0
     ]
+    ncols = len(A[0])
     for f in (nbasis(t) for t in t_vector[1:-1]):
-        A.extend(f)  # Qi, Di
+        A.extend([row[:ncols] for row in f])  # Qi, Di
     # swapped equations!
     A.append([0.0] * (count - 2) + [-1.0, +1.0])  # Dn
     A.append([0.0] * (count - 1) + [+1.0])  # Qn
+    assert len(set(len(row) for row in A)) == 1, "inhomogeneous matrix detected"
 
     # Build right handed matrix B
     B: list[Vec3] = []
@@ -1052,7 +1054,7 @@ class BSpline:
         knots = self.knots()
         lower_bound = knots[self.order - 1]
         upper_bound = knots[self.count]
-        return linspace(lower_bound, upper_bound, segments + 1)
+        return np.linspace(lower_bound, upper_bound, segments + 1)
 
     def flattening(self, distance: float, segments: int = 4) -> Iterator[Vec3]:
         """Adaptive recursive flattening. The argument `segments` is the
@@ -1081,23 +1083,16 @@ class BSpline:
                 yield from subdiv(m, e, mid_t, end_t)
 
         evaluator = self.evaluator
-        knots: list[float] = self.knots()  # type: ignore
-        if self.is_clamped:
-            lower_bound = 0.0
-        else:
-            lower_bound = knots[self.order - 1]
-            knots = knots[: self.count + 1]
-
-        knots = list(set(knots))
-        knots.sort()
-        t = lower_bound
+        knots = np.unique(np.array(self.knots()))
+        seg_f64 = np.float64(segments)
+        t = knots[0]
         start_point = evaluator.point(t)
         yield start_point
         for t1 in knots[1:]:
-            delta = (t1 - t) / segments
+            delta = (t1 - t) / seg_f64
             while t < t1:
                 next_t = t + delta
-                if math.isclose(next_t, t1):
+                if np.isclose(next_t, t1):
                     next_t = t1
                 end_point = evaluator.point(next_t)
                 yield from subdiv(start_point, end_point, t, next_t)
@@ -1162,7 +1157,7 @@ class BSpline:
 
         """
         if self._basis.is_rational:
-            raise TypeError("Rational B-splines not supported.")
+            return self._insert_knot_rational(t)
 
         knots = list(self._basis.knots)
         cpoints = list(self._control_points)
@@ -1183,6 +1178,33 @@ class BSpline:
         knots.insert(k + 1, t)  # knot[k] <= t < knot[k+1]
         return BSpline(cpoints, self.order, knots)
 
+    def _insert_knot_rational(self, t: float) -> BSpline:
+        """Knot insertion for rational B-splines."""
+        if not self._basis.is_rational:
+            raise TypeError("Requires a rational B-splines.")
+
+        knots: list[float] = list(self._basis.knots)
+        # homogeneous point representation: (x*w, y*w, z*w, w)
+        hg_points: np.typing.NDArray = to_homogeneous_points(self)
+        p: int = self.degree
+
+        def new_point(index: int) -> np.typing.NDArray:
+            a: float = (t - knots[index]) / (knots[index + p] - knots[index])
+            return hg_points[index - 1] * (1 - a) + hg_points[index] * a
+
+        if t <= 0.0 or t >= self.max_t:
+            raise DXFValueError("Invalid position t")
+
+        k: int = self._basis.find_span(t)
+        if k < p:
+            raise DXFValueError("Invalid position t")
+        new_points: Any = hg_points.tolist()
+        new_points[k - p + 1 : k] = [new_point(i) for i in range(k - p + 1, k + 1)]
+
+        points, weights = from_homogeneous_points(new_points)
+        knots.insert(k + 1, t)  # knot[k] <= t < knot[k+1]
+        return BSpline(points, self.order, knots=knots, weights=weights)
+
     def knot_refinement(self, u: Iterable[float]) -> BSpline:
         """Insert multiple knots, without altering the shape of the curve.
         Returns a new :class:`BSpline` object.
@@ -1322,6 +1344,65 @@ class BSpline:
             params = list(subdivide_params(params))
         return params
 
+    def degree_elevation(self, t: int) -> BSpline:
+        """Returns a new :class:`BSpline` with a t-times elevated degree.
+
+        Degree elevation increases the degree of a curve without changing the shape of the
+        curve. This method implements the algorithm A5.9 of the "The NURBS Book" by
+        Piegl & Tiller.
+
+        .. versionadded:: 1.4
+
+        """
+        return degree_elevation(self, t)
+
+    def point_inversion(
+        self, point: UVec, *, epsilon=1e-8, max_iterations=100, init=8
+    ) -> float:
+        """Returns the parameter t for a point on the curve that is closest to the input
+        point.
+
+        This is an iterative search using Newton's method, so there is no guarantee
+        of success, especially for splines with many turns.
+
+        Args:
+            point(UVec): point on the curve or near the curve
+            epsilon(float): desired precision (distance input point to point on curve)
+            max_iterations(int): max iterations for Newton's method
+            init(int): number of points to calculate in the initialization phase
+
+        .. versionadded:: 1.4
+
+        """
+        return point_inversion(
+            self, Vec3(point), epsilon=epsilon, max_iterations=max_iterations, init=init
+        )
+
+    def measure(self, segments: int = 100) -> Measurement:
+        """Returns a B-spline measurement tool.
+
+        All measurements are based on the approximated curve.
+
+        Args:
+            segments: count of segments for B-spline approximation.
+
+        .. versionadded:: 1.4
+
+        """
+        return Measurement(self, segments)
+
+    def split(self, t: float) -> tuple[BSpline, BSpline]:
+        """Splits the B-spline at parameter `t` and returns two new B-splines.
+
+        Raises:
+            ValuerError: t out of range 0 < t < max_t
+
+        .. versionadded:: 1.4
+
+        """
+
+        return split_bspline(self, t)
+
 
 def subdivide_params(p: list[float]) -> Iterable[float]:
     for i in range(len(p) - 1):
@@ -1565,3 +1646,425 @@ def bspline_basis_vector(
     if math.isclose(u, knots[-1]):
         basis[-1] = 1.0
     return basis
+
+
+@no_type_check
+def degree_elevation(spline: BSpline, t: int) -> BSpline:
+    """Returns a new :class:`BSpline` with a t-times elevated degree.
+
+    Degree elevation increases the degree of a curve without changing the shape of the
+    curve. This function implements the algorithm A5.9 of the "The NURBS Book" by
+    Piegl & Tiller.
+
+    .. versionadded:: 1.4
+
+    """
+    # Naming and structure have been retained to facilitate comparison with the original
+    # algorithm during debugging.
+    t = int(t)
+    if t < 1:
+        return spline
+    p = spline.degree  # degree of spline
+    # Pw: control points
+    if spline.is_rational:
+        # homogeneous point representation (x*w, y*w, z*w, w)
+        dim = 4
+        Pw = to_homogeneous_points(spline)
+    else:
+        # non-rational splines: (x, y, z)
+        dim = 3
+        Pw = np.array(spline.control_points)
+
+    # knot vector:
+    U = np.array(spline.knots())
+
+    # n + 1: count of control points (text book definition)
+    n = len(Pw) - 1
+    m = n + p + 1
+    ph = p + t
+    ph2 = ph // 2
+
+    # control points of the elevated B-spline
+    Qw = np.zeros(shape=(len(Pw) * (2 + t), dim))  # size not known yet???
+
+    # knot vector of the elevated B-spline
+    Uh = np.zeros(m * (2 + t))  # size not known yet???
+
+    # coefficients for degree elevating the Bezier segments
+    bezalfs = np.zeros(shape=(p + t + 1, p + 1))
+
+    # Bezier control points of the current segment
+    bpts = np.zeros(shape=(p + 1, dim))
+
+    # (p+t)th-degree Bezier control points of the current segment
+    ebpts = np.zeros(shape=(p + t + 1, dim))
+
+    # leftmost control points of the next Bezier segment
+    Nextbpts = np.zeros(shape=(p - 1, dim))
+
+    # knot insertion alphas
+    alfs = np.zeros(p - 1)
+    bezalfs[0, 0] = 1.0
+    bezalfs[ph, p] = 1.0
+    binom = linalg.binomial_coefficient
+    for i in range(1, ph2 + 1):
+        inv = 1.0 / binom(ph, i)
+        mpi = min(p, i)
+        for j in range(max(0, i - t), mpi + 1):
+            bezalfs[i, j] = inv * binom(p, j) * binom(t, i - j)
+    for i in range(ph2 + 1, ph):
+        mpi = min(p, i)
+        for j in range(max(0, i - t), mpi + 1):
+            bezalfs[i, j] = bezalfs[ph - i, p - j]
+    mh = ph
+    kind = ph + 1
+    r = -1
+    a = p
+    b = p + 1
+    cind = 1
+    ua = U[0]
+    Qw[0] = Pw[0]
+
+    # for i in range(0, ph + 1):
+    #     Uh[i] = ua
+    Uh[: ph + 1] = ua
+
+    # for i in range(0, p + 1):
+    #     bpts[i] = Pw[i]
+    # initialize first Bezier segment
+    bpts[: p + 1] = Pw[: p + 1]
+
+    while b < m:  # big loop thru knot vector
+        i = b
+        while (b < m) and (math.isclose(U[b], U[b + 1])):
+            b += 1
+        mul = b - i + 1
+        mh = mh + mul + t
+        ub = U[b]
+        oldr = r
+        r = p - mul
+        # insert knot u(b) r-times
+        if oldr > 0:
+            lbz = (oldr + 2) // 2
+        else:
+            lbz = 1
+        if r > 0:
+            rbz = ph - (r + 1) // 2
+        else:
+            rbz = ph
+        if r > 0:
+            # insert knot to get Bezier segment
+            numer = ub - ua
+            for k in range(p, mul, -1):
+                alfs[k - mul - 1] = numer / (U[a + k] - ua)
+            for j in range(1, r + 1):
+                save = r - j
+                s = mul + j
+                for k in range(p, s - 1, -1):
+                    bpts[k] = alfs[k - s] * bpts[k] + (1.0 - alfs[k - s]) * bpts[k - 1]
+                Nextbpts[save] = bpts[p]
+            # end of insert knot
+        for i in range(lbz, ph + 1):
+            # degree elevate bezier
+            # only points lbz, .. ,ph are used below
+            ebpts[i] = 0.0
+            mpi = min(p, i)
+            for j in range(max(0, i - t), mpi + 1):
+                ebpts[i] = ebpts[i] + bezalfs[i, j] * bpts[j]
+            # end degree elevate bezier
+        if oldr > 1:
+            # must remove knot u=U[a] oldr times
+            first = kind - 2
+            last = kind
+            den = ub - ua
+            bet = (ub - Uh[kind - 1]) / den
+            for tr in range(1, oldr):
+                # knot removal loop
+                i = first
+                j = last
+                kj = j - kind + 1
+                while j - i > tr:
+                    # loop and compute new control points for one removal step
+                    if i < cind:
+                        alf = (ub - Uh[i]) / (ua - Uh[i])
+                        Qw[i] = alf * Qw[i] + (1.0 - alf) * Qw[i - 1]
+                    if j >= lbz:
+                        if j - tr <= kind - ph + oldr:
+                            gam = (ub - Uh[j - tr]) / den
+                            ebpts[kj] = gam * ebpts[kj] + (1.0 - gam) * ebpts[kj + 1]
+                        else:
+                            ebpts[kj] = bet * ebpts[kj] + (1.0 - bet) * ebpts[kj + 1]
+                    i += 1
+                    j -= 1
+                    kj -= 1
+                first -= 1
+                last += 1
+            # end of removing knot, u=U[a]
+        if a != p:
+            # load the knot ua
+            # for i in range(0, ph - oldr):
+            #     Uh[kind] = ua
+            i = ph - oldr
+            Uh[kind : kind + i] = ua
+            kind += i
+        for j in range(lbz, rbz + 1):
+            # load control points into Qw
+            Qw[cind] = ebpts[j]
+            cind += 1
+        if b < m:
+            # set up for next pass thru loop
+            # for j in range(0, r):
+            #     bpts[j] = Nextbpts[j]
+            bpts[:r] = Nextbpts[:r]
+            # for j in range(r, p + 1):
+            #     bpts[j] = Pw[b - p + j]
+            bpts[r : p + 1] = Pw[b - p + r : b + 1]
+            a = b
+            b += 1
+            ua = ub
+        else:  # end knot
+            # for i in range(0, ph + 1):
+            #     Uh[kind + i] = ub
+            Uh[kind : kind + ph + 1] = ub
+
+    nh = mh - ph - 1
+    count_cpts = nh + 1  # text book n+1 == count of control points
+    order = ph + 1
+
+    weights = None
+    cpoints = Qw[:count_cpts, :3]
+    if dim == 4:
+        # homogeneous point representation (x*w, y*w, z*w, w)
+        weights = Qw[:count_cpts, 3]
+        cpoints = [p / w for p, w in zip(cpoints, weights)]
+        # if weights: ... not supported for numpy arrays
+        weights = weights.tolist()
+    return BSpline(
+        cpoints, order=order, weights=weights, knots=Uh[: count_cpts + order]
+    )
+
+
+def to_homogeneous_points(spline: BSpline) -> np.typing.NDArray:
+    weights = np.array(spline.weights(), dtype=np.float64)
+    if not len(weights):
+        weights = np.ones(spline.count)
+
+    return np.array(
+        [(v.x * w, v.y * w, v.z * w, w) for v, w in zip(spline.control_points, weights)]
+    )
+
+
+def from_homogeneous_points(
+    hg_points: Iterable[Sequence[float]],
+) -> tuple[list[Vec3], list[float]]:
+    points: list[Vec3] = []
+    weights: list[float] = []
+    for point in hg_points:
+        w = float(point[3])
+        points.append(Vec3(point[:3]) / w)
+        weights.append(w)
+    return points, weights
+
+
+def point_inversion(
+    spline: BSpline, point: Vec3, *, epsilon=1e-8, max_iterations=100, init=8
+) -> float:
+    """Returns the parameter t for a point on the curve that is closest to the input
+    point.
+
+    This is an iterative search using Newton's method, so there is no guarantee
+    of success, especially for splines with many turns.
+
+    Args:
+        spline(BSpline): curve
+        point(Vec3): point on the curve or near the curve
+        epsilon(float): desired precision (distance input point to point on curve)
+        max_iterations(int): max iterations for Newton's method
+        init(int): number of points to calculate in the initialization phase
+
+    .. versionadded:: 1.4
+
+    """
+    max_t = spline.max_t
+    prev_distance = float("inf")
+    u = max_t / 2
+
+    # Initialization phase
+    t = np.linspace(0, max_t, init, endpoint=True)
+    chk_points = list(spline.points(t))
+    for u1, p in zip(t, chk_points):
+        distance = point.distance(p)
+        if distance < prev_distance:
+            u = float(u1)
+            prev_distance = distance
+
+    no_progress_counter: int = 0
+    for iteration in range(max_iterations):
+        # Evaluate the B-spline curve at the current parameter value
+        p, dpdu = spline.derivative(u, n=1)
+
+        # Calculate the difference between the current point and the target point
+        diff = p - point
+
+        # Check if the difference is within the desired epsilon
+        distance = diff.magnitude
+        if distance < epsilon:
+            break  # goal reached
+        if math.isclose(prev_distance, distance):
+            no_progress_counter += 1
+            if no_progress_counter > 2:
+                break
+        else:
+            no_progress_counter = 0
+        prev_distance = distance
+
+        # Update the parameter value using Newton's method
+        u -= diff.dot(dpdu) / dpdu.dot(dpdu)
+
+        # Clamp the parameter value within the valid range
+        if u < 0.0:
+            u = 0.0
+        elif u > max_t:
+            u = max_t
+    return u
+
+
+class Measurement:
+    """B-spline measurement tool.
+
+    All measurements are based on the approximated curve.
+
+    .. versionadded:: 1.4
+
+    """
+
+    def __init__(self, spline: BSpline, segments: int) -> None:
+        if not isinstance(spline, BSpline):
+            raise TypeError(f"BSpline instance expected, got {type(spline)}")
+        segments = int(segments)
+        if segments < 1:
+            raise ValueError(f"invalid segment count: {segments}")
+        self._spline = spline
+        self._parameters = np.linspace(0.0, spline.max_t, segments + 1, endpoint=True)
+        points = list(self._spline.points(self._parameters))
+        bbox = BoundingBox(points)
+        self.extmin = bbox.extmin
+        self.extmax = bbox.extmax
+        self._distances = self._measure(points)  # distance between points
+        self._offsets = np.cumsum(self._distances)  # distance along curve from start
+
+    @staticmethod
+    def _measure(points: list[Vec3]) -> np.typing.NDArray:
+        prev = points[0]
+        distances = np.zeros(len(points), dtype=np.float64)
+        for index, point in enumerate(points):
+            distances[index] = prev.distance(point)
+            prev = point
+        return distances
+
+    @property
+    def length(self) -> float:
+        """Returns the approximated length of the B-spline."""
+        return float(self._offsets[-1])
+
+    def distance(self, t: float) -> float:
+        """Returns the distance along the curve from the start point for then given
+        parameter t.
+        """
+        if t <= 0.0:
+            return 0.0
+        if t >= self._spline.max_t:
+            return self.length
+        params = self._parameters
+        index = np.searchsorted(params, t, side="right")
+        t1 = params[index]
+        t2 = params[index + 1]
+        distance = self._distances[index + 1] * (t - t1) / (t2 - t1)
+        return float(self._offsets[index] + distance)
+
+    def param_at(self, distance: float) -> float:
+        """Returns the parameter t for a given distance along the curve from the
+        start point.
+        """
+        index = np.searchsorted(self._offsets, distance, side="right")
+        if index <= 0:
+            return 0.0
+        params = self._parameters
+        if index >= len(params):
+            return self._spline.max_t
+        prev = index - 1
+        prev_distance = distance - self._offsets[prev]
+        t1 = params[prev]
+        t2 = params[index]
+        return float(t1 + (t2 - t1) * (prev_distance / self._distances[index]))
+
+    def divide(self, count: int) -> list[float]:
+        """Returns the interpolated B-spline parameters for dividing the curve into
+        `count` segments of equal length.
+
+        The method yields only the dividing parameters, the first (0.0) and last parameter
+        (max_t) are not included e.g. dividing a B-spline by 3 yields two parameters
+        [t1, t2] that divides the B-spline into 3 parts of equal length.
+
+        """
+        if count < 1:
+            raise ValueError(f"invalid count: {count}")
+        total_length: float = self.length
+        seg_length = total_length / count
+        distance = seg_length
+        max_t = self._spline.max_t
+        result: list[float] = []
+        while distance < total_length:
+            t = self.param_at(distance)
+            if not math.isclose(max_t, t):
+                result.append(t)
+            distance += seg_length
+        return result
+
+
+def split_bspline(spline: BSpline, t: float) -> tuple[BSpline, BSpline]:
+    """Splits a B-spline at a parameter t.
+
+    Raises:
+        ValuerError: t out of range 0 < t < max_t
+
+    .. versionadded:: 1.4
+
+    """
+    tol = 1e-12
+    if t < tol:
+        raise ValueError("t must be greater than 0")
+    if t > spline.max_t - tol:
+        raise ValueError("t must be smaller than max_t")
+    order = spline.order
+
+    # Clamp spline at parameter t
+    u = np.full(order, t)
+    spline = spline.knot_refinement(u)
+
+    knots = np.array(spline.knots(), dtype=np.float64)
+    # Determine the knot span
+    span = np.searchsorted(knots, t, side="right")
+
+    # Calculate the new knot vector
+    knots1 = knots[:span]
+    knots2 = knots[span:]
+
+    # Append the desired parameter value to the knot vector
+    knots2 = np.concatenate((u, knots2))
+
+    # Split control points
+    points = spline.control_points
+    split_index = len(knots1) - order
+    points1 = points[:split_index]
+    points2 = points[split_index:]
+    weights1: Sequence[float] = []
+    weights2: Sequence[float] = []
+    weights = spline.weights()
+    if len(weights):
+        weights1 = weights[:split_index]
+        weights2 = weights[split_index:]
+    return (
+        BSpline(points1, order, knots=knots1, weights=weights1),
+        BSpline(points2, order, knots=knots2, weights=weights2),
+    )
