@@ -193,11 +193,14 @@ class Recover:
             recover_tool.section_dict["TABLES"] = tables
         if recover_tool.dxfversion > "AC1009":
             recover_tool.recover_rootdict()
+            recover_tool.fix_broken_layout_links()
         section_dict = recover_tool.section_dict
+
+        is_r12 = recover_tool.dxfversion <= "AC1009"
         for name, entities in section_dict.items():
             if name in {"TABLES", "BLOCKS", "OBJECTS", "ENTITIES"}:
                 section_dict[name] = list(
-                    recover_tool.check_entities(entities)  # type: ignore
+                    recover_tool.check_entities(entities, is_r12)  # type: ignore
                 )
 
         return recover_tool
@@ -416,14 +419,20 @@ class Recover:
                 header.append(tag)
                 var_name = None
 
-    def check_entities(self, entities: list[Tags]) -> Iterator[Tags]:
+    def check_entities(self, entities: list[Tags], is_r12: bool) -> Iterator[Tags]:
+        subclass_markers = (100,)
         for entity in entities:
             _, dxftype = entity[0]
             if dxftype in EXCLUDE_STRUCTURE_CHECK:
                 yield entity
             else:
                 # raises DXFStructureError() for invalid entities
-                yield Tags(entity_structure_validator(entity))
+                tags = Tags(entity_structure_validator(entity))
+                if is_r12:
+                    # subclass markers (100, ...) in DXF R12 files confuses the 
+                    # ezdxf parser #1106
+                    tags.remove_tags(subclass_markers)
+                yield tags
 
     def recover_rootdict(self):
         objects = self.section_dict.get("OBJECTS")
@@ -448,6 +457,12 @@ class Recover:
                 )
             )
 
+    def fix_broken_layout_links(self):
+        """Fixes broke links (block_record_handle) between LAYOUT and BLOCK_RECORD
+        entities. See issue #997 for more information.
+        """
+        pass
+
 
 def _detect_dxf_version(header: list) -> str:
     next_is_dxf_version = False
@@ -507,9 +522,9 @@ def safe_tag_loader(
     encoding = detect_encoding(detector_stream)
 
     # Apply repair filter:
-    tags = repair.tag_reorder_layer(tags)  # type: ignore
+    tags = repair.tag_reorder_layer(tags)
     tags = repair.filter_invalid_point_codes(tags)  # type: ignore
-    tags = repair.filter_invalid_handles(tags)  # type: ignore
+    tags = repair.filter_invalid_handles(tags)
     return byte_tag_compiler(tags, encoding, messages=messages, errors=errors)
 
 
@@ -523,8 +538,8 @@ def _search_int(s: Union[str, bytes]) -> int:
     an exception. e.g. "42xyz" is a valid integer 42
 
     """
-    res = re.search(  # type: ignore
-        INT_PATTERN_S if isinstance(s, str) else INT_PATTERN_B, s
+    res = re.search(
+        INT_PATTERN_S if isinstance(s, str) else INT_PATTERN_B, s  # type: ignore
     )
     if res:
         s = res.group()
@@ -541,8 +556,8 @@ def _search_float(s: Union[str, bytes]) -> float:
     an exception. e.g. "47.11xyz" is a valid double 47.11
 
     """
-    res = re.search(  # type: ignore
-        FLOAT_PATTERN_S if isinstance(s, str) else FLOAT_PATTERN_B, s
+    res = re.search(
+        FLOAT_PATTERN_S if isinstance(s, str) else FLOAT_PATTERN_B, s  # type: ignore
     )
     if res:
         s = res.group()
