@@ -8,6 +8,7 @@ from __future__ import annotations
 from typing import Sequence, Optional
 import abc
 from ezdxf.lldxf import const
+from ezdxf import colors
 from ezdxf.entities.mtext import MText, MTextColumns
 from ezdxf.enums import (
     MTextParagraphAlignment,
@@ -158,9 +159,12 @@ def column_heights(columns: MTextColumns) -> list[Optional[float]]:
         heights = list(columns.heights)
         # last height has to be auto height = None
         heights[-1] = None
-    else:  # static, dynamic auto
-        heights = [columns.defined_height] * columns.count
-    return heights
+        return heights
+    # static, dynamic auto
+    defined_height = abs(columns.defined_height)
+    if defined_height < 1e-6:
+        return [None]
+    return [defined_height] * columns.count
 
 
 class AbstractMTextRenderer(abc.ABC):
@@ -195,7 +199,7 @@ class AbstractMTextRenderer(abc.ABC):
         ctx.aci = mtext.dxf.color
         rgb = mtext.rgb
         if rgb is not None:
-            ctx.rgb = rgb
+            ctx.rgb = colors.RGB(*rgb)
         return ctx
 
     def get_font(self, ctx: MTextContext) -> fonts.AbstractFont:
