@@ -1,21 +1,22 @@
-#  Copyright (c) 2022, Manfred Moitzi
+#  Copyright (c) 2022-2023, Manfred Moitzi
 #  License: MIT License
 from __future__ import annotations
 from typing import (
     Iterator,
     Sequence,
-    List,
-    Tuple,
-    Dict,
     TYPE_CHECKING,
     Callable,
     Any,
     Union,
+    Optional,
+    Tuple,
 )
+from typing_extensions import TypeAlias
 from collections import defaultdict
 import enum
 import math
 import dataclasses
+import random
 from ezdxf.math import (
     Vec2,
     Vec3,
@@ -45,7 +46,7 @@ class IntersectionType(enum.IntEnum):
 
 
 class HatchingError(Exception):
-    """Base exception of the :mod:`hatching` module."""
+    """Base exception class of the :mod:`hatching` module."""
 
     pass
 
@@ -72,6 +73,7 @@ class Line:
 @dataclasses.dataclass(frozen=True)
 class Intersection:
     """Represents an intersection."""
+
     type: IntersectionType = IntersectionType.NONE
     p0: Vec2 = NONE_VEC2
     p1: Vec2 = NONE_VEC2
@@ -95,6 +97,7 @@ class HatchLine:
         distance: the normal distance to the base hatch line as float
 
     """
+
     origin: Vec2
     direction: Vec2
     distance: float
@@ -137,9 +140,7 @@ class HatchLine:
             return Intersection(IntersectionType.REGULAR, a.lerp(b, factor))
         return Intersection()  # no intersection
 
-    def intersect_cubic_bezier_curve(
-        self, curve: Bezier4P
-    ) -> Sequence[Intersection]:
+    def intersect_cubic_bezier_curve(self, curve: Bezier4P) -> Sequence[Intersection]:
         """Returns 0 to 3 :class:`Intersection` points of this hatch line with
         a cubic Bèzier curve.
 
@@ -187,7 +188,7 @@ class PatternRenderer:
     def sequence_origin(self, index: float) -> Vec2:
         return self.origin + self.direction * (self.pattern_length * index)
 
-    def render(self, start: Vec2, end: Vec2) -> Iterator[Tuple[Vec2, Vec2]]:
+    def render(self, start: Vec2, end: Vec2) -> Iterator[tuple[Vec2, Vec2]]:
         """Yields the pattern lines as pairs of :class:`~ezdxf.math.Vec2`
         instances from the start- to the end point on the hatch line.
         For points the start- and end point are the same :class:`~ezdxf.math.Vec2`
@@ -237,7 +238,7 @@ class PatternRenderer:
                 e_offset,
             )
 
-    def render_full_pattern(self, index: float) -> Iterator[Tuple[Vec2, Vec2]]:
+    def render_full_pattern(self, index: float) -> Iterator[tuple[Vec2, Vec2]]:
         # fast pattern rendering
         direction = self.direction
         start_point = self.sequence_origin(index)
@@ -252,7 +253,7 @@ class PatternRenderer:
 
     def render_offset_to_offset(
         self, index: float, s_offset: float, e_offset: float
-    ) -> Iterator[Tuple[Vec2, Vec2]]:
+    ) -> Iterator[tuple[Vec2, Vec2]]:
         direction = self.direction
         origin = self.sequence_origin(index)
         start_point = origin + direction * s_offset
@@ -282,18 +283,23 @@ class HatchBaseLine:
         direction: the hatch line direction as :class:`~ezdxf.math.Vec2` instance, must not (0, 0)
         offset: the offset of the hatch line origin to the next or to the previous hatch line
         line_pattern: line pattern as sequence of floats, see also :class:`PatternRenderer`
+        min_hatch_line_distance: minimum hatch line distance to render, raises an
+            :class:`DenseHatchingLinesError` exception if the distance between hatch
+            lines is smaller than this value
 
     Raises:
         HatchLineDirectionError: hatch baseline has no direction, (0, 0) vector
         DenseHatchingLinesError: hatching lines are too narrow
 
     """
+
     def __init__(
         self,
         origin: Vec2,
         direction: Vec2,
         offset: Vec2,
-        line_pattern: List[float] = None,
+        line_pattern: Optional[list[float]] = None,
+        min_hatch_line_distance=MIN_HATCH_LINE_DISTANCE,
     ):
         self.origin = origin
         try:
@@ -302,10 +308,10 @@ class HatchBaseLine:
             raise HatchLineDirectionError("hatch baseline has no direction")
         self.offset = offset
         self.normal_distance: float = (-offset).det(self.direction - offset)
-        if abs(self.normal_distance) < MIN_HATCH_LINE_DISTANCE:
+        if abs(self.normal_distance) < min_hatch_line_distance:
             raise DenseHatchingLinesError("hatching lines are too narrow")
         self._end = self.origin + self.direction
-        self.line_pattern: List[float] = line_pattern if line_pattern else []
+        self.line_pattern: list[float] = line_pattern if line_pattern else []
 
     def __repr__(self):
         return (
@@ -316,9 +322,7 @@ class HatchBaseLine:
     def hatch_line(self, distance: float) -> HatchLine:
         """Returns the :class:`HatchLine` at the given signed `distance`."""
         factor = distance / self.normal_distance
-        return HatchLine(
-            self.origin + self.offset * factor, self.direction, distance
-        )
+        return HatchLine(self.origin + self.offset * factor, self.direction, distance)
 
     def signed_distance(self, point: Vec2) -> float:
         """Returns the signed normal distance of the given `point` from this
@@ -328,14 +332,13 @@ class HatchBaseLine:
         return (self.origin - point).det(self._end - point)
 
     def pattern_renderer(self, distance: float) -> PatternRenderer:
-        """Returns the :class:`PatternRenderer` for the given signed `distance`.
-        """
+        """Returns the :class:`PatternRenderer` for the given signed `distance`."""
         return PatternRenderer(self.hatch_line(distance), self.line_pattern)
 
 
 def hatch_line_distances(
     point_distances: Sequence[float], normal_distance: float
-) -> List[float]:
+) -> list[float]:
     """Returns all hatch line distances in the range of the given point
     distances.
     """
@@ -343,14 +346,12 @@ def hatch_line_distances(
     normal_factors = [d / normal_distance for d in point_distances]
     max_line_number = int(math.ceil(max(normal_factors)))
     min_line_number = int(math.ceil(min(normal_factors)))
-    return [
-        normal_distance * num for num in range(min_line_number, max_line_number)
-    ]
+    return [normal_distance * num for num in range(min_line_number, max_line_number)]
 
 
 def intersect_polygon(
     baseline: HatchBaseLine, polygon: Sequence[Vec2]
-) -> Iterator[Tuple[Intersection, float]]:
+) -> Iterator[tuple[Intersection, float]]:
     """Yields all intersection points of the hatch defined by the `baseline` and
     the given `polygon`.
 
@@ -395,7 +396,7 @@ def intersect_polygon(
 def hatch_polygons(
     baseline: HatchBaseLine,
     polygons: Sequence[Sequence[Vec2]],
-    terminate: Callable[[], bool] = None,
+    terminate: Optional[Callable[[], bool]] = None,
 ) -> Iterator[Line]:
     """Yields all pattern lines for all hatch lines generated by the given
     :class:`HatchBaseLine`, intersecting the given 2D polygons as :class:`Line`
@@ -431,7 +432,7 @@ def hatch_polygons(
 
 def intersect_path(
     baseline: HatchBaseLine, path: Path
-) -> Iterator[Tuple[Intersection, float]]:
+) -> Iterator[tuple[Intersection, float]]:
     """Yields all intersection points of the hatch defined by the `baseline` and
     the given single `path`.
 
@@ -467,7 +468,7 @@ def intersect_path(
                     yield ip, hatch_line_distance
 
 
-def _path_elements(path: Path) -> Union[Bezier4P, Tuple[Vec2, Vec2]]:
+def _path_elements(path: Path) -> Union[Bezier4P, tuple[Vec2, Vec2]]:
     if len(path) == 0:
         return
     start = path.start
@@ -494,7 +495,7 @@ def _path_elements(path: Path) -> Union[Bezier4P, Tuple[Vec2, Vec2]]:
 def hatch_paths(
     baseline: HatchBaseLine,
     paths: Sequence[Path],
-    terminate: Callable[[], bool] = None,
+    terminate: Optional[Callable[[], bool]] = None,
 ) -> Iterator[Line]:
     """Yields all pattern lines for all hatch lines generated by the given
     :class:`HatchBaseLine`, intersecting the given 2D :class:`~ezdxf.path.Path`
@@ -520,14 +521,16 @@ def hatch_paths(
     yield from _hatch_geometry(baseline, paths, intersect_path, terminate)
 
 
-IFuncType = Callable[[HatchBaseLine, Any], Iterator[Tuple[Intersection, float]]]
+IFuncType: TypeAlias = Callable[
+    [HatchBaseLine, Any], Iterator[Tuple[Intersection, float]]
+]
 
 
 def _hatch_geometry(
     baseline: HatchBaseLine,
     geometries: Sequence[Any],
     intersection_func: IFuncType,
-    terminate: Callable[[], bool] = None,
+    terminate: Optional[Callable[[], bool]] = None,
 ) -> Iterator[Line]:
     """Returns all pattern lines intersecting the given geometries.
 
@@ -538,7 +541,7 @@ def _hatch_geometry(
     otherwise ``False``. Can be used to implement a timeout.
 
     """
-    points: Dict[float, List[Intersection]] = defaultdict(list)
+    points: dict[float, list[Intersection]] = defaultdict(list)
     for geometry in geometries:
         if terminate and terminate():
             return
@@ -567,19 +570,14 @@ def _hatch_geometry(
             yield Line(start, end, distance)
 
 
-def _line_segments(
-    vertices: List[Intersection], distance: float
-) -> Iterator[Line]:
+def _line_segments(vertices: list[Intersection], distance: float) -> Iterator[Line]:
     if len(vertices) < 2:
         return
     vertices.sort(key=lambda p: p.p0.round(SORT_NDIGITS))
     inside = False
     prev_point = NONE_VEC2
     for ip in vertices:
-        if (
-            ip.type == IntersectionType.NONE
-            or ip.type == IntersectionType.COLLINEAR
-        ):
+        if ip.type == IntersectionType.NONE or ip.type == IntersectionType.COLLINEAR:
             continue
         # REGULAR, START, END
         point = ip.p0
@@ -595,8 +593,10 @@ def _line_segments(
 
 
 def hatch_entity(
-    polygon: DXFPolygon, filter_text_boxes=True
-) -> Iterator[Tuple[Vec3, Vec3]]:
+    polygon: DXFPolygon,
+    filter_text_boxes=True,
+    jiggle_origin: bool = True,
+) -> Iterator[tuple[Vec3, Vec3]]:
     """Yields the hatch pattern of the given HATCH or MPOLYGON entity as 3D lines.
     Each line is a pair of :class:`~ezdxf.math.Vec3` instances as start- and end
     vertex, points are represented as lines of zero length, which means the
@@ -609,6 +609,8 @@ def hatch_entity(
         polygon: :class:`~ezdxf.entities.Hatch` or :class:`~ezdxf.entities.MPolygon`
             entity
         filter_text_boxes: ignore text boxes if ``True``
+        jiggle_origin: move pattern line origins a small amount to avoid intersections
+            in corner points which causes errors in patterns
 
     """
     if polygon.pattern is None or polygon.dxf.solid_fill:
@@ -618,8 +620,9 @@ def hatch_entity(
     ocs = polygon.ocs()
     elevation = polygon.dxf.elevation.z
     paths = hatch_boundary_paths(polygon, filter_text_boxes)
+    # todo: MPOLYGON offset
     # All paths in OCS!
-    for baseline in pattern_baselines(polygon):
+    for baseline in pattern_baselines(polygon, jiggle_origin=jiggle_origin):
         for line in hatch_paths(baseline, paths):
             line_pattern = baseline.pattern_renderer(line.distance)
             for s, e in line_pattern.render(line.start, line.end):
@@ -630,9 +633,7 @@ def hatch_entity(
                 yield Vec3(s), Vec3(e)
 
 
-def hatch_boundary_paths(
-    polygon: DXFPolygon, filter_text_boxes=True
-) -> List[Path]:
+def hatch_boundary_paths(polygon: DXFPolygon, filter_text_boxes=True) -> list[Path]:
     """Returns the hatch boundary paths as :class:`ezdxf.path.Path` instances
     of HATCH and MPOLYGON entities. Ignores text boxes if argument
     `filter_text_boxes` is ``True``.
@@ -641,10 +642,7 @@ def hatch_boundary_paths(
 
     loops = []
     for boundary in polygon.paths.rendering_paths(polygon.dxf.hatch_style):
-        if (
-            filter_text_boxes
-            and boundary.path_type_flags & const.BOUNDARY_PATH_TEXTBOX
-        ):
+        if filter_text_boxes and boundary.path_type_flags & const.BOUNDARY_PATH_TEXTBOX:
             continue
         path = from_hatch_boundary_path(boundary)
         for sub_path in path.sub_paths():
@@ -654,9 +652,22 @@ def hatch_boundary_paths(
     return loops
 
 
-def pattern_baselines(polygon: DXFPolygon) -> Iterator[HatchBaseLine]:
+def _jiggle_factor():
+    # range 0.0003 .. 0.0010
+    return random.random() * 0.0007 + 0.0003
+
+
+def pattern_baselines(
+    polygon: DXFPolygon,
+    min_hatch_line_distance: float = MIN_HATCH_LINE_DISTANCE,
+    *,
+    jiggle_origin: bool = False,
+) -> Iterator[HatchBaseLine]:
     """Yields the hatch pattern baselines of HATCH and MPOLYGON entities as
-    :class:`HatchBaseLine` instances.
+    :class:`HatchBaseLine` instances.  Set `jiggle_origin` to ``True`` to move pattern
+    line origins a small amount to avoid intersections in corner points which causes
+    errors in patterns.
+
     """
     pattern = polygon.pattern
     if not pattern:
@@ -665,8 +676,25 @@ def pattern_baselines(polygon: DXFPolygon) -> Iterator[HatchBaseLine]:
     # usage!
     # The stored scale and angle is just for reconstructing the base pattern
     # when applying a new scaling or rotation.
+
+    jiggle_offset = Vec2()
+    if jiggle_origin:
+        # move origin of base pattern lines a small amount to avoid intersections with
+        # boundary corner points
+        offsets: list[float] = [line.offset.magnitude for line in pattern.lines]
+        if len(offsets):
+            # calculate the same random jiggle offset for all pattern base lines
+            mean = sum(offsets) / len(offsets)
+            x = _jiggle_factor() * mean
+            y = _jiggle_factor() * mean
+            jiggle_offset = Vec2(x, y)
+
     for line in pattern.lines:
         direction = Vec2.from_deg_angle(line.angle)
         yield HatchBaseLine(
-            line.base_point, direction, line.offset, line.dash_length_items
+            origin=line.base_point + jiggle_offset,
+            direction=direction,
+            offset=line.offset,
+            line_pattern=line.dash_length_items,
+            min_hatch_line_distance=min_hatch_line_distance,
         )
