@@ -14,12 +14,13 @@ from typing_extensions import TypeAlias
 from abc import abstractmethod
 from collections import namedtuple
 import math
+import numpy as np
+
 from ezdxf.math import (
     Vec2,
     Vec3,
     UVec,
     BSpline,
-    linspace,
     ConstructionRay,
     OCS,
     ParallelRaysError,
@@ -29,20 +30,11 @@ from ezdxf.math import (
 
 if TYPE_CHECKING:
     from ezdxf.document import Drawing
-    from ezdxf.entities import (
-        DXFGraphic,
-        Solid,
-        Trace,
-        Face3d,
-        LWPolyline,
-        Polyline
-    )
+    from ezdxf.entities import DXFGraphic, Solid, Trace, Face3d, LWPolyline, Polyline
 
 __all__ = ["TraceBuilder", "LinearTrace", "CurvedTrace"]
 
-LinearStation = namedtuple(
-    "LinearStation", ("vertex", "start_width", "end_width")
-)
+LinearStation = namedtuple("LinearStation", ("vertex", "start_width", "end_width"))
 # start_width of the next (following) segment
 # end_width of the next (following) segment
 
@@ -165,14 +157,10 @@ class LinearTrace(AbstractTrace):
         point = Vec2(point)
         stations = self._stations
 
-        if bool(stations) and stations[-1].vertex.isclose(
-            point, abs_tol=self.abs_tol
-        ):
+        if bool(stations) and stations[-1].vertex.isclose(point, abs_tol=self.abs_tol):
             # replace last station
             stations.pop()
-        stations.append(
-            LinearStation(point, float(start_width), float(end_width))
-        )
+        stations.append(LinearStation(point, float(start_width), float(end_width)))
 
     def faces(self) -> Iterable[Face]:
         """Yields all faces as 4-tuples of :class:`~ezdxf.math.Vec2` objects.
@@ -210,7 +198,8 @@ class LinearTrace(AbstractTrace):
         ) -> Vec2:
             """Intersect two rays but take parallel rays into account."""
             # check for nearly parallel rays pi/100 ~1.8 degrees
-            if ray1.direction.angle_between(ray2.direction) < 0.031415:
+            angle = abs(ray1.direction.angle_between(ray2.direction))
+            if angle < 0.031415 or abs(math.pi - angle) < 0.031415:
                 return default
             try:
                 return ray1.intersect(ray2)
@@ -233,9 +222,7 @@ class LinearTrace(AbstractTrace):
             # Start- and end vertex are never to close together, close stations
             # will be merged in method LinearTrace.add_station().
             segments.append(
-                _normal_offset_points(
-                    start_vertex, end_vertex, start_width, end_width
-                )
+                _normal_offset_points(start_vertex, end_vertex, start_width, end_width)
             )
 
         # offset rays:
@@ -255,9 +242,7 @@ class LinearTrace(AbstractTrace):
                 if is_closed:
                     # Compute first two vertices as intersection of first and
                     # last segment
-                    last_offset_ray1, last_offset_ray2 = offset_rays(
-                        len(segments) - 1
-                    )
+                    last_offset_ray1, last_offset_ray2 = offset_rays(len(segments) - 1)
                     vtx0 = intersect(last_offset_ray1, offset_ray1, up1)
                     vtx1 = intersect(last_offset_ray2, offset_ray2, down1)
 
@@ -348,12 +333,12 @@ class CurvedTrace(AbstractTrace):
         """
         curve_trace = cls()
         count = segments + 1
-        t = linspace(0, spline.max_t, count)
-        for ((point, derivative), width) in zip(
-            spline.derivatives(t, n=1), linspace(start_width, end_width, count)
+        t = np.linspace(0, spline.max_t, count)
+        for (point, derivative), width in zip(
+            spline.derivatives(t, n=1), np.linspace(start_width, end_width, count)
         ):
             normal = Vec2(derivative).orthogonal(True)
-            curve_trace._append(Vec2(point), normal, width)
+            curve_trace._append(Vec2(point), normal, width)  # type: ignore
         return curve_trace
 
     @classmethod
@@ -386,9 +371,9 @@ class CurvedTrace(AbstractTrace):
         center = Vec2(arc.center)
         for point, width in zip(
             arc.vertices(arc.angles(count)),
-            linspace(start_width, end_width, count),
+            np.linspace(start_width, end_width, count),
         ):
-            curve_trace._append(point, point - center, width)
+            curve_trace._append(point, point - center, width)  # type: ignore
         return curve_trace
 
     def _append(self, point: Vec2, normal: Vec2, width: float) -> None:
@@ -463,9 +448,7 @@ class TraceBuilder(Sequence):
         in :ref:`WCS`.
         """
         for face in self.faces():
-            yield tuple(
-                ocs.points_to_wcs(Vec3(v.x, v.y, elevation) for v in face)
-            )
+            yield tuple(ocs.points_to_wcs(Vec3(v.x, v.y, elevation) for v in face))
 
     def polygons(self) -> Iterable[Polygon]:
         """Yields for each sub-trace a single polygon as sequence of
@@ -474,17 +457,13 @@ class TraceBuilder(Sequence):
         for trace in self._traces:
             yield trace.polygon()
 
-    def polygons_wcs(
-        self, ocs: OCS, elevation: float
-    ) -> Iterable[Sequence[Vec3]]:
+    def polygons_wcs(self, ocs: OCS, elevation: float) -> Iterable[Sequence[Vec3]]:
         """Yields for each sub-trace a single polygon as sequence of
         :class:`~ezdxf.math.Vec3` objects in :ref:`WCS`.
         """
         for trace in self._traces:
             yield tuple(
-                ocs.points_to_wcs(
-                    Vec3(v.x, v.y, elevation) for v in trace.polygon()
-                )
+                ocs.points_to_wcs(Vec3(v.x, v.y, elevation) for v in trace.polygon())
             )
 
     def virtual_entities(
@@ -516,18 +495,14 @@ class TraceBuilder(Sequence):
         traces = self._traces
         if len(traces) < 2:
             return
-        if isinstance(traces[0], LinearTrace) and isinstance(
-            traces[-1], LinearTrace
-        ):
+        if isinstance(traces[0], LinearTrace) and isinstance(traces[-1], LinearTrace):
             first = cast(LinearTrace, traces.pop(0))
             last = cast(LinearTrace, traces[-1])
             for point, start_width, end_width in first:
                 last.add_station(point, start_width, end_width)
 
     @classmethod
-    def from_polyline(
-        cls, polyline: DXFGraphic, segments: int = 64
-    ) -> TraceBuilder:
+    def from_polyline(cls, polyline: DXFGraphic, segments: int = 64) -> TraceBuilder:
         """
         Create a complete trace from a LWPOLYLINE or a 2D POLYLINE entity, the
         trace consist of multiple sub-traces if :term:`bulge` values are
@@ -583,14 +558,14 @@ class TraceBuilder(Sequence):
             points.append(points[0])
 
         trace = cls()
-        store_bulge = None
-        store_start_width = None
-        store_end_width = None
-        store_point = None
+        store_bulge = 0.0
+        store_start_width = 0.0
+        store_end_width = 0.0
+        store_point: UVec | None = None
 
         linear_trace = LinearTrace()
         for point, start_width, end_width, bulge in points:
-            if store_bulge:
+            if store_bulge != 0.0:
                 center, start_angle, end_angle, radius = bulge_to_arc(
                     store_point, point, store_bulge
                 )
@@ -609,9 +584,9 @@ class TraceBuilder(Sequence):
                         ew = store_end_width
                         sw = store_start_width
                     trace.append(CurvedTrace.from_arc(arc, sw, ew, segments))
-                store_bulge = None
+                store_bulge = 0.0
 
-            if bulge != 0:  # arc from prev_point to point
+            if bulge != 0.0:  # arc from prev_point to point
                 if linear_trace.is_started:
                     linear_trace.add_station(point, start_width, end_width)
                     trace.append(linear_trace)
