@@ -1,4 +1,4 @@
-# Copyright (c) 2019-2021 Manfred Moitzi
+# Copyright (c) 2019-2024 Manfred Moitzi
 # License: MIT License
 import pytest
 import ezdxf
@@ -65,6 +65,15 @@ def test_for_valid_layer_name(entity, auditor):
     assert auditor.errors[0].code == AuditError.INVALID_LAYER_NAME
 
 
+def test_if_layer_linetype_exist():
+    doc = ezdxf.new()
+    doc.layers.add("Layer0", linetype="DoesNotExist")
+    auditor = doc.audit()
+    assert len(auditor.fixes) == 1
+    fix = auditor.fixes[0]
+    assert fix.code == AuditError.UNDEFINED_LINETYPE
+
+
 def test_for_existing_owner(entity, auditor):
     entity.dxf.owner = "FFFFFF"
     auditor.check_owner_exist(entity)
@@ -100,9 +109,7 @@ def test_block_cycle_detector_setup():
 
     auditor = Auditor(doc)
     auditor.check_block_reference_cycles()
-    assert (
-        len(auditor.errors) == 3
-    ), "one entry for each involved block: 'a', 'b', 'c'"
+    assert len(auditor.errors) == 3, "one entry for each involved block: 'a', 'b', 'c'"
     assert auditor.errors[0].code == AuditError.INVALID_BLOCK_REFERENCE_CYCLE
     assert auditor.errors[1].code == AuditError.INVALID_BLOCK_REFERENCE_CYCLE
     assert auditor.errors[2].code == AuditError.INVALID_BLOCK_REFERENCE_CYCLE
@@ -204,7 +211,8 @@ def test_remove_standalone_attrib_entities_from_blocks():
     # The model space is just a BLOCK!
     doc = ezdxf.new()
     msp = doc.modelspace()
-    msp.add_entity(Attrib())
+    # Missing tag is a different issue!
+    msp.add_entity(Attrib.new(dxfattribs={"tag": "test"}))
     auditor = doc.audit()
     assert len(list(msp)) == 0
     assert len(auditor.fixes) == 1
@@ -220,3 +228,58 @@ def test_fix_invalid_transparency():
     line.audit(auditor)
     assert line.dxf.hasattr("transparency") is False
     assert len(auditor.fixes) == 1
+
+
+def test_fix_entities_with_invalid_owner_handle():
+    doc = ezdxf.new()
+    msp = doc.modelspace()
+    line = msp.add_line((0, 0), (1, 0))
+
+    block = doc.blocks.new("TEST")
+    block.add_entity(line)
+
+    result = doc.audit()
+    fix = result.fixes[0]
+    assert fix.code == AuditError.REMOVED_ENTITY_WITH_INVALID_OWNER_HANDLE
+
+    assert len(block) == 1
+    assert len(msp) == 0
+    assert line.dxf.owner == block.dxf.handle
+
+
+def test_destroyed_modelspace():
+    """User destroyed modelspace layout?"""
+    doc = ezdxf.new()
+    msp = doc.modelspace()
+    # You can shoot yourself in the foot if you want:
+    msp.block_record.destroy()
+
+    result = doc.audit()
+    assert result.has_errors is True, "non-recoverable error expected"
+    assert (
+        any(error.code == AuditError.MODELSPACE_NOT_FOUND for error in result.errors)
+        is True
+    )
+
+
+def test_destroyed_active_paperspace():
+    """User destroyed paperspace layouts?"""
+    doc = ezdxf.new()
+    # You can shoot yourself in the foot if you want:
+    for block in doc.blocks:
+        if block.is_any_paperspace:
+            block.block_record.destroy()
+
+    result = doc.audit()
+    assert result.has_errors is True, "non-recoverable error expected"
+    assert (
+        any(
+            error.code == AuditError.ACTIVE_PAPERSPACE_LAYOUT_NOT_FOUND
+            for error in result.errors
+        )
+        is True
+    )
+
+
+if __name__ == "__main__":
+    pytest.main([__file__])
