@@ -351,21 +351,21 @@ class TestMTextContextParsing:
         t0 = list(MTextParser(r"\fArial Narrow;word"))[0]
         font_face = t0.ctx.font_face
         assert font_face.family == "Arial Narrow"
-        assert font_face.style == "normal"
-        assert font_face.weight == "normal"
+        assert font_face.style == "Regular"
+        assert font_face.weight == 400
 
     def test_font_b1_i0(self):
         t0 = list(MTextParser(r"\fArial Narrow|b1;word"))[0]
-        assert t0.ctx.font_face.weight == "bold"
+        assert t0.ctx.font_face.weight == 700
 
     def test_font_b0_i1(self):
         t0 = list(MTextParser(r"\fArial Narrow|i1;word"))[0]
-        assert t0.ctx.font_face.style == "italic"
+        assert t0.ctx.font_face.style == "Italic"
 
     def test_font_b1_i1(self):
         t0 = list(MTextParser(r"\fArial Narrow|i1|b1;word"))[0]
-        assert t0.ctx.font_face.style == "italic"
-        assert t0.ctx.font_face.weight == "bold"
+        assert t0.ctx.font_face.style == "Italic"
+        assert t0.ctx.font_face.weight == 700
 
     def test_font_uppercase_command(self):
         t0 = list(MTextParser(r"\FArial Narrow;word"))[0]
@@ -681,9 +681,7 @@ class TestMTextParagraphProperties:
         )
 
     def test_reset_arguments(self):
-        t0, t1 = list(
-            MTextParser(r"\pi1,l2,r3,qc,t1,2,3;word\pi*,l*,r*,q*,t;word")
-        )
+        t0, t1 = list(MTextParser(r"\pi1,l2,r3,qc,t1,2,3;word\pi*,l*,r*,q*,t;word"))
         assert t0.ctx.paragraph == (
             1,
             2,
@@ -698,6 +696,52 @@ class TestMTextParagraphProperties:
         t = tokens[0]
         assert t.ctx.paragraph.tab_stops == tuple()
 
+    def test_unknown_escape_sequence(self):
+        t0, t1 = list(MTextParser(r"word\Yword"))
+        assert t1.type == TokenType.WORD
+        assert t1.data == r"\Yword"
+
+
+class TestMTextParserYieldsPropertyChangeCommands:
+    def test_change_height(self):
+        t0, t1, t2 = list(MTextParser(r"word\H150;word", yield_property_commands=True))
+        assert t1.type == TokenType.PROPERTIES_CHANGED
+        assert t1.ctx.cap_height == 150
+        assert t1.data == r"\H150;"
+
+    def test_change_height_without_semicolon(self):
+        t0, t1, t2 = list(MTextParser(r"word\H150word", yield_property_commands=True))
+        assert t1.type == TokenType.PROPERTIES_CHANGED
+        assert t1.ctx.cap_height == 150
+        assert t1.data == r"\H150"
+
+    def test_change_char_tracking(self):
+        t0, t1, t2 = list(MTextParser(r"word\T150;word", yield_property_commands=True))
+        assert t1.type == TokenType.PROPERTIES_CHANGED
+        assert t1.ctx.char_tracking_factor == 150
+        assert t1.data == r"\T150;"
+
+    def test_change_paragraph(self):
+        t0, t1 = list(
+            MTextParser(r"\pxt4,c5,r6,7.5;word", yield_property_commands=True)
+        )
+        assert t0.type == TokenType.PROPERTIES_CHANGED
+        assert t0.data == r"\pxt4,c5,r6,7.5;"
+
+    def test_one_char_commands(self):
+        t0, t1, t2 = list(
+            MTextParser(r"\Lword\l", yield_property_commands=True)
+        )
+        assert t0.type == TokenType.PROPERTIES_CHANGED
+        assert t0.data == r"\L"
+        assert t2.type == TokenType.PROPERTIES_CHANGED
+        assert t2.data == r"\l"
+
+    def test_unknown_escape_sequence(self):
+        t0, t1 = list(MTextParser(r"word\Yword", yield_property_commands=True))
+        assert t1.type == TokenType.WORD
+        assert t1.data == r"\Yword"
+
 
 if __name__ == "__main__":
     pytest.main([__file__])
