@@ -1,8 +1,11 @@
-# Copyright (c) 2012-2021 Manfred Moitzi
+# Copyright (c) 2012-2024 Manfred Moitzi
 # License: MIT License
+import math
+
 import pytest
+import numpy as np
 from ezdxf.math import Vec3, BSpline, close_vectors
-from ezdxf.math.bspline import normalize_knots, subdivide_params, linspace
+from ezdxf.math.bspline import normalize_knots, subdivide_params
 
 DEFPOINTS = [
     (0.0, 0.0, 0.0),
@@ -12,7 +15,7 @@ DEFPOINTS = [
     (50.0, 0.0, 30.0),
 ]
 
-PARAMS = list(linspace(0, 1, 21))
+PARAMS = list(np.linspace(0, 1, 21))
 
 
 def test_is_clamped(weired_spline1):
@@ -88,23 +91,111 @@ def test_normalize_knots_if_needed():
     assert k[0] == 0.0
 
 
-def test_bspline_insert_knot():
-    bspline = BSpline(
-        [
-            (0, 0),
-            (10, 20),
-            (30, 10),
-            (40, 10),
-            (50, 0),
-            (60, 20),
-            (70, 50),
-            (80, 70),
-        ]
-    )
-    t = bspline.max_t / 2
-    assert len(bspline.control_points) == 8
-    bspline2 = bspline.insert_knot(t)
-    assert len(bspline2.control_points) == 9
+class TestInsertKnotNonRational:
+    @pytest.fixture
+    def spline(self):
+        s = BSpline(
+            [
+                (0, 0),
+                (10, 20),
+                (30, 10),
+                (40, 10),
+                (50, 0),
+                (60, 20),
+                (70, 50),
+                (80, 70),
+            ]
+        )
+        t = s.max_t / 2
+        return s.insert_knot(t)
+
+    def test_knots(self, spline):
+        assert (
+            all(
+                math.isclose(k, e)
+                for k, e in zip(
+                    spline.knots(),
+                    (0.0, 0.0, 0.0, 0.0, 0.2, 0.4, 0.5, 0.6, 0.8, 1.0, 1.0, 1.0, 1.0),
+                )
+            )
+            is True
+        )
+
+    def test_control_points(self, spline):
+        assert (
+            all(
+                p.isclose(e)
+                for p, e in zip(
+                    spline.control_points,
+                    (
+                        (0.0, 0.0, 0.0),
+                        (10.0, 20.0, 0.0),
+                        (30.0, 10.0, 0.0),
+                        (38.333333333333336, 10.0, 0.0),
+                        (45.0, 5.000000000000001, 0.0),
+                        (51.66666666666667, 3.3333333333333326, 0.0),
+                        (60.0, 20.0, 0.0),
+                        (70.0, 50.0, 0.0),
+                        (80.0, 70.0, 0.0),
+                    ),
+                )
+            )
+            is True
+        )
+
+
+class TestInsertKnotRational:
+    @pytest.fixture
+    def spline(self):
+        s = BSpline(
+            [(0, 0), (0, 1), (1, 1)],
+            weights=[1, 1 / math.sqrt(2), 1],
+            order=3,
+        )
+
+        t = s.max_t / 2
+        return s.insert_knot(t)
+
+    def test_knots(self, spline):
+        assert (
+            all(
+                math.isclose(k, e)
+                for k, e in zip(
+                    spline.knots(),
+                    (0.0, 0.0, 0.0, 0.5, 1.0, 1.0, 1.0),
+                )
+            )
+            is True
+        )
+
+    def test_control_points(self, spline):
+        assert (
+            all(
+                p.isclose(e)
+                for p, e in zip(
+                    spline.control_points,
+                    (
+                        (0.0, 0.0, 0.0),
+                        (0.0, 0.41421356237309503, 0.0),
+                        (0.585786437626905, 1.0, 0.0),
+                        (1.0, 1.0, 0.0),
+                    ),
+                )
+            )
+            is True
+        )
+
+    def test_weights(self, spline):
+        assert (
+            all(
+                math.isclose(w, e)
+                for w, e in zip(
+                    spline.weights(),
+                    (1.0, 0.8535533905932737, 0.8535533905932737, 1.0),
+                )
+            )
+            is True
+        )
 
 
 def test_transform_interface():
@@ -333,9 +424,7 @@ def test_flattening():
     bspline = BSpline.from_fit_points(fitpoints)
     assert all(
         a.isclose(b)
-        for a, b in zip(
-            bspline.flattening(0.01, segments=4), EXPECTED_FLATTENING
-        )
+        for a, b in zip(bspline.flattening(0.01, segments=4), EXPECTED_FLATTENING)
     )
 
 
@@ -529,9 +618,7 @@ def test_bspline_point_calculation_to_pre_calculated_results(order, results):
 
 def test_bspline_derivative_calculation_to_pre_calculated_results():
     spline = BSpline(DEFPOINTS, order=4)
-    for points, expected in zip(
-        spline.derivatives(PARAMS, n=2), DERIVATIVES_ORDER_4
-    ):
+    for points, expected in zip(spline.derivatives(PARAMS, n=2), DERIVATIVES_ORDER_4):
         for p, e in zip(points, expected):
             assert p.isclose(e)
 
@@ -545,3 +632,53 @@ def test_bspline_point_calculation_against_derivative_calculation():
     curve_points = [p[0] for p in spline.derivatives(PARAMS, n=1)]
     for p, expected in zip(curve_points, spline.points(PARAMS)):
         assert p.isclose(expected)
+
+
+def test_bezier_decomposition_issue_1261():
+    bspline = BSpline(
+        (
+            Vec3(138.7341842517817, 140.6329647855705, 3.9968e-12),
+            Vec3(137.8688179331311, 141.9813616807911, 4.4658e-12),
+            Vec3(133.4979383171521, 146.8141917003327, 4.7002e-12),
+            Vec3(117.801399951808, 150.5582827051759, 3.6238e-12),
+            Vec3(104.4582484561979, 147.3755444269566, 4.0679e-12),
+            Vec3(99.74073200372732, 144.6589073782108, 3.233e-12),
+        ),
+        knots=(
+            0.0,
+            0.0,
+            0.0,
+            0.0,
+            0.149930310426686,
+            0.574965155213343,
+            1.0,
+            1.0,
+            1.0,
+            1.0,
+        ),
+    )
+    result = list(bspline.bezier_decomposition())
+    expected = [
+        [
+            Vec3(138.7341842517817, 140.6329647855705, 3.9968e-12),
+            Vec3(137.8688179331311, 141.9813616807911, 4.4658e-12),
+            Vec3(136.7290491458935, 143.24159058075014, 4.526923121019351e-12),
+            Vec3(135.27281224889438, 144.31957761671416, 4.530024100563068e-12),
+        ],
+        [
+            Vec3(135.27281224889438, 144.31957761671416, 4.530024100563068e-12),
+            Vec3(131.14455144741166, 147.3755444269546, 4.538815013856715e-12),
+            Vec3(124.47297569960983, 148.96691356606527, 4.081307506928357e-12),
+            Vec3(117.80139995180639, 148.96691356606576, 3.963578753464179e-12),
+        ],
+        [
+            Vec3(117.80139995180639, 148.96691356606576, 3.963578753464179e-12),
+            Vec3(111.12982420400294, 148.96691356606624, 3.84585e-12),
+            Vec3(104.4582484561979, 147.3755444269566, 4.0679e-12),
+            Vec3(99.74073200372732, 144.6589073782108, 3.233e-12),
+        ],
+    ]
+    assert len(result) == len(expected)
+    for bez_res, bez_exp in zip(result, expected):
+        for vec3_res, vec3_exp in zip(bez_res, bez_exp):
+            assert vec3_res.isclose(vec3_exp)
