@@ -3,7 +3,7 @@ import pytest
 import ezdxf
 import math
 from math import isclose
-from ezdxf.math import Vec3, global_bspline_interpolation, close_vectors
+from ezdxf.math import Vec3, global_bspline_interpolation, close_vectors, cad_fit_point_interpolation
 from ezdxf.math.parametrize import (
     uniform_t_vector,
     distance_t_vector,
@@ -21,7 +21,6 @@ from ezdxf.math.bspline import (
     natural_knots_unconstrained,
     double_knots,
     create_t_vector,
-    normalize_knots,
 )
 
 POINTS1 = Vec3.list([(1, 1), (2, 4), (4, 1), (7, 6)])
@@ -46,7 +45,7 @@ def test_chord_length_t_array(fit_points):
     t_vector = list(distance_t_vector(fit_points))
     assert len(t_vector) == len(fit_points)
     assert t_vector[0] == 0.0
-    assert t_vector[-1] == 1.0
+    assert t_vector[-1] == pytest.approx(1.0)
     for t1, t2 in zip(t_vector, t_vector[1:]):
         assert t1 <= t2
 
@@ -55,7 +54,7 @@ def test_centripetal_length_t_array(fit_points):
     t_vector = list(centripetal_t_vector(fit_points))
     assert len(t_vector) == len(fit_points)
     assert t_vector[0] == 0.0
-    assert t_vector[-1] == 1.0
+    assert t_vector[-1] == pytest.approx(1.0)
     for t1, t2 in zip(t_vector, t_vector[1:]):
         assert t1 <= t2
 
@@ -80,7 +79,7 @@ def test_arc_length_t_array(fit_points):
     t_vector = list(arc_t_vector(fit_points))
     assert len(t_vector) == len(fit_points)
     assert t_vector[0] == 0.0
-    assert t_vector[-1] == 1.0
+    assert t_vector[-1] == pytest.approx(1.0)
     for t1, t2 in zip(t_vector, t_vector[1:]):
         assert t1 <= t2
 
@@ -198,13 +197,18 @@ def test_bspline_interpolation(fit_points):
 
     t_array = list(create_t_vector(fit_points, "chord"))
     assert t_array[0] == 0.0
-    assert t_array[-1] == 1.0
+    assert t_array[-1] == pytest.approx(1.0)
     assert len(t_array) == len(fit_points)
 
     t_points = [spline.point(t) for t in t_array]
     assert close_vectors(t_points, fit_points)
 
 
+@pytest.mark.parametrize("method", ["distance", "centripetal", "arc"])
+def test_create_t_vectors_for_identical_points(method):
+    assert len(list(create_t_vector(Vec3.list([(0, 0), (0, 0)]), method))) == 0
+
+
 def test_bspline_interpolation_first_derivatives(fit_points):
     tangents = estimate_tangents(fit_points)
     spline = global_bspline_interpolation(
@@ -277,3 +281,32 @@ def test_check_values():
     for p1, p2 in zip(result, expected):
         assert isclose(p1[0], p2[0], abs_tol=1e-6)
         assert isclose(p1[1], p2[1], abs_tol=1e-6)
+
+
+def test_cad_fit_point_interpolation_for_2_points():
+    points = Vec3.list([(0, 0), (0, 10)])
+    control_points, knots = cad_fit_point_interpolation(points)
+    assert control_points[0].isclose(points[0])
+    assert control_points[-1].isclose(points[-1])
+    assert len(control_points) == 4
+    assert len(knots) == required_knot_values(4, 4) == 8
+
+
+def test_cad_fit_point_interpolation_for_5_points():
+    points = Vec3.list([(0, 0), (0, 10), (10, 10), (20, 10), (20, 0)])
+    control_points, knots = cad_fit_point_interpolation(points)
+    assert len(control_points) == 7
+    assert len(knots) == required_knot_values(7, 4) == 11
+
+    # Checked visually by BricsCAD 2022 and TrueView 2022:
+    # See function check_visually_fit_points_to_cad_cv() in script
+    # exploration/spline_end_tangent_estimation.py
+    assert control_points[0].isclose(points[0])
+    assert control_points[1].isclose(Vec3(-0.8333333333333334, 4.285714285714286, -0.0))
+    assert control_points[2].isclose(Vec3(-2.5, 12.857142857142858, 0.0))
+    assert control_points[3].isclose(Vec3(10.0, 8.571428571428573, 0.0))
+    assert control_points[4].isclose(Vec3(22.5, 12.857142857142858, 0.0))
+    assert control_points[5].isclose(Vec3(20.833333333333332, 4.2857142857142865, -0.0))
+    assert control_points[6].isclose(points[-1])
+
+
