@@ -10,6 +10,7 @@ from ezdxf.math import (
     ConstructionLine,
     ConstructionCircle,
     Vec2,
+    UVec,
 )
 
 HALF_PI = math.pi / 2.0
@@ -152,7 +153,7 @@ class TestCircleInterectRay:
         assert p2.isclose((10, 13), abs_tol=1e-5)
 
 
-def test_cicles_do_not_intersect():
+def test_circles_do_not_intersect():
     M1 = (30, 30)
     M2 = (40, 40)
     M3 = (30.3, 30.3)
@@ -187,7 +188,7 @@ def test_two_circles_touching_at_one_point(center, point):
     circle2 = ConstructionCircle(center, 1.5)
     points = circle1.intersect_circle(circle2)
     assert len(points) == 1
-    return points[0].isclose(point, abs_tol=1e-9)
+    assert points[0].isclose(point, abs_tol=1e-9) is True
 
 
 def test_intersect_circle_intersect():
@@ -199,58 +200,65 @@ def test_intersect_circle_intersect():
         assert len(points) == 2
         a, b = points
 
-        result1 = a.isclose(p1, abs_tol=abs_tol) and b.isclose(
-            p2, abs_tol=abs_tol
-        )
-        result2 = a.isclose(p2, abs_tol=abs_tol) and b.isclose(
-            p1, abs_tol=abs_tol
-        )
+        result1 = a.isclose(p1, abs_tol=abs_tol) and b.isclose(p2, abs_tol=abs_tol)
+        result2 = a.isclose(p2, abs_tol=abs_tol) and b.isclose(p1, abs_tol=abs_tol)
         return result1 or result2
 
     circle1 = ConstructionCircle((40, 20), 5)
     assert (
-        check_intersection((46.0, 20.0), (44.8958, 21.0153), (44.8958, 18.9847))
-        is True
+        check_intersection((46.0, 20.0), (44.8958, 21.0153), (44.8958, 18.9847)) is True
     )
     assert (
-        check_intersection((44.0, 20.0), (44.8438, 21.2402), (44.8438, 18.7598))
-        is True
+        check_intersection((44.0, 20.0), (44.8438, 21.2402), (44.8438, 18.7598)) is True
     )
     assert (
-        check_intersection((40.0, 26.0), (38.9847, 24.8958), (41.0153, 24.8958))
-        is True
+        check_intersection((40.0, 26.0), (38.9847, 24.8958), (41.0153, 24.8958)) is True
     )
     assert (
-        check_intersection((40.0, 24.0), (38.7598, 24.8438), (41.2402, 24.8438))
-        is True
+        check_intersection((40.0, 24.0), (38.7598, 24.8438), (41.2402, 24.8438)) is True
     )
     assert (
-        check_intersection((34.0, 20.0), (35.1042, 18.9847), (35.1042, 21.0153))
-        is True
+        check_intersection((34.0, 20.0), (35.1042, 18.9847), (35.1042, 21.0153)) is True
     )
     # assert check_intersection( (36.,20.),  (35.1563, 18.7598),  (35.1563, 21.2402)))
     assert (
-        check_intersection((40.0, 14.0), (38.9847, 15.1042), (41.0153, 15.1042))
+        check_intersection((40.0, 14.0), (38.9847, 15.1042), (41.0153, 15.1042)) is True
+    )
+    assert (
+        check_intersection((40.0, 14.0), (38.9847, 15.1042), (41.0153, 15.1042)) is True
+    )
+    assert (
+        check_intersection((36.8824, 17.4939), (35.4478, 17.9319), (37.0018, 15.9987))
         is True
     )
     assert (
-        check_intersection((40.0, 14.0), (38.9847, 15.1042), (41.0153, 15.1042))
-        is True
-    )
-    assert (
-        check_intersection(
-            (36.8824, 17.4939), (35.4478, 17.9319), (37.0018, 15.9987)
-        )
-        is True
-    )
-    assert (
-        check_intersection(
-            (35.3236, 16.2408), (35.5481, 17.7239), (36.8203, 16.1413)
-        )
+        check_intersection((35.3236, 16.2408), (35.5481, 17.7239), (36.8203, 16.1413))
         is True
     )
 
 
+@pytest.mark.parametrize(
+    "center, expected",
+    [
+        [(1, 0), (-1, 0)],  # right of inner circle
+        [(-1, 0), (1, 0)],  # left of inner circle
+        [(0, 1), (0, -1)],  # above of inner circle
+        [(0, -1), (0, 1)],  # below of inner circle
+    ],
+    ids=[
+        "outer circle right of inner circle",
+        "outer circle left of inner circle",
+        "outer circle above of inner circle",
+        "outer circle below of inner circle",
+    ],
+)
+def test_inner_circle_touches_outer_circle_issue_982(center: UVec, expected: UVec):
+    inner_circle = ConstructionCircle((0, 0), 1)
+    outer_circle = ConstructionCircle(center, 2)
+    pnt = inner_circle.intersect_circle(outer_circle)[0]
+    assert pnt.isclose(expected)
+
+
 def test_vertices():
     circle = ConstructionCircle((0, 0), 1.0)
     vertices = list(circle.vertices([0, math.pi * 0.5, math.pi, math.pi * 1.5]))
