@@ -12,28 +12,28 @@ def test_filled_solid_arrow():
 
 
 def test_arrow_name():
-    assert ARROWS.arrow_name('_CLOSEDFILLED') == ''
-    assert ARROWS.arrow_name('') == ''
-    assert ARROWS.arrow_name('_DOTSMALL') == 'DOTSMALL'
-    assert ARROWS.arrow_name('_boxBlank') == 'BOXBLANK'
-    assert ARROWS.arrow_name('EZ_ARROW') == 'EZ_ARROW'
-    assert ARROWS.arrow_name('abcdef') == 'abcdef'
+    assert ARROWS.arrow_name("_CLOSEDFILLED") == ""
+    assert ARROWS.arrow_name("") == ""
+    assert ARROWS.arrow_name("_DOTSMALL") == "DOTSMALL"
+    assert ARROWS.arrow_name("_boxBlank") == "BOXBLANK"
+    assert ARROWS.arrow_name("EZ_ARROW") == "EZ_ARROW"
+    assert ARROWS.arrow_name("abcdef") == "abcdef"
 
 
 def test_closed_arrow_doc_r12():
-    doc = ezdxf.new(dxfversion='R12', setup=True)
+    doc = ezdxf.new(dxfversion="R12", setup=True)
     blocks = doc.blocks
     name = ARROWS.create_block(blocks, ARROWS.closed)
     arrow_entities = list(blocks.get(name))
-    assert arrow_entities[0].dxftype() == 'POLYLINE'
+    assert arrow_entities[0].dxftype() == "POLYLINE"
 
 
 def test_closed_arrow_doc_r2000():
-    doc = ezdxf.new(dxfversion='R2000', setup=True)
+    doc = ezdxf.new(dxfversion="R2000", setup=True)
     blocks = doc.blocks
     name = ARROWS.create_block(blocks, ARROWS.closed)
     arrow_entities = list(blocks.get(name))
-    assert arrow_entities[0].dxftype() == 'LWPOLYLINE'
+    assert arrow_entities[0].dxftype() == "LWPOLYLINE"
 
 
 def test_render_arrow():
